searchData={"content_type":"text/markdown","items":[{"doc":"The ArangoDB adapter for Ecto.\n\nArangoXEcto is provides the following functionality:\n\n  * Full Ecto compatability\n\n  * ArangoDB anonymous graphs\n\n  * Geographic data\n\n  * ArangoSearch (views) and Analyzers\n\nThe functions in this module are the base functions for interacting with ArangoDB that aren't through Ecto.\nAll the Ecto functionality hindges on the functions in this module. For example, queries throuh \nEcto will use on the `aql_query/4` function in this module.","ref":"ArangoXEcto.html","title":"ArangoXEcto","type":"module"},{"doc":"To use the ArangoDB with Ecto you will need to use the `ArangoXEcto.Adapter` module in your repo.\n\n    defmodule MyApp.Repo do\n      use Ecto.Repo,\n        otp_app: :my_app,\n        adapter: ArangoXEcto.Adapter\n    end\n\nYou will also have to set the configuration as below. You can set either static or dynamic mode,\nthe default is static mode and the option can be omitted. Find out more in the next section.\n\n    config :my_app, MyApp.Repo,\n      database: \"my_db\",\n      endpoints: \"http://1.2.3.4:8529\",\n      username: \"my_user,\n      password: \"my_password\",\n      static: true","ref":"ArangoXEcto.html#module-ecto-compatibility","title":"Ecto compatibility - ArangoXEcto","type":"module"},{"doc":"In addition to default config variables, the `static` boolean config option controls the use of\nmigrations. By default the value is `true` and will act as other Ecto adapters whereby migrations\nare needed to create the collections. Any collections that do not exist on insert or query will \nresult in an error being raised.\n\nIf you set it to `false`, then you will operate in dynamic mode, therefore collections that don't\nexist will be created on insert and on query no error will be raised.\n\nWhether static (default) or dynamic is chosen depends on the database design of the project. \nFor a production setup where lots of control is required, it is recommended to have `static` set \nto `true`, hence this is the default. Sometimes in dev it can be easier to work in dynamic mode\nfor smaller projects, however this can cause database inconsistencies in larger projects.","ref":"ArangoXEcto.html#module-static-or-dynamic-mode","title":"Static or Dynamic Mode - ArangoXEcto","type":"module"},{"doc":"Instead of using the `Ecto.Schema` module the `ArangoXEcto.Schema` module needs to be used. This\nis to enable the use of a `binary_id` that ArangoDB requires.\n\n    defmodule MyApp.Accounts.User do\n      use ArangoXEcto.Schema\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n\n        timestamps()\n      end\n\n      @doc false\n      def changeset(app, attrs) do\n        app\n        |> cast(attrs, [:first_name, :last_name])\n        |> validate_required([:first_name, :last_name])\n      end\n    end\n\nWhen using dynamic mode you can define `options/1` and `indexes/1` can be optionally called inside\nthe schema to set options and indexes to be created in dynamic mode. These options will have no\neffect in static mode.\n\nFor example, if you wanted to use a uuid as the key type and create an index on the email the\nfollowing can be used.\n\n    defmodule MyApp.Accounts.User do\n        use ArangoXEcto.Schema\n        import Ecto.Changeset\n\n        options [\n          keyOptions: %{type: :uuid}\n        ]\n\n        indexes [\n          [fields: [:email]]\n        ]\n\n        schema \"users\" do\n          field :email, :string\n\n          timestamps()\n        end\n\n        @doc false\n        def changeset(app, attrs) do\n          app\n          |> cast(attrs, [:first_name, :last_name])\n          |> validate_required([:first_name, :last_name])\n        end\n    end\n\nPlease refer to `ArangoXEcto.Schema` for more information on the available options.","ref":"ArangoXEcto.html#module-schemas","title":"Schemas - ArangoXEcto","type":"module"},{"doc":"Migrations in ArangoXEcto operate just like in other Ecto adapters. However, ArangoXEcto uses its\nown migration commands to allow for interoperability between databases.\n\nFor example, you might want to create a users collection with an index and use ArangoSearch with a\ncustom analyzer to make querying more efficient. The below migration could be used to do just that.\n\n    defmodule MyProject.Repo.Migrations.CreateUsers do\n      use ArangoXEcto.Migration\n\n      def change do\n        create analyzer(:norm_en, :norm, [:frequency, :position], %{\n           locale: \"en\",\n           accent: false,\n           case: :lower\n         })\n\n        create collection(:users) do\n          add :first_name, :string, comment: \"first_name column\"\n          add :last_name, :string\n          add :gender, :integer\n          add :age, :integer\n          add :location, :map\n\n          timestamps()\n        end\n\n        create index(:users, [:age])\n        create unique_index(:users, [:location])\n\n        create view(:user_search, commitIntervalMsec: 1, consolidationIntervalMsec: 1) do\n          add_sort(:created_at, :desc)\n          add_sort(:first_name)\n\n          add_store([:first_name, :last_name], :none)\n\n          add_link(\"users\", %Link{\n            includeAllFields: true,\n            fields: %{\n              last_name: %Link{\n                analyzers: [:identity, :norm_en]\n              }\n            }\n          })\n        end\n      end\n    end\n\nYou can find more information about how migrations are implemented in the `ArangoXEcto.Migration`\nmodule.","ref":"ArangoXEcto.html#module-migrations","title":"Migrations - ArangoXEcto","type":"module"},{"doc":"Sandbox for concurrent tests is implemented similar to the implementation in `ecto_sql`. See\n`ArangoXEcto.Sandbox` for more info.","ref":"ArangoXEcto.html#module-sandbox","title":"Sandbox - ArangoXEcto","type":"module"},{"doc":"Ecto traditionally is made for relational databases and hence relations between schemas is\nrepresented in such a way. A graph relation (edges) can be thought of as many-to-many relations in\nthe relational database world. When it comes to representing graph relations in Ecto, that is\nexactly how it works in ArangoXEcto.\n\nTo use a graph relation you define an `outgoing` and an `incoming` relation on the two schemas,\nfor example the following will create a graph relationship between `User -> UserPost -> Post`.\nThe `UserPost` edge module will be created automatically.\n\n    defmodule MyApp.User do\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n        \n        outgoing :posts, MyApp.Post\n      end\n    end\n\n    defmodule MyApp.Post do\n      schema \"posts\" do\n        field :title, :string\n        \n        incoming :users, MyApp.User\n      end\n    end\n\nBehind the scenes this works very similar to a `many_to_many` relationships but has a few key\ndifferences. As you may know, in a graph an edge can have multiple different node types that it \nconnects to/from. ArangoXEcto uses a special relation type that is based on the principal of\nmany-to-many but modified to allow such relationships.\n\nFor example, you may have a `User` schema that you want to connect to `Post` and `Comment`. You\nmay not want to have two seperate relationships (e.g. `:posts` and `:comments`) and want to\ncombine it all under one (e.g. `:content`). This is what the ArangoXEcto adapter enables. You can\nread more about this type of relationship below.","ref":"ArangoXEcto.html#module-graphs","title":"Graphs - ArangoXEcto","type":"module"},{"doc":"When in dynamic mode the ArangoXEcto adapter dynamically creates and manages edge collections. \nEach edge collection will be created as an Ecto schema when they are first used. This means that \nyou don't need to create edge collections manually. When in static mode you need to define edges\nmanually.\n\nThe edge module will be created under the closest common parent module of the passed modules\nplus the `Edges` alias. The order of the edge name will always be alphabetical to prevent \nduplicate edges. For example, if the modules were `MyApp.Apple.User` and `MyApp.Apple.Banana.Post`\nthen the edge would be created at `MyApp.Apple.Edges.PostUser`. This assumes that the edge \ncollection name was generated and not explicitly defined, if it was `PostUser` would be replaced \nwith the camel case of that collection name (i.e. `address_people` would be `AddressPeople`).","ref":"ArangoXEcto.html#module-edge-modules","title":"Edge Modules - ArangoXEcto","type":"module"},{"doc":"Creating a graph relation that has multiple schemas through the same edge is possible in\nArangoXEcto. For example, take the following.\n\n```mermaid\ngraph TD\nUser --> edge(UsersContent)\nedge(UsersContent) --> Post\nedge(UsersContent) --> Comment\n```\n\nUsers can have posts or comments through the same edge. This works because edges can have multiple\nfrom and to schemas. For example if we were to define the `UsersContent` edge it would look like\nthis:\n\n    defmodule MyApp.Edges.UsersContent do\n      use ArangoXEcto.Edge,\n          from: User,\n          to: [Post, Comment]\n\n      schema \"users_content\" do\n        edge_fields()\n      end\n    end\n\nWe can then define our outgoing definition for the User schema like the following.\n\n    defmodule MyApp.User do\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n        \n        outgoing(\n          :my_content,\n          %{\n            Post => &Post.changeset/2,\n            Comment => &Comment.changeset/2\n          },\n          edge: ArangoXEcto.Edges.UsersContents\n        )\n      end\n    end\n\nThe definition of outgoing here is slightly different from above. Instead of specifying a single\nschema module we specify a map where the keys are schema modules and the values are a list of\nfield names to check if it exists in a result to match it with the module. This is an OR\ncondition, so if you had `[:a, :b]` that would mean if a result had `:a` it would match or if it\nhad `:b` it would match. If a result had both it would match.\n\nNotice that the edge here was explicitly defined. This is not necessary if you aren't doing\nanything custom on the edge and not including this would just generate an edge like the one\nabove.\n\n#### Preloading\n\nDue to how graph relations work the regular `c:Ecto.Repo.preload/3` function will only return maps\ninstead of structs. If you want to preload the structs you can use the `ArangoXEcto.preload/4`\nfunction.\n\n    iex> Repo.all(User) |> ArangoXEcto.preload(Repo, [:my_content])\n    [%User{id: 123, my_content: [%Post{...}, %Comment{...}]}]","ref":"ArangoXEcto.html#module-multiple-edge-schemas","title":"Multiple edge schemas - ArangoXEcto","type":"module"},{"doc":"GeoJSON is supported through the `Geo` module and a handy wrapper is provided in the\n`ArangoXEcto.GeoData` module. This module will just verify and create Geo structs that can then be\nused in geojson fields. The `ArangoXEcto.Types.GeoJSON` module defines the GeoJSON type that can\nbe used when defining fields, for example below.\n\n    schema \"users\" do\n      field :location, ArangoXEcto.Types.GeoJSON\n    end\n\nYou can define a geoJson index on the field using either migrations or dynamic index creation.\n\n    # Migration\n    create index(:users, [:location], type: :geo, geoJson: true)\n\n    # Dynamic creation\n    indexes [\n      [fields: [:location], type: :geo, geoJson: true]\n    ]\n\nYou can use `Geo` structs to query the database.","ref":"ArangoXEcto.html#module-geodata","title":"Geodata - ArangoXEcto","type":"module"},{"doc":"ArangoDB uses ArangoSearch to index collections to improve the searchability of them. With this\nyou can do things like have multiple collections in one \"view\". A view is a concept that is\nessentially take one or more collections and indexes it using whatever \"analyzer(s)\" you choose to\napply.\n\nTo query a View you just use it as if you were using any other collection schema. It will return\nthe results the same as if you were querying that schema.\n\nTo create a View it is very similar to how you create a collection schema. The following will\ncreate a view that has primary sorts on :created_at and :name, it will store the :email and the\n:first_name & :last_name fields. Most importantly it will create a link to the `MyApp.Users`\nschema and set the analyzer for the `:name` field to `:text_en`. Some options can be set also.\n\n  defmodule MyApp.UserSearch do\n    use ArangoXEcto.View\n\n    alias ArangoXEcto.View.Link\n\n    view \"user_search\" do\n      primary_sort :created_at, :desc\n      primary_sort :name\n\n      store_value [:email], :lz4\n      store_value [:first_name, :last_name], :none\n\n      link MyApp.Users, %Link{\n        includeAllFields: true,\n        fields: %{\n          name: %Link{\n            analyzers: [:text_en]\n          }\n        }\n      }\n\n      options  [\n        primarySortCompression: :lz4\n      ]\n    end\n  end\n\nQuerying is done exactly the same as a normal schema except the result will not be a struct.\n\n    iex> Repo.all(MyApp.UsersView)\n    [%{first_name: \"John\", last_name: \"Smith\"}, _]\n\nYou can find out more info in the `ArangoXEcto.View` module.\n\nTo query using the search function you can use the `ArangoXEcto.Query.search/3` and\n`ArangoXEcto.Query.or_search/3` functions. For example, if we wanted to search using the text\nanalyzer we could do the following.\n\n    import ArangoXEcto.Query\n\n    from(UsersView)\n    |> search([uv], fragment(\"ANALYZER(? == ?, \\\"text_en\\\")\", uv.first_name, \"John\"))\n    |> Repo.all()","ref":"ArangoXEcto.html#module-arangosearch","title":"ArangoSearch - ArangoXEcto","type":"module"},{"doc":"ArangoXEcto supplies multiple mix tasks for use with migrations. They are:\n\n  * `mix arango.migrate` - Runs forward through migrations\n  * `mix arango.rollback` - Rolls back migrations\n  * `mix arango.gen.migration` - Generates a migration file\n\nMore information on their usage can be found in their various documentation.","ref":"ArangoXEcto.html#module-mix-tasks","title":"Mix Tasks - ArangoXEcto","type":"module"},{"doc":"Runs an Arangox function using a repo\n\nThis is simply a helper function that extracts the connection from the repo and runs a regular \nquery. It is mainly for internal use but since it can be useful for some custom usecases it is\nincluded in the documentation.","ref":"ArangoXEcto.html#api_query/6","title":"ArangoXEcto.api_query/6","type":"function"},{"doc":"* `repo` - The Ecto repo module used for connection\n  * `function` - An atom of the Arangox function to run\n  * `path` - The path of the query\n  * `body` - The body of the request\n  * `headers` - The headers of the request\n  * `opts` - The last opts arg passed to the function","ref":"ArangoXEcto.html#api_query/6-parameters","title":"Parameters - ArangoXEcto.api_query/6","type":"function"},{"doc":"* `:abort`\n  * `:cursor`\n  * `:delete`\n  * `:delete!`\n  * `:get`\n  * `:get!`\n  * `:head`\n  * `:head!`\n  * `:options`\n  * `:options!`\n  * `:patch`\n  * `:patch!`\n  * `:post`\n  * `:post!`\n  * `:put`\n  * `:put!`\n  * `:request`\n  * `:request!`\n  * `:run`\n  * `:status`\n  * `:transaction` (use built in `c:Ecto.Repo.transaction/2` instead)","ref":"ArangoXEcto.html#api_query/6-supported-functions","title":"Supported Functions - ArangoXEcto.api_query/6","type":"function"},{"doc":"iex> ArangoXEcto.api_query(Repo, :get, \"/_api/collection\")\n    {:ok, %Arangox.Response{body: ...}}\n\n    iex> ArangoXEcto.api_query(Repo, :non_existent, \"/_api/collection\")\n    ** (ArgumentError) Invalid function passed to `Arangox` module","ref":"ArangoXEcto.html#api_query/6-examples","title":"Examples - ArangoXEcto.api_query/6","type":"function"},{"doc":"Runs a raw AQL query on the database.\n\nThis will create a transaction and cursor on Arango and run the raw query.\n\nIf there is an error in the query such as a syntax error, an `Arangox.Error` will be raised.","ref":"ArangoXEcto.html#aql_query/4","title":"ArangoXEcto.aql_query/4","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `query` - The AQL query string to execute\n  * `vars` - A keyword list or a map with the values for variables in the query\n  * `opts` - Options to be passed for the transaction","ref":"ArangoXEcto.html#aql_query/4-parameters","title":"Parameters - ArangoXEcto.aql_query/4","type":"function"},{"doc":"Accepts any of the options accepted by DBConnection.transaction/3, as well as any of the following:\n\n  * :prefix - The prefix to use for the database. Defaults to the repo database.\n  * :read - An array of collection names or a single collection name as a binary to mark as read-only.\n  * :write - An array of collection names or a single collection name as a binary to mark as writable. If not set then no writes will be stored.\n  * :exclusive - An array of collection names or a single collection name as a binary.\n  * :properties - A list or map of additional body attributes to append to the request body when beginning a transaction.\n\nIf doing a write operation, the `:write` operation must be passed. This is not explicitly required for read operations.","ref":"ArangoXEcto.html#aql_query/4-options","title":"Options - ArangoXEcto.aql_query/4","type":"function"},{"doc":"iex> ArangoXEcto.aql_query(\n          Repo,\n          \"FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var\",\n          fname: \"John\",\n          lname: \"Smith\"\n        )\n    {:ok, {1, \n      [\n        %{\n          \"_id\" => \"users/12345\",\n          \"_key\" => \"12345\",\n          \"_rev\" => \"_bHZ8PAK---\",\n          \"first_name\" => \"John\",\n          \"last_name\" => \"Smith\"\n        }\n      ]}}","ref":"ArangoXEcto.html#aql_query/4-examples","title":"Examples - ArangoXEcto.aql_query/4","type":"function"},{"doc":"Same as `ArangoXEcto.aql_query/4 but will raise an error.`\n\nIf there is an error in the query such as a syntax error, an `Arangox.Error` will be raised.","ref":"ArangoXEcto.html#aql_query!/4","title":"ArangoXEcto.aql_query!/4","type":"function"},{"doc":"iex> ArangoXEcto.aql_query!(\n          Repo,\n          \"FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var\",\n          fname: \"John\",\n          lname: \"Smith\"\n        )\n     {1, [\n      %{\n        \"_id\" => \"users/12345\",\n        \"_key\" => \"12345\",\n        \"_rev\" => \"_bHZ8PAK---\",\n        \"first_name\" => \"John\",\n        \"last_name\" => \"Smith\"\n      }\n    ]}","ref":"ArangoXEcto.html#aql_query!/4-examples","title":"Examples - ArangoXEcto.aql_query!/4","type":"function"},{"doc":"Checks if a collection exists\n\nThis will return true if the collection exists in the database, matches the specified type and is not a system\ndatabase, otherwise it will be false.","ref":"ArangoXEcto.html#collection_exists?/4","title":"ArangoXEcto.collection_exists?/4","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for the query\n  * `collection_name` - Name of the collection to check\n  * `type` - The type of collection to check against, defaults to a regular document\n  * `opts` - Options to be used, available options shown below, defaults to none","ref":"ArangoXEcto.html#collection_exists?/4-parameters","title":"Parameters - ArangoXEcto.collection_exists?/4","type":"function"},{"doc":"* `:prefix` - The prefix to use for database collection checking","ref":"ArangoXEcto.html#collection_exists?/4-options","title":"Options - ArangoXEcto.collection_exists?/4","type":"function"},{"doc":"Checking a document collection exists\n\n    iex> ArangoXEcto.collection_exists?(Repo, :users)\n    true\n\nChecking an edge collection exists\n\n    iex> ArangoXEcto.collection_exists?(Repo, \"my_edge\", :edge)\n    true\n\nChecking a system document collection exists does not work\n\n    iex> ArangoXEcto.collection_exists?(Repo, \"_system_test\")\n    false\n\nChecking a collection exists with database prefix\n\n    iex> ArangoXEcto.collection_exists?(Repo, \"test\", \"tenant1\")\n    true","ref":"ArangoXEcto.html#collection_exists?/4-examples","title":"Examples - ArangoXEcto.collection_exists?/4","type":"function"},{"doc":"Creates analyzers in a module\n\nThis function is only to be used in dynamic mode and will raise an error if called in static mode.\nInstead use migrations if in static mode.\n\nWhen operating in dynamic mode the analyzers are automatically created on view create,\nsee `ArangoXEcto.create_view/2` for more info.\n\nThis will first force delete any analyzers with the same name before creating\nthe ones defined.","ref":"ArangoXEcto.html#create_analyzers/3","title":"ArangoXEcto.create_analyzers/3","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `analyzer_module` - The Analyzer module\n  * `opts` - Additional options to use for analyzer creation","ref":"ArangoXEcto.html#create_analyzers/3-parameters","title":"Parameters - ArangoXEcto.create_analyzers/3","type":"function"},{"doc":"* `:prefix` - The prefix for the tenant to create analyzers for","ref":"ArangoXEcto.html#create_analyzers/3-options","title":"Options - ArangoXEcto.create_analyzers/3","type":"function"},{"doc":"iex> ArangoXEcto.create_analyzers(Repo, MyApp.Analyzers)\n    {:ok, [%Arangox.Response{}, ...]}\n\nIf there is an error in and of the schemas\n\n    iex> ArangoXEcto.create_analyzers(Repo, MyApp.Analyzers)\n    {:error, [%Arangox.Response{}, ...], [%Arangox.Error{}, ...]}","ref":"ArangoXEcto.html#create_analyzers/3-examples","title":"Examples - ArangoXEcto.create_analyzers/3","type":"function"},{"doc":"Creates a collection defined by some schema\n\nThis function is only to be used in dynamic mode and will raise an error if called in static mode.\nInstead use migrations if in static mode.","ref":"ArangoXEcto.html#create_collection/3","title":"ArangoXEcto.create_collection/3","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `schema` - The Collection Schema to be created\n  * `opts` - Additional options to pass","ref":"ArangoXEcto.html#create_collection/3-parameters","title":"Parameters - ArangoXEcto.create_collection/3","type":"function"},{"doc":"* `:prefix` - The prefix to use for database tenant collection creation","ref":"ArangoXEcto.html#create_collection/3-options","title":"Options - ArangoXEcto.create_collection/3","type":"function"},{"doc":"iex> ArangoXEcto.create_collection(Repo, MyApp.Users)\n    :ok\n\nIf there is an error in the schema\n\n    iex> ArangoXEcto.create_collection(Repo, MyApp.Users)\n    {:error, %Arangox.Error{}}","ref":"ArangoXEcto.html#create_collection/3-examples","title":"Examples - ArangoXEcto.create_collection/3","type":"function"},{"doc":"Creates an edge between two modules\n\nIf in dynamic mode, the edge collection will be created dynamically if no additional fields are\nrequired, otherwise an edge schema needs to be specified.\n\nEither an edge module or a collection name is required to create the edge, otherwise one will be\ngenerated automatically using the from and two schemas.\n\nThe order of the `from` and `to` matters. A changeset validation error will be raised if\nthe `from` is not defined in the `from` definition of a provided edge. The same goes for the `to`.","ref":"ArangoXEcto.html#create_edge/4","title":"ArangoXEcto.create_edge/4","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `from` - The Ecto Schema struct to use for the from vertex\n  * `to` - The Ecto Schema struct to use for the to vertex\n  * `opts` - Options to use","ref":"ArangoXEcto.html#create_edge/4-parameters","title":"Parameters - ArangoXEcto.create_edge/4","type":"function"},{"doc":"Accepts the following options:\n\n  * `:edge` - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides `collection_name`.\n  * `:collection_name` - The name of the collection to use when generating an edge module.\n  * `:fields` - The values of the fields to set on the edge. Requires `edge` to be set otherwise it is ignored.","ref":"ArangoXEcto.html#create_edge/4-options","title":"Options - ArangoXEcto.create_edge/4","type":"function"},{"doc":"iex> ArangoXEcto.create_edge(Repo, user1, user2)\n    %UserUser{_from: \"users/12345\", _to: \"users/54321\"}\n\nCreate an edge with a specific edge collection name\n\n    iex> ArangoXEcto.create_edge(Repo, user1, user2, collection_name: \"friends\")\n    %Friends{_from: \"users/12345\", _to: \"users/54321\"}\n\nCreate a edge schema and use it to create an edge relation\n\n    defmodule UserPosts do\n      use ArangoXEcto.Edge,\n          from: User,\n          to: Post\n\n      import Ecto.Changeset\n\n      schema \"user_posts\" do\n        edge_fields()\n\n        field(:type, :string)\n      end\n\n      def changeset(edge, attrs) do\n        edges_changeset(edge, attrs)\n        |> cast(attrs, [:type])\n        |> validate_required([:type])\n      end\n    end\n\n    iex> ArangoXEcto.create_edge(Repo, user1, user2, edge: UserPosts, fields: %{type: \"wrote\"})\n    %UserPosts{_from: \"users/12345\", _to: \"users/54321\", from: #Ecto.Association.NotLoaded , to: #Ecto.Association.NotLoaded , type: \"wrote\"}","ref":"ArangoXEcto.html#create_edge/4-examples","title":"Examples - ArangoXEcto.create_edge/4","type":"function"},{"doc":"Creates a view defined by some view schema\n\nThis function is only to be used in dynamic mode and will raise an error if called in static mode.\nInstead use migrations if in static mode.\n\nThis will automatically create analyzers and linked collections. For the analyzers to be \nautomatically created the analyzers module needs to be passed as an option into the view\nmodule definition. See `ArangoXEcto.View` for more info.","ref":"ArangoXEcto.html#create_view/3","title":"ArangoXEcto.create_view/3","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `view` - The View Schema to be created\n  * `opts` - Additional options to use for analyzer creation","ref":"ArangoXEcto.html#create_view/3-parameters","title":"Parameters - ArangoXEcto.create_view/3","type":"function"},{"doc":"* `:prefix` - The prefix for the tenant to create analyzers for","ref":"ArangoXEcto.html#create_view/3-options","title":"Options - ArangoXEcto.create_view/3","type":"function"},{"doc":"iex> ArangoXEcto.create_view(Repo, MyApp.Views.UserSearch)\n    {:ok, %Arangox.Response{}}\n\nIf there is an error in the schema\n\n    iex> ArangoXEcto.create_view(Repo, MyApp.Views.UserSearch)\n    {:error, %Arangox.Error{}}","ref":"ArangoXEcto.html#create_view/3-examples","title":"Examples - ArangoXEcto.create_view/3","type":"function"},{"doc":"Deletes all edges matching matching the query\n\nIf the `:conditions` option is set then those conditions must be true to delete.\n\nTo just delete one edge do so like any other Ecto Schema struct, i.e. using `Ecto.Repo` methods.","ref":"ArangoXEcto.html#delete_all_edges/4","title":"ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for queries\n  * `from` - The Ecto Schema struct to use for the from vertex\n  * `to` - The Ecto Schema struct to use for the to vertex\n  * `opts` - Options to use","ref":"ArangoXEcto.html#delete_all_edges/4-parameters","title":"Parameters - ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"* `:edge` - A specific edge module to use for the edge. Overrides `:collection_name`.\n  * `:collection_name` - The name of the collection to use.\n  * `:conditions` - A keyword list of conditions to filter for edge deletion","ref":"ArangoXEcto.html#delete_all_edges/4-options","title":"Options - ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"Deletes all edges from user1 to user2\n\n    iex> ArangoXEcto.delete_all_edges(Repo, user1, user2)\n    :ok\n\nDeletes all edges from user1 to user2 in specific collection\n\n    iex> ArangoXEcto.delete_all_edges(Repo, user1, user2, collection_name: \"friends\")\n    :ok\n\nDeletes all edges from user1 to user2 that have matching conditions\n\n    iex> ArangoXEcto.delete_all_edges(Repo, user1, user2, conditions: [type: \"best_friend\"])\n    :ok","ref":"ArangoXEcto.html#delete_all_edges/4-examples","title":"Examples - ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"Returns true if a Schema is a document schema\n\nChecks for the presence of the `__schema__/1` function on the module and not an edge.","ref":"ArangoXEcto.html#document?/1","title":"ArangoXEcto.document?/1","type":"function"},{"doc":"* `module` - the module to check","ref":"ArangoXEcto.html#document?/1-parameters","title":"Parameters - ArangoXEcto.document?/1","type":"function"},{"doc":"Returns true if a Schema is an edge\n\nChecks for the presence of the `__edge__/0` function on the module.","ref":"ArangoXEcto.html#edge?/1","title":"ArangoXEcto.edge?/1","type":"function"},{"doc":"* `module` - the module to check","ref":"ArangoXEcto.html#edge?/1-parameters","title":"Parameters - ArangoXEcto.edge?/1","type":"function"},{"doc":"Generates an edge schema dynamically\n\nIf a collection name is not provided one will be dynamically generated. The naming convention\nis the names of the modules in alphabetical order. E.g. `User` and `Post` will combine for a collection\nname of `posts_users` (uses the collection names of the modules) and an edge module name of `PostUser`. \nThis order is used to prevent duplicates if the from and to orders are switched.\n\nYou can also use a list of from and to modules, e.g. from: User, to: [Post, Comment], would result\nin a module name of CommentPostUser and a collection name of comments_posts_users. This may not\nalways be an optimal naming convention so it is generally a good idea to make your own edge collection.\n\nThis will create the Ecto Module in the environment dynamically. It will create it under the closest\ncommon parent module of the passed modules plus the `Edges` alias. For example, if the modules were\n`MyApp.Apple.User` and `MyApp.Apple.Banana.Post` then the edge would be created at `MyApp.Apple.Edges.PostUser`.\n\nReturns the Edge Module name as an atom.","ref":"ArangoXEcto.html#edge_module/3","title":"ArangoXEcto.edge_module/3","type":"function"},{"doc":"* `from_modules` - Ecto Schema modules for the from part of the edge\n  * `to_modules` - Ecto Schema modules for the to part of the edge\n  * `opts` - Options passed for module generation","ref":"ArangoXEcto.html#edge_module/3-parameters","title":"Parameters - ArangoXEcto.edge_module/3","type":"function"},{"doc":"* `:collection_name` - The name of collection to use instead of generating it","ref":"ArangoXEcto.html#edge_module/3-options","title":"Options - ArangoXEcto.edge_module/3","type":"function"},{"doc":"iex> ArangoXEcto.edge_module(MyProject.User, MyProject.Company, [collection_name: \"works_for\"])\n    MyProject.Edges.WorksFor\n\n    iex> ArangoXEcto.edge_module(MyProject.User, [MyProject.Company, MyProject.Group])\n    MyProject.Edges.CompanyGroupUser","ref":"ArangoXEcto.html#edge_module/3-examples","title":"Examples - ArangoXEcto.edge_module/3","type":"function"},{"doc":"Gets an ID from a module and a key","ref":"ArangoXEcto.html#get_id_from_module/2","title":"ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"* `module` - Module to get the collection name from\n  * `key` - The `_key` to use in the id","ref":"ArangoXEcto.html#get_id_from_module/2-parameters","title":"Parameters - ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"iex> ArangoXEcto.get_id_from_module(User, \"123456\")\n    \"users/123456\"","ref":"ArangoXEcto.html#get_id_from_module/2-example","title":"Example - ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"Gets an ID from a schema struct\n\nIf the struct has been queried from the database it will have the `:__id__` field with the id\nwhich is used. Otherwise it is generated using the schema name and the id (the `_key`).","ref":"ArangoXEcto.html#get_id_from_struct/1","title":"ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"* `struct` - The Ecto struct","ref":"ArangoXEcto.html#get_id_from_struct/1-parameters","title":"Parameters - ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"If the User schema's collection name is `users` the following would be:\n\n    iex> user = %User{id: \"123456\"}\n    %User{id: \"123456\"}\n\n    iex> ArangoXEcto.get_id_from_struct(user)\n    \"users/123456\"","ref":"ArangoXEcto.html#get_id_from_struct/1-example","title":"Example - ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"Returns the name of a database for prefix\n\nThis simply prepends the tenant name to the database supplied in the repo config.","ref":"ArangoXEcto.html#get_prefix_database/2","title":"ArangoXEcto.get_prefix_database/2","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for the query\n  * `prefix` - The prefix to get the database name for","ref":"ArangoXEcto.html#get_prefix_database/2-parameters","title":"Parameters - ArangoXEcto.get_prefix_database/2","type":"function"},{"doc":"Loads raw map into Ecto structs\n\nUses `c:Ecto.Repo.load/2` to load a deep map result into Ecto structs.\n\nIf a list of maps are passed then the maps are enumerated over.\n\nA list of modules can also be passes for possible types. The `_id`\nfield is used to check against the module schemas. This is especially\nuseful for querying against Arango views where there may be multiple\nschema results returned.\n\nIf a module is passed that isn't a Ecto Schema then an error will be\nraised.","ref":"ArangoXEcto.html#load/2","title":"ArangoXEcto.load/2","type":"function"},{"doc":"* `maps` - List of maps or singular map to convert to a struct\n  * `module` - Module(s) to use for the struct","ref":"ArangoXEcto.html#load/2-parameters","title":"Parameters - ArangoXEcto.load/2","type":"function"},{"doc":"iex> {:ok, users} = ArangoXEcto.aql_query(\n          Repo,\n          \"FOR user IN users RETURN user\"\n        )\n    {:ok,\n      [\n        %{\n          \"_id\" => \"users/12345\",\n          \"_key\" => \"12345\",\n          \"_rev\" => \"_bHZ8PAK---\",\n          \"first_name\" => \"John\",\n          \"last_name\" => \"Smith\"\n        }\n      ]}\n\n    iex> ArangoXEcto.load(users, User)\n    [\n      %User{\n        id: \"12345\",\n        first_name: \"John\",\n        last_name: \"Smith\"\n      }\n    ]","ref":"ArangoXEcto.html#load/2-example","title":"Example - ArangoXEcto.load/2","type":"function"},{"doc":"Preloads all associations on the given struct or structs.\n\nThis is similar to `c:Ecto.Repo.preload/3` except it loads graph associations. It functions the same\nas `c:Ecto.Repo.preload/3` but has a slight syntax change.\n\nThis is needed because graph associations use a fake relation module to simulate the required\nbehaviour. This means that due to a check in Ecto preloads, it won't match the queried structs.\nTherefore this function applies some logic to preload the data properly.","ref":"ArangoXEcto.html#preload/4","title":"ArangoXEcto.preload/4","type":"function"},{"doc":"* `structs_or_struct_or_nil` - A singular or list of structs to preload (if nil will return nil)\n  * `repo` - the Ecto repo to use\n  * `preloads` - the preload fields (see `c:Ecto.Repo.preload/3` for more info)\n  * `opts` - the options to use (same as `c:Ecto.Repo.preload/3`","ref":"ArangoXEcto.html#preload/4-parameters","title":"Parameters - ArangoXEcto.preload/4","type":"function"},{"doc":"iex> Repo.all(User) |> ArangoXEcto.preload(Repo, [:my_content])\n    [%User{id: 123, my_content: [%Post{...}, %Comment{...}]}]","ref":"ArangoXEcto.html#preload/4-example","title":"Example - ArangoXEcto.preload/4","type":"function"},{"doc":"","ref":"ArangoXEcto.html#t:query/0","title":"ArangoXEcto.query/0","type":"type"},{"doc":"Returns the type of a module\n\nThis is just a shortcut to using `edge?/1`, `document?/1` and `view?/1`. If it is none of them nil is returned.","ref":"ArangoXEcto.html#schema_type/1","title":"ArangoXEcto.schema_type/1","type":"function"},{"doc":"* `module` - the module to check","ref":"ArangoXEcto.html#schema_type/1-parameters","title":"Parameters - ArangoXEcto.schema_type/1","type":"function"},{"doc":"A real view\n\n    iex> ArangoXEcto.schema_type(MyApp.SomeView)\n    :view\n\nA real edge schema\n\n    iex> ArangoXEcto.schema_type(MyApp.RealEdge)\n    :edge\n\nSome module that is not an Ecto schema\n\n    iex> ArangoXEcto.schema_type(MyApp.RandomModule)\n    nil","ref":"ArangoXEcto.html#schema_type/1-examples","title":"Examples - ArangoXEcto.schema_type/1","type":"function"},{"doc":"Same as schema_type/1 but throws an error on none\n\nThis is just a shortcut to using `is_edge/1`, `is_document/1` and `is_view/1`. If it is none of them nil is returned.","ref":"ArangoXEcto.html#schema_type!/1","title":"ArangoXEcto.schema_type!/1","type":"function"},{"doc":"* `module` - the module to check","ref":"ArangoXEcto.html#schema_type!/1-parameters","title":"Parameters - ArangoXEcto.schema_type!/1","type":"function"},{"doc":"A real edge schema\n\n    iex> ArangoXEcto.schema_type!(MyApp.RealEdge)\n    :edge\n\nSome module that is not an Ecto schema\n\n    iex> ArangoXEcto.schema_type!(MyApp.RandomModule)\n    ** (ArgumentError) Not an Ecto Schema","ref":"ArangoXEcto.html#schema_type!/1-examples","title":"Examples - ArangoXEcto.schema_type!/1","type":"function"},{"doc":"","ref":"ArangoXEcto.html#t:vars/0","title":"ArangoXEcto.vars/0","type":"type"},{"doc":"Returns true if a Schema is a view schema\n\nChecks for the presence of the `__view__/1` function on the module.","ref":"ArangoXEcto.html#view?/1","title":"ArangoXEcto.view?/1","type":"function"},{"doc":"* `module` - the module to check","ref":"ArangoXEcto.html#view?/1-parameters","title":"Parameters - ArangoXEcto.view?/1","type":"function"},{"doc":"Checks if a view exists\n\nThis will return true if the view exists in the database, otherwise false.","ref":"ArangoXEcto.html#view_exists?/2","title":"ArangoXEcto.view_exists?/2","type":"function"},{"doc":"* `repo` - The Ecto repo module to use for the query\n  * `view_name` - Name of the collection to check","ref":"ArangoXEcto.html#view_exists?/2-parameters","title":"Parameters - ArangoXEcto.view_exists?/2","type":"function"},{"doc":"Checking a document collection exists\n\n    iex> ArangoXEcto.view_exists?(Repo, :users_search)\n    true","ref":"ArangoXEcto.html#view_exists?/2-examples","title":"Examples - ArangoXEcto.view_exists?/2","type":"function"},{"doc":"Ecto adapter for ArangoDB using ArangoX\n\nThis implements methods for `Ecto.Adapter`. These functions should not be accessed directly\nand should only be used by Ecto. Direct interaction functions are in the `ArangoXEcto` module.","ref":"ArangoXEcto.Adapter.html","title":"ArangoXEcto.Adapter","type":"module"},{"doc":"Returns true if a connection has been checked out","ref":"ArangoXEcto.Adapter.html#checked_out?/1","title":"ArangoXEcto.Adapter.checked_out?/1","type":"function"},{"doc":"Checks out a connection for the duration of the given function.","ref":"ArangoXEcto.Adapter.html#checkout/3","title":"ArangoXEcto.Adapter.checkout/3","type":"function"},{"doc":"Returns the dumpers for a given type.","ref":"ArangoXEcto.Adapter.html#dumpers/2","title":"ArangoXEcto.Adapter.dumpers/2","type":"function"},{"doc":"Ensure all applications necessary to run the adapter are started","ref":"ArangoXEcto.Adapter.html#ensure_all_started/2","title":"ArangoXEcto.Adapter.ensure_all_started/2","type":"function"},{"doc":"Receives a DDL command and executes it","ref":"ArangoXEcto.Adapter.html#execute_ddl/3","title":"ArangoXEcto.Adapter.execute_ddl/3","type":"function"},{"doc":"Initialise adapter with `config`","ref":"ArangoXEcto.Adapter.html#init/1","title":"ArangoXEcto.Adapter.init/1","type":"function"},{"doc":"Returns the loaders of a given type.","ref":"ArangoXEcto.Adapter.html#loaders/2","title":"ArangoXEcto.Adapter.loaders/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#lock_with_migrations/3","title":"ArangoXEcto.Adapter.lock_with_migrations/3","type":"function"},{"doc":"Methods to manipulate an Ecto Changeset\n\nThe methods within the module are specific to ArangoXEcto and only exist here if there isn't an \nadequite function available already in the `Ecto.Changeset` module.\n\nWhen working with changesets you need to cast or put assocs, well you need to do that for graph\nrelations to. However, you need to use a special version of the function called\n`ArangoXEcto.Changeset.cast_graph/3`. This works like the following.\n\n    defmodule MyApp.User do\n      import Ecto.Changeset\n      import ArangoXEcto.Changeset\n\n      ...\n\n      def changeset(user, attrs \\ %{}) do\n        user\n        |> cast(attrs, [...])\n        |> cast_graph(:my_content, with: %{\n          Post => [:title],\n          Comment => [:text]\n        })\n      end\n    end\n\nYou can also use the put version (`ArangoXEcto.Changeset.put_graph/4`) just like you would use\n`put_assoc/4`.","ref":"ArangoXEcto.Changeset.html","title":"ArangoXEcto.Changeset","type":"module"},{"doc":"`cast_graph` and `cast_assoc` (also the put versions) are very similar in functionality but with \nsome key differences. Firstly, the syntax. You would have seen above that `cast_graph/3` allows\nfor a module or a mapper. Secondly, `cast_graph/3` allows for the special multiple nodes to edges\nbehaviour as seen above.\n\n> #### Note {: .error}\n>\n> `cast_assoc/3` and `put_assoc/4` will **NOT** work properly because of how the graph relations\n> work. So you will need to make sure to use the `cast_graph/3` and `put_graph/4` functions.","ref":"ArangoXEcto.Changeset.html#module-cast_graph-vs-cast_assoc","title":"cast_graph vs cast_assoc - ArangoXEcto.Changeset","type":"module"},{"doc":"Casts the given graph association with the changeset parameters.\n\nThis operates simarly to `Ecto.Changeset.cast_assoc/3` (and indeed was based\non the implementation) but allows for a relation to represent multiple \npossible options. For example, in the following `:my_content` can represent \neither a post or a comment.\n\n    # Implicit use the module changesets\n    ArangoXEcto.Changeset.cast_graph(changeset, :my_content)\n\n    # Explicit use the module changesets\n    ArangoXEcto.Changeset.cast_graph(changeset, :my_content, with: %{\n      Post => &Post.changeset/2,\n      Comment => &Comment.changeset/2,\n    })\n\nLike with `Ecto.Changeset.cast_assoc/3`, the changeset must have explicitly preloaded the graph\nassociation before being cast. Refer to the Ecto docs for more info.","ref":"ArangoXEcto.Changeset.html#cast_graph/3","title":"ArangoXEcto.Changeset.cast_graph/3","type":"function"},{"doc":"* `changeset` - The Ecto changeset to cast\n  * `name` - The name of the field to cast\n  * `opts` - Options to use (described below)","ref":"ArangoXEcto.Changeset.html#cast_graph/3-parameters","title":"Parameters - ArangoXEcto.Changeset.cast_graph/3","type":"function"},{"doc":"* `:required` - if the graph association is a required field. A non-empty list is \n    satisfactory.\n\n  * `:required_message` - the message on failure, defaults to \"can't be blank\"\n\n  * `:invalid_message` - the message on failure, defaults to \"is invalid\"\n\n  * `:force_update_on_change` - force the parent record to be updated in the\n    repository if there is a change, defaults to `true`\n\n  * `:with` - can either be a function or a map where the keys are a schema module and the values\n    is the function to use. The function is used to build the changeset from params. Defaults to\n    the `changeset/2` function of the associated module. It can be an anonymous function that \n    expects two arguments: the associated struct to be cast and its parameters. It must return a\n    changeset. Functions with arity 3 are accepted, and the third argument will be the position\n    of the associated element in the list, or `nil`, if the association is being replaced.\n\n  * `:drop_param` - the parameter name which keeps a list of indexes to drop from the relation\n    parameters\n\n  * `:sort_param` - the parameter name which keeps a list of indexes to sort from the relation\n    parameters. Unknown indexes are considered to be new entries. Non-listed indexes will come \n    before any sorted ones.","ref":"ArangoXEcto.Changeset.html#cast_graph/3-options","title":"Options - ArangoXEcto.Changeset.cast_graph/3","type":"function"},{"doc":"Puts the given graph association entry or entries into a graph association.\n\nThis operates simarly to `Ecto.Changeset.put_assoc/4` (and indeed was based\noff of the implementation) but allows for a relation to represent multiple \npossible options. For example, in the following `:my_content` can represent \neither a post or a comment.\n\n    ArangoXEcto.Changeset.put_graph(changeset, :my_content, [\n      %Post{title: \"abc\"},\n      %Comment{text: \"cba\"}\n    ])\n\nThis operates on the graph association as a whole. So for example the above will replace the\n`:my_content` edges between the nodes. This works exactly the same as\n`Ecto.Changeset.put_assoc/4`. Therefore, if you want to update the content you will have to\npreload the value and update from there. The `:on_replace` value for the field applies here.\n\nAn empty list can be given too.\n\nThe data provided is the same as the available options in `Ecto.Changeset.put_assoc/4`. I.e. you\ncan provide a map, a keyword list, a changeset or a struct as the options in the list.\n\nAlthough it accepts an `opts` argument, there are no options currently supported by \n`put_graph/4`.","ref":"ArangoXEcto.Changeset.html#put_graph/4","title":"ArangoXEcto.Changeset.put_graph/4","type":"function"},{"doc":"* `changeset` - The Ecto changeset to apply the change to\n  * `name` - The name of the field to put the value to\n  * `value` - The value of the field to apply\n  * `opts` - none available currently","ref":"ArangoXEcto.Changeset.html#put_graph/4-parameters","title":"Parameters - ArangoXEcto.Changeset.put_graph/4","type":"function"},{"doc":"Defines an Arango Edge collection as an Ecto Schema Module.\n\nThis will define the required fields of an edge (`_from` and `_to`) and will define the default changeset.\n\nEdge modules are dynamically created in the environment if they don't already exist. For more on \nhow edge modules are dynamically generated, please read `ArangoXEcto.edge_module/3`.\n\nEdges utilise Ecto relationships so that the powerful Ecto features can be used. Each Node \nrequires a relationship to be set, either a `outgoing/3` or `incoming/3`. Behind the scenes this \ncreates a relationship similar to an Ecto many_to_many relationship and generates (or uses the \nprovided) edge module as the intermediary schema. \n\nSince the edge collection uses the full `_id` instead of the `_key` for the `_from` and `_to` \nfields, we can't use the `:id` field. Schemas actually have an additional field which is \n`:__id__` and stores the `_id` value.","ref":"ArangoXEcto.Edge.html","title":"ArangoXEcto.Edge","type":"behaviour"},{"doc":"Defining an edge is similar to how you would define a collection.\n\nWhen defining an edge schema you must define the `:from` and `:to` fields on the schema. You can\nuse the `edge_fields/0` function as a shortcut to define these fields.\n\nA `changeset/2` function is automatically defined on the custom schema module but this must be \noverridden this so that you can cast and validate the custom fields. The `edges_changeset/2` \nmethod should be called to automatically implement the casting and validation of the `_from` and \n`_to` fields. It does not have to be before any custom field operations but it good convention to \ndo so.","ref":"ArangoXEcto.Edge.html#module-creating-an-edge","title":"Creating an edge - ArangoXEcto.Edge","type":"behaviour"},{"doc":"defmodule MyProject.UserPosts do\n      use ArangoXEcto.Edge,\n          from: User,\n          to: Post\n\n      import Ecto.Changeset\n\n      schema \"user_posts\" do\n        edge_fields()\n\n        field(:type, :string)\n      end\n\n      def changeset(edge, attrs) do\n        edges_changeset(edge, attrs)\n        |> cast(attrs, [:type])\n        |> validate_required([:type])\n      end\n    end","ref":"ArangoXEcto.Edge.html#module-example","title":"Example - ArangoXEcto.Edge","type":"behaviour"},{"doc":"Unlike in a regular many-to-many relationship, edges in ArangoDB can have multiple schemas\nassociated with it in the same from/to fields. In ArangoXEcto this is represented by using a list\nfor the `from` and `to` options when defining an edge. For example, you could have an edge like\nthe following.\n\n    defmodule MyProject.MyContent do\n      use ArangoXEcto.Edge,\n          from: User,\n          to: [Post, Comment]\n\n      ...\n    end","ref":"ArangoXEcto.Edge.html#module-multiple-from-and-to-schemas","title":"Multiple from and to schemas - ArangoXEcto.Edge","type":"behaviour"},{"doc":"","ref":"ArangoXEcto.Edge.html#c:changeset/2","title":"ArangoXEcto.Edge.changeset/2","type":"callback"},{"doc":"Macro to define the required edge fields i.e. `_from` and `_to`.\n\nThis is required when using a custom edge schema and can be used as below.\n\n    schema \"user_posts\" do\n      edge_fields()\n\n      field(:type, :string)\n    end\n\nThe options passed are directly supplied to the \nArangoXEcto.Association.EdgeMany relation.","ref":"ArangoXEcto.Edge.html#edge_fields/1","title":"ArangoXEcto.Edge.edge_fields/1","type":"macro"},{"doc":"Default changeset for an edge.\n\nCasts and requires the `_from` and `_to` fields. This will also verify the format of both fields to match that of\nan Arango id.\n\nAny custom changeset should first use this changeset.\n\nDirect use of the `edges_changeset/2` function is discouraged unless per the use case mentioned above.","ref":"ArangoXEcto.Edge.html#edges_changeset/2","title":"ArangoXEcto.Edge.edges_changeset/2","type":"function"},{"doc":"To add a required `type` field, you could do the following:\n\n    def changeset(edge, attrs) do\n      edges_changeset(edge, attrs)\n      |> cast(attrs, [:type])\n      |> validate_required([:type])\n    end","ref":"ArangoXEcto.Edge.html#edges_changeset/2-example","title":"Example - ArangoXEcto.Edge.edges_changeset/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Edge.html#t:t/0","title":"ArangoXEcto.Edge.t/0","type":"type"},{"doc":"ArangoDB specific `Ecto.Query` functions.\n\nThe `all/1`, `delete_all/1` and `update_all/1` functions in this module convert `Ecto.Query` \nstructs into AQL syntax.\n\nThe Arango specific functions are available to support ArangoSearch questions and perform graph\nqueries.","ref":"ArangoXEcto.Query.html","title":"ArangoXEcto.Query","type":"module"},{"doc":"ArangoSearch queries can be performed through the `search/3` and `or_search/3` macros. For\nexample, if you wanted to do a wildcard search for the title of your favorite movie you can do the\nfollowing.\n\n    from(MoviesView)\n    |> search([mv], fragment(\"LIKE(?, ?)\", mv.title, \"%War%\"))\n    |> Repo.all()\n\nYou can see that a fragment is used to incluse custom AQL. If you wanted to use a specific \nanalyzer, for example for case-insensitive searches, you could so using a fragment also like \nbelow.\n\n    from(MoviesView)\n    |> search([mv], fragment(\"ANALYZER(LIKE(?, ?), \\\"norm_en\\\")\", mv.title, \"%war%\"))\n    |> Repo.all()","ref":"ArangoXEcto.Query.html#module-arangosearch-queries","title":"ArangoSearch queries - ArangoXEcto.Query","type":"module"},{"doc":"You can do graph queries inside Ecto queries. To do this you just use the `graph/5` macro\nprovided. This will generate the required fragment to do graph queries.\n\nFor example, take the following AQL query.\n\n```\nFOR u IN 1..3 OUTBOUND \"user/bob\" friends\n  RETURN u\n```\n\nYou can represent that using the following Ecto query. Note that a select is required since Ecto\nneeds to know what fields to get when querying on a fragment.\n\n    from(\n      u in graph(1..3, :outbound, \"user/bob\", \"friends\"), \n        select: map(u, [:first_name, :last_name])\n    )\n    |> Repo.all()\n\n> #### Credit {: .info}\n>\n> This initial code for this module was used from \n> https://github.com/ArangoDB-Community/arangodb_ecto/blob/master/lib/arangodb_ecto/query.ex.\n> Credit for the initial code goes to `mpoeter`. Updates were made since to enable additional\n> functionality and to work with the latest Ecto version.","ref":"ArangoXEcto.Query.html#module-graph-searches","title":"Graph searches - ArangoXEcto.Query","type":"module"},{"doc":"Creates an AQL query to fetch all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#all/1","title":"ArangoXEcto.Query.all/1","type":"function"},{"doc":"Creates an AQL query to delete all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#delete_all/1","title":"ArangoXEcto.Query.delete_all/1","type":"function"},{"doc":"Query an ArangoDB graph\n\nThis is a helper around searching a graph. This essentially generates a fragment that is used in\nthe from or join part of the Ecto query.\n\nYou **MUST** provide a select in the query, otherwise an error be raised since Ecto can't work out\nwhat fields to use when a fragment is used.\n\nThere is a limitiation that you can't use the `Ecto.Query.API.struct/2` function since the query\nis a fragment. If you need to load your results into a struct then you can use the\n`ArangoXEcto.load/2` function.","ref":"ArangoXEcto.Query.html#graph/5","title":"ArangoXEcto.Query.graph/5","type":"macro"},{"doc":"For example, lets take the following query.\n\n```\nFOR u IN 1..3 OUTBOUND \"user/bob\" friends\n  RETURN u\n```\n\nYou can represent that using the following Ecto query. Note that like you would in any other Ecto\nquery, you need to pin any variables used.\n\n    from(\n      u in graph(1..3, :outbound, \"user/bob\", \"friends\"), \n        select: map(u, [:first_name, :last_name])\n    )\n    |> Repo.all()","ref":"ArangoXEcto.Query.html#graph/5-example","title":"Example - ArangoXEcto.Query.graph/5","type":"macro"},{"doc":"You can also provide a keyword list as options as the last parameter to the macro.\n\n```\nFOR u IN 1..3 OUTBOUND \"user/bob\" friends OPTIONS { vertexCollections: [\"users\"] }\n  RETURN u\n```\n\nThis can be done using like the following.\n\n    from(u in graph(1..3, :outbound, \"user/bob\", \"friends\", vertexCollections: [\"users\"]))\n    |> select([u], map(u, [:first_name, :last_name]))\n    |> Repo.all()","ref":"ArangoXEcto.Query.html#graph/5-providing-options","title":"Providing options - ArangoXEcto.Query.graph/5","type":"macro"},{"doc":"If you want to select all of the fields on a struct you can use the following code and pin the\n`select_fields` variable. Replace the `User` schema with the schema in question.\n\n    select_fields = Enum.map(User.__schema__(:fields), &User.__schema__(:field_source, &1))","ref":"ArangoXEcto.Query.html#graph/5-dynamically-selecting-fields","title":"Dynamically selecting fields - ArangoXEcto.Query.graph/5","type":"macro"},{"doc":"A OR search query expression.\n\nExtention to the `Ecto.Query` api for arango searches.\n\nThis function is the same as `ArangoXEcto.Query.search/3` except implements as an or clause.\nThis also follows the same syntax as the `Ecto.Query.or_where/3` function.","ref":"ArangoXEcto.Query.html#or_search/3","title":"ArangoXEcto.Query.or_search/3","type":"macro"},{"doc":"An AND search query expression.\n\nExtention to the `Ecto.Query` api for arango searches.\n\nThe expression syntax is exactly the same as the regular `Ecto.Query.where/3` clause.\nRefer to that for more info on syntax or for advanced AQL queries see the section below.\n\nYou will need to import this function like you do for `Ecto.Query`.","ref":"ArangoXEcto.Query.html#search/3","title":"ArangoXEcto.Query.search/3","type":"macro"},{"doc":"This will store the search in the Ecto query `where` clause\nwith a custom search operation. This is to prevent having to create a seperate query\ntype. When converting the Ecto query to an AQL query, this is caught and changed\ninto an AQL `SEARCH` expression.","ref":"ArangoXEcto.Query.html#search/3-implementation","title":"Implementation - ArangoXEcto.Query.search/3","type":"macro"},{"doc":"To save having to learn some new kind of query format for so many different possible search\nscenarios, you can just use AQL directly in. This is powered by the `Ecto.Query.API.fragment/1`\nfunction.\n\nBelow is an example of how you can implement using a custom analyzer:\n\n    from(UsersView)\n    |> search([uv], fragment(\"ANALYZER(? == ?, \\\"identity\\\")\", uv.first_name, \"John\"))\n    |> Repo.all()","ref":"ArangoXEcto.Query.html#search/3-using-analyzers-and-advanced-aql","title":"Using Analyzers and advanced AQL - ArangoXEcto.Query.search/3","type":"macro"},{"doc":"Creates an AQL query to update all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#update_all/1","title":"ArangoXEcto.Query.update_all/1","type":"function"},{"doc":"An ArangoDB specific implementation of the `Ecto.Schema` module.\n\nThe primary key is the Arango `_key` field and is mapped to `:id`. The `_id` field is also \nprovided in the `:__id__` field. The `:__id__` field is primarily used for graph associations but\nis available in all structs.\n\nSchema modules should use this module by adding `use ArangoXEcto.Schema` to the module. The only\nexception to this is if the collection is an edge collection, in that case refer to \n`ArangoXEcto.Edge`.\n\nThis module works, for the most part, the same as the `Ecto.Schema` module. Where the two modules\ndiffer is that this module sets the primary key and adds the `:__id__` field, as described above.\nIn addition, additional functionality for ArangoDB is available, such as `outgoing` and `incoming`\nrelations and collection indexes & options.","ref":"ArangoXEcto.Schema.html","title":"ArangoXEcto.Schema","type":"module"},{"doc":"defmodule MyProject.Accounts.User do\n      use ArangoXEcto.Schema\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n\n        outgoing :posts, Post\n\n        timestamps()\n      end\n\n      @doc false\n      def changeset(app, attrs) do\n        app\n        |> cast(attrs, [:first_name, :last_name])\n        |> validate_required([:first_name, :last_name])\n      end\n    end","ref":"ArangoXEcto.Schema.html#module-example","title":"Example - ArangoXEcto.Schema","type":"module"},{"doc":"Defines an incoming relationship\n\nThis is almost exactly the same as `outgoing/3` except it defines that the relationship is\nincoming. This means that on the edge the from and to order is swapped. It will work exactly the\nsame if you used an `incoming/3` or an `outgoing/3`.\n\nOne thing to keep in mind is that this function will not automaticallly define an edge module\nand will require an `outbound/3` definition. This doesn't matter when supplying an edge schema as\nthe options.\n\nFor more information and available options see `outgoing/3`.","ref":"ArangoXEcto.Schema.html#incoming/3","title":"ArangoXEcto.Schema.incoming/3","type":"macro"},{"doc":"defmodule MyProject.Post do\n      use ArangoXEcto.Schema\n\n      schema \"posts\" do\n        field :title, :string\n\n        # Will use the automatically generated edge\n        incoming :users, MyProject.User\n\n        # Will use the UserPosts edge\n        incoming :users, MyProject.User, edge: MyProject.UserPosts\n\n        # Creates a multi relation\n        incoming :entities, %{\n          MyProject.User => [:first_name],\n          MyProject.Company => [:name]\n        }, edge: MyProject.Entities\n      end\n    end\n\nFor the multi relation example, the following could be the edge definition.\n\n    defmodule MyProject.Entities do\n      use ArangoXEcto.Edge,\n        from: [MyProject.User, MyProject.Company],\n        to: MyProject.Post\n\n      schema \"entities\" do\n        edge_fields()\n      end\n    end","ref":"ArangoXEcto.Schema.html#incoming/3-example","title":"Example - ArangoXEcto.Schema.incoming/3","type":"macro"},{"doc":"Defines indexes for collection dynamic creation\n\nWhen using dynamic mode for collection creation this will add indexes after the collection is \ncreated. If in static mode, this will simply be ignored.\n\nThis accepts a list of keyword lists that are the indexes. The available options can be found in \nthe `ArangoXEcto.Migration.Index` module. \n\nThis function also works in edge collections.","ref":"ArangoXEcto.Schema.html#indexes/1","title":"ArangoXEcto.Schema.indexes/1","type":"macro"},{"doc":"To create a generic hash index you don't need to pass the type.\n\n    indexes [\n      [fields: [:email, :username]]\n    ]\n\nTo create a geoJson index set the type to geo and set `geoJson` to true.\n\n    indexes [\n      [fields: [:point], type: :geo, geoJson: true]\n    ]\n\nTo create a two separate indexes just supply them separately.\n\n    indexes [\n      [fields: [:email]],\n      [fields: [:username]]\n    ]","ref":"ArangoXEcto.Schema.html#indexes/1-example","title":"Example - ArangoXEcto.Schema.indexes/1","type":"macro"},{"doc":"Defines collection options for dynamic creation\n\nWhen using dynamic mode for collection creation this will add options for when the collection is \ncreated. If in static mode, this will simply be ignored.\n\nThis accepts a keyword list of options. The available options can be found in the \n`ArangoXEcto.Migration.Collection` module.\n\nThis function also works in edge collections.","ref":"ArangoXEcto.Schema.html#options/1","title":"ArangoXEcto.Schema.options/1","type":"macro"},{"doc":"To specify a UUID as the collection key type you just supply it as the key option type.\n\n    options [\n      keyOptions: %{type: :uuid}\n    ]","ref":"ArangoXEcto.Schema.html#options/1-example","title":"Example - ArangoXEcto.Schema.options/1","type":"macro"},{"doc":"Defines an outgoing relationship to many objects\n\nWhile not strictly required, you can add an `incoming/3` definition in the other target node so\nthat the reverse can be queried also.\n\nIf in static mode you still have to have a migration, even for the dynamically generated edges.\nYou can use the rules of how edges are named from `ArangoXEcto.edge_module/3` for the name of the\nedge or if you want to be explicit (and it often is good practice to do so) you can create an edge\nmodule and pass the edge.","ref":"ArangoXEcto.Schema.html#outgoing/3","title":"ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"* `name` - The name of the relation\n  * `target` - The target of the relation, can either be another schema or a mapper (see below for\n              more info)\n  * `opts` - See below","ref":"ArangoXEcto.Schema.html#outgoing/3-parameters","title":"Parameters - ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"* `:edge` - The edge schema to use, default will generate a new edge\n  * `:on_replace` - The action taken on the association when the record is\n                    replaced when casting or manipulating the parent changeset.\n  * `:where` - A filter for the association. See \"Filtering associations\" in \n              `Ecto.Schema.has_many/3`.","ref":"ArangoXEcto.Schema.html#outgoing/3-options","title":"Options - ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"In ArangoDB you can have multiple different types of collections related through an edge. To \nrepresent this we use a map to represent the target. The keys of the map is the schema that we\nwant to use and the value is a list of field names. When Ecto maps a result to a struct it will\nuse the list of fields to determine which schema to use. If any of the fields in the list are \npresent then that schema will be chosen. If two match, then the first will be selected.\n\n    outgoing :content, %{\n      MyProject.Post => [:title],\n      MyProject.Comment => [:text]\n    }, edge: MyProject.Content\n\nYou can specify an `incoming/3` relationship in, for example, the `MyProject.Post` and the\n`MyProject.Comment` schemas seperatley. The only requirement would be to ensure the same edge is\nused.\n\nThe edge definition would look something like the following. Take note of the list supplied for\nthe `:to` option.\n\n    defmodule MyProject.Content do\n      use ArangoXEcto.Edge,\n        from: MyProject.User,\n        to: [MyProject.Post, MyProject.Comment]\n\n      schema \"content\" do\n        edge_fields()\n      end\n    end","ref":"ArangoXEcto.Schema.html#outgoing/3-mapping-a-target","title":"Mapping a target - ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"defmodule MyProject.User do\n      use ArangoXEcto.Schema\n\n      schema \"users\" do\n        field :name, :string\n\n        # Will use the automatically generated edge\n        outgoing :posts, MyProject.Post\n\n        # Will use the UserPosts edge\n        outgoing :posts, MyProject.Post, edge: MyProject.UserPosts\n\n        # Creates a multi relation\n        outgoing :content, %{\n          MyProject.Post => [:title],\n          MyProject.Comment => [:text]\n        }, edge: MyProject.Content\n      end\n    end","ref":"ArangoXEcto.Schema.html#outgoing/3-example","title":"Example - ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"Defines an ArangoDB schema.\n\nA wrapper around the `Ecto.Schema.schema/2` to add the `:__id__` field. For more info, see the\ndocs for `Ecto.Schema.schema/2`.","ref":"ArangoXEcto.Schema.html#schema/2","title":"ArangoXEcto.Schema.schema/2","type":"macro"},{"doc":"Generates a ArangoXEcto migration.\n\nUnlike the default `ecto_sql` implementation, this will only generated migrations for repos that\nuse the ArangoXEcto.Adapter. This prevents it trying to run ArangoDB migrations on \nnon-ArangoXEcto repos. This is particularly useful for when using multiple database providers in\none app, e.g. ArangoXEcto and PostgreSQL (through `ecto_sql`).\n\nThe repository must be set under `:ecto_repos` in the current app\nconfiguration or given via the `-r` option.","ref":"Mix.Tasks.Arango.Gen.Migration.html","title":"mix arango.gen.migration","type":"task"},{"doc":"mix arango.gen.migration create_users\n    mix arango.gen.migration create_users -r Custom.Repo\n\nThe generated migration filename will be prefixed with the current timestamp\nin UTC which is used for versioning and ordering.\n\nBy default, the migration will be generated to the\n`priv/YOUR_REPO/migrations` directory of the current application but it can\nbe configured to be any subdirectory of `priv` by specifying the `:priv` key\nunder the repository configuration.\n\nThis generator will automatically open the generated file if you have\n`ECTO_EDITOR` set in your environment variable.","ref":"Mix.Tasks.Arango.Gen.Migration.html#module-examples","title":"Examples - mix arango.gen.migration","type":"task"},{"doc":"* `-r`, `--repo` - the repo to generate migration for\n  * `--no-compile` - does not compile applications before running\n  * `--no-deps-check` - does not check dependencies before running\n  * `--migrations-path` - the path to run the migrations from, defaults to `priv/repo/migrations`","ref":"Mix.Tasks.Arango.Gen.Migration.html#module-command-line-options","title":"Command line options - mix arango.gen.migration","type":"task"},{"doc":"If the current app configuration specifies a custom migration module\nthe generated migration code will use that rather than the default\n`ArangoXEcto.Migration`:\n\n    config :arangox_ecto, migration_module: MyApplication.CustomMigrationModule","ref":"Mix.Tasks.Arango.Gen.Migration.html#module-configuration","title":"Configuration - mix arango.gen.migration","type":"task"},{"doc":"Runs the pending migrations for the given repository.\n\nUnlike the default `ecto_sql` implementation, these migrations will only be run for repos that use\nthe ArangoXEcto.Adapter. This prevents it trying to run ArangoDB migrations on non-ArangoXEcto\nrepos. This is particularly useful for when using multiple database providers in one app, e.g.\nArangoXEcto and PostgreSQL (through `ecto_sql`).\n\nMigrations are expected at \"priv/YOUR_REPO/migrations\" directory\nof the current application, where \"YOUR_REPO\" is the last segment\nin your repository name. For example, the repository `MyApp.Repo`\nwill use \"priv/repo/migrations\". The repository `Whatever.MyRepo`\nwill use \"priv/my_repo/migrations\".\n\nYou can configure a repository to use another directory by specifying\nthe `:priv` key under the repository configuration. The \"migrations\"\npart will be automatically appended to it. For instance, to use\n\"priv/custom_repo/migrations\":\n\n    config :my_app, MyApp.Repo, priv: \"priv/custom_repo\"\n\nThis task runs all pending migrations by default. To migrate up to a\nspecific version number, supply `--to version_number`. To migrate a\nspecific number of times, use `--step n`.\n\nThe repositories to migrate are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nSince ArangoXEcto tasks can only be executed once, if you need to migrate\nmultiple repositories, set `:ecto_repos` accordingly or pass the `-r`\nflag multiple times.\n\nIf a repository has not yet been started, one will be started outside\nyour application supervision tree and shutdown afterwards.","ref":"Mix.Tasks.Arango.Migrate.html","title":"mix arango.migrate","type":"task"},{"doc":"$ mix arango.migrate\n    $ mix arango.migrate -r Custom.Repo\n                        \n    $ mix arango.migrate -n 3\n    $ mix arango.migrate --step 3\n                        \n    $ mix arango.migrate --to 20080906120000","ref":"Mix.Tasks.Arango.Migrate.html#module-examples","title":"Examples - mix arango.migrate","type":"task"},{"doc":"* `--all` - run all pending migrations\n\n  * `--log-migrations` - log migration commands\n\n  * `--log-migrator` - log the migrator\n\n  * `--log-level` - the level to set for `Logger`. This task\n    does not start your application, so whatever level you have configured in\n    your config files will not be used. If this is not provided, no level\n    will be set, so that if you set it yourself before calling this task\n    then this won't interfere. Can be any of the `t:Logger.level/0` levels\n\n  * `--migrations-path` - the path to load the migrations from, defaults to\n    `\"priv/repo/migrations\"`. This option may be given multiple times in which\n    case the migrations are loaded from all the given directories and sorted\n    as if they were in the same one\n\n  * `--no-compile` - does not compile applications before migrating\n\n  * `--no-deps-check` - does not check dependencies before migrating\n\n  * `--pool-size` - the pool size if the repository is started\n    only for the task (defaults to 2)\n\n  * `--prefix` - the prefix to run migrations on\n\n  * `--quiet` - do not log migration commands\n\n  * `-r`, `--repo` - the repo to migrate\n\n  * `--step`, `-n` - run n number of pending migrations\n\n  * `--strict-version-order` - abort when applying a migration with old\n    timestamp (otherwise it emits a warning)\n\n  * `--to` - run all migrations up to and including version\n\n  * `--to-exclusive` - run all migrations up to and excluding version","ref":"Mix.Tasks.Arango.Migrate.html#module-command-line-options","title":"Command line options - mix arango.migrate","type":"task"},{"doc":"Reverts applied migrations for the given repository.\n\nUnlike the default `ecto_sql` implementation, the rollbacks will only be run for repos that use\nthe ArangoXEcto.Adapter. This prevents it trying to rollback ArangoDB migrations on \nnon-ArangoXEcto repos. This is particularly useful for when using multiple database providers \nin one app, e.g. ArangoXEcto and PostgreSQL (through `ecto_sql`).\n\nThis task rolls back the last applied migration by default. To roll\nback to a version number, supply `--to version_number`. To roll\nback a specific number of times, use `--step n`. To undo all applied\nmigrations, provide `--all`.\n\nThe repositories to rollback are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nSince ArangoXEcto tasks can only be executed once, if you need to migrate\nmultiple repositories, set `:ecto_repos` accordingly or pass the `-r`\nflag multiple times.\n\nIf a repository has not yet been started, one will be started outside\nyour application supervision tree and shutdown afterwards.","ref":"Mix.Tasks.Arango.Rollback.html","title":"mix arango.rollback","type":"task"},{"doc":"$ mix arango.rollback\n    $ mix arango.rollback -r Custom.Repo\n          \n    $ mix arango.rollback -n 3\n    $ mix arango.rollback --step 3\n          \n    $ mix arango.rollback --to 20080906120000","ref":"Mix.Tasks.Arango.Rollback.html#module-examples","title":"Examples - mix arango.rollback","type":"task"},{"doc":"* `--all` - run all pending migrations\n\n  * `--log-migrations` - log migration commands\n\n  * `--log-migrator` - log the migrator\n\n  * `--log-level` - the level to set for `Logger`. This task\n    does not start your application, so whatever level you have configured in\n    your config files will not be used. If this is not provided, no level\n    will be set, so that if you set it yourself before calling this task\n    then this won't interfere. Can be any of the `t:Logger.level/0` levels\n\n  * `--migrations-path` - the path to load the migrations from, defaults to\n    `\"priv/repo/migrations\"`. This option may be given multiple times in which\n    case the migrations are loaded from all the given directories and sorted\n    as if they were in the same one\n\n  * `--no-compile` - does not compile applications before migrating\n\n  * `--no-deps-check` - does not check dependencies before migrating\n\n  * `--pool-size` - the pool size if the repository is started\n    only for the task (defaults to 2)\n\n  * `--prefix` - the prefix to run migrations on\n\n  * `--quiet` - do not log migration commands\n\n  * `-r`, `--repo` - the repo to migrate\n\n  * `--step`, `-n` - run n number of pending migrations\n\n  * `--strict-version-order` - abort when applying a migration with old\n    timestamp (otherwise it emits a warning)\n\n  * `--to` - run all migrations up to and including version\n\n  * `--to-exclusive` - run all migrations up to and excluding version","ref":"Mix.Tasks.Arango.Rollback.html#module-command-line-options","title":"Command line options - mix arango.rollback","type":"task"},{"doc":"Defines an analyzer for use in views\n\nThis is only used when in dynamic mode. When using static mode you will need to define migrations\nfor analyzers and any Analyzer definitions using this module will be ignored.\n\nSince analyzer defintions are short and you may have many of them, you can just define multiple\nanalyzer modules in one file, e.g. named analyzers.ex.","ref":"ArangoXEcto.Analyzer.html","title":"ArangoXEcto.Analyzer","type":"module"},{"doc":"defmodule MyApp.Analyzers do\n      use ArangoXEcto.Analyzer\n\n      norm :norm_en, [:frequency, :norm, :position], %{\n        locale: \"en\",\n        accent: false,\n        case: :lower\n      }\n\n      # this exists by default but is just used as an example\n      text :text_en, [:frequency, :norm, :position], %{\n        locale: \"en\",\n        accent: false,\n        stemming: true,\n        case: :lower\n      }\n\n      # Needed to compile the analyzers\n      build()\n    end","ref":"ArangoXEcto.Analyzer.html#module-example","title":"Example - ArangoXEcto.Analyzer","type":"module"},{"doc":"The following are the features available to all the analyzers. Some analyzers and functions need certin features enabled,\nrefer to the [ArangoDB docs](https://www.arangodb.com/docs/stable/analyzers.html#analyzer-features) for more info.\n\n    * `:frequency` - (boolean) - track how often a term occurs.\n    * `:norm` - (boolean) - calculate and store the field normalization factor that is used to score fairer if the same term is repeated, reducing its importance.\n    * `:position` - (boolean) - enumerate the tokens for position-dependent queries.","ref":"ArangoXEcto.Analyzer.html#module-features","title":"Features - ArangoXEcto.Analyzer","type":"module"},{"doc":"Defines an aql type analyzer.\n\nAn Analyzer capable of running a restricted AQL query to perform data manipulation / filtering.\n\nRefer to the [ArangoDB AQL Docs](https://www.arangodb.com/docs/stable/analyzers.html#aql) for more info.","ref":"ArangoXEcto.Analyzer.html#aql/3","title":"ArangoXEcto.Analyzer.aql/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#aql/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.aql/3","type":"macro"},{"doc":"* `queryString` - (string) - AQL query to be executed.\n    * `collapsePositions` - (boolean) - whether to set the position to 0 for all members of the query result array (true) or\n      set the position corresponding to the index of the result array member (false).\n    * `keepNull` - (boolean) - either treat treat null like an empty string or discard null.\n    * `batchSize` - (integer) - number between 1 and 1000 (default = 1) that determines the batch size for reading data from the query.\n    * `memoryLimit` - (integer) - memory limit for query execution in bytes. (default is 1048576 = 1Mb) Maximum is 33554432U (32Mb).\n    * `returnType` - (string) - data type of the returned tokens.\n      `:string` - convert emitted tokens to strings.\n      `:number` - convert emitted tokens to numbers.\n      `:bool` - convert emitted tokens to booleans.","ref":"ArangoXEcto.Analyzer.html#aql/3-properties","title":"Properties - ArangoXEcto.Analyzer.aql/3","type":"macro"},{"doc":"Compiles analyzers","ref":"ArangoXEcto.Analyzer.html#build/0","title":"ArangoXEcto.Analyzer.build/0","type":"macro"},{"doc":"Defines a classification type analyzer.\n\nAn Analyzer capable of classifying tokens in the input text.\n\nRefer to the [ArangoDB Classification Docs](https://www.arangodb.com/docs/stable/analyzers.html#classification) for more info.","ref":"ArangoXEcto.Analyzer.html#classification/3","title":"ArangoXEcto.Analyzer.classification/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#classification/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.classification/3","type":"macro"},{"doc":"* `model_location` - (string) - the on-disk path to the trained fastText supervised model.\n    * `top_k` - (number) - the number of class labels that will be produced per input (default: 1).\n    * `threshold` - (number) - the probability threshold for which a label will be assigned to an input. A fastText\n      model produces a probability per class label, and this is what will be filtered (default: 0.99).","ref":"ArangoXEcto.Analyzer.html#classification/3-properties","title":"Properties - ArangoXEcto.Analyzer.classification/3","type":"macro"},{"doc":"Defines a collation type analyzer.\n\nAn Analyzer capable of converting the input into a set of language-specific tokens. This makes comparisons follow the\nrules of the respective language, most notable in range queries against Views.\n\nRefer to the [ArangoDB Collation Docs](https://www.arangodb.com/docs/stable/analyzers.html#collation) for more info.","ref":"ArangoXEcto.Analyzer.html#collation/3","title":"ArangoXEcto.Analyzer.collation/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#collation/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.collation/3","type":"macro"},{"doc":"* `locale` - (string) - a locale in the format language[_COUNTRY] (square brackets denote optional parts), e.g. \"de\" or \"en_US\".","ref":"ArangoXEcto.Analyzer.html#collation/3-properties","title":"Properties - ArangoXEcto.Analyzer.collation/3","type":"macro"},{"doc":"Defines a delimiter type analyzer.\n\nAn Analyzer capable of breaking up delimited text into tokens as per RFC 4180 (without starting new records on newlines).\n\nRefer to the [ArangoDB Delimiter Docs](https://www.arangodb.com/docs/stable/analyzers.html#delimiter) for more info.","ref":"ArangoXEcto.Analyzer.html#delimiter/3","title":"ArangoXEcto.Analyzer.delimiter/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#delimiter/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.delimiter/3","type":"macro"},{"doc":"* `delimiter` - (string) - the delimiting character(s). The whole string is considered as one delimiter.","ref":"ArangoXEcto.Analyzer.html#delimiter/3-properties","title":"Properties - ArangoXEcto.Analyzer.delimiter/3","type":"macro"},{"doc":"Defines a geo_s2 type analyzer.\n\nAn Analyzer capable of breaking up a GeoJSON object or coordinate array in [longitude, latitude] order into a set of indexable\ntokens for further usage with ArangoSearch Geo functions.\n\nRefer to the [ArangoDB Geo S2 Docs](https://www.arangodb.com/docs/stable/analyzers.html#geo_s2) for more info.","ref":"ArangoXEcto.Analyzer.html#geo_s2/3","title":"ArangoXEcto.Analyzer.geo_s2/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#geo_s2/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.geo_s2/3","type":"macro"},{"doc":"* `format` - (atom) - the internal binary representation to use for storing the geo-spatial data in an index \n      * `:latLngDouble` (default) - store each latitude and longitude value as an 8-byte floating-point value\n        (16 bytes per coordinate pair). This format preserves numeric values exactly and is more compact than the\n        VelocyPack format used by the geojson Analyzer.\n      * `:latLngInt` - store each latitude and longitude value as an 4-byte integer value (8 bytes per coordinate\n        pair). This is the most compact format but the precision is limited to approximately 1 to 10 centimeters.\n      * `:s2Point` - store each longitude-latitude pair in the native format of Google S2 which is used for\n        geo-spatial calculations (24 bytes per coordinate pair). This is not a particular compact format but it\n        reduces the number of computations necessary when you execute geo-spatial queries. This format preserves\n        numeric values exactly.\n    * `type` - (atom) - type of geojson object\n      * `:shape` (default) - index all GeoJSON geometry types (Point, Polygon etc.)\n      * `:centroid` - compute and only index the centroid of the input geometry\n      * `:point` - only index GeoJSON objects of type Point, ignore all other geometry types\n    * `options` - (map) - options for fine-tuning geo queries. These options should generally remain unchanged \n      * `:maxCells` (number, optional) - maximum number of S2 cells (default: 20)\n      * `:minLevel` (number, optional) - the least precise S2 level (default: 4)\n      * `:maxLevel` (number, optional) - the most precise S2 level (default: 23)","ref":"ArangoXEcto.Analyzer.html#geo_s2/3-properties","title":"Properties - ArangoXEcto.Analyzer.geo_s2/3","type":"macro"},{"doc":"Defines a geojson type analyzer.\n\nAn Analyzer capable of breaking up a GeoJSON object or coordinate array in [longitude, latitude] order into a set\nof indexable tokens for further usage with ArangoSearch Geo functions.\n\nRefer to the [ArangoDB GeoJSON Docs](https://www.arangodb.com/docs/stable/analyzers.html#geojson) for more info.","ref":"ArangoXEcto.Analyzer.html#geojson/3","title":"ArangoXEcto.Analyzer.geojson/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#geojson/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.geojson/3","type":"macro"},{"doc":"* `type` - (atom) - type of geojson object\n      * `:shape` (default) - index all GeoJSON geometry types (Point, Polygon etc.)\n      * `:centroid` - compute and only index the centroid of the input geometry\n      * `:point` - only index GeoJSON objects of type Point, ignore all other geometry types\n    * `options` - (map) - options for fine-tuning geo queries. These options should generally remain unchanged \n      * `:maxCells` (number, optional) - maximum number of S2 cells (default: 20)\n      * `:minLevel` (number, optional) - the least precise S2 level (default: 4)\n      * `:maxLevel` (number, optional) - the most precise S2 level (default: 23)","ref":"ArangoXEcto.Analyzer.html#geojson/3-properties","title":"Properties - ArangoXEcto.Analyzer.geojson/3","type":"macro"},{"doc":"Defines a geopoint type analyzer.\n\nAn Analyzer capable of breaking up a coordinate array in [latitude, longitude] order or a JSON object describing a\ncoordinate pair using two separate attributes into a set of indexable tokens for further usage with ArangoSearch Geo functions.\n\nRefer to the [ArangoDB Geo Point Docs](https://www.arangodb.com/docs/stable/analyzers.html#geopoint) for more info.","ref":"ArangoXEcto.Analyzer.html#geopoint/3","title":"ArangoXEcto.Analyzer.geopoint/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#geopoint/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.geopoint/3","type":"macro"},{"doc":"* `latitude` - (list of string) - list of strings that describes the attribute path of the latitude value\n      relative to the field for which the Analyzer is defined in the View\n    * `longitude` - (list of string) - list of strings that describes the attribute path of the longitude value\n      relative to the field for which the Analyzer is defined in the View\n    * `options` - (map) - options for fine-tuning geo queries. These options should generally remain unchanged \n      * `:maxCells` (number, optional) - maximum number of S2 cells (default: 20)\n      * `:minLevel` (number, optional) - the least precise S2 level (default: 4)\n      * `:maxLevel` (number, optional) - the most precise S2 level (default: 23)","ref":"ArangoXEcto.Analyzer.html#geopoint/3-properties","title":"Properties - ArangoXEcto.Analyzer.geopoint/3","type":"macro"},{"doc":"Defines an identity type analyzer.\n\nAn Analyzer applying the identity transformation, i.e. returning the input unmodified.\n\nRefer to the [ArangoDB Identity Docs](https://www.arangodb.com/docs/stable/analyzers.html#identity) for more info.\n\nThis does not accept any properties.","ref":"ArangoXEcto.Analyzer.html#identity/2","title":"ArangoXEcto.Analyzer.identity/2","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))","ref":"ArangoXEcto.Analyzer.html#identity/2-parameters","title":"Parameters - ArangoXEcto.Analyzer.identity/2","type":"macro"},{"doc":"Defines a minhash type analyzer.\n\nAn Analyzer that computes so called MinHash signatures using a locality-sensitive hash function. It applies an Analyzer of\nyour choice before the hashing, for example, to break up text into words.\n\nRefer to the [ArangoDB MinHash Docs](https://www.arangodb.com/docs/stable/analyzers.html#minhash) for more info.","ref":"ArangoXEcto.Analyzer.html#minhash/4","title":"ArangoXEcto.Analyzer.minhash/4","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below\n    * `analyzer` - block with one analyzer (if more then one is supplied, only the last will be used","ref":"ArangoXEcto.Analyzer.html#minhash/4-parameters","title":"Parameters - ArangoXEcto.Analyzer.minhash/4","type":"macro"},{"doc":"* `numHashes` - (number) - the size of the MinHash signature.","ref":"ArangoXEcto.Analyzer.html#minhash/4-properties","title":"Properties - ArangoXEcto.Analyzer.minhash/4","type":"macro"},{"doc":"Defines a nearest_neighbors type analyzer.\n\nAn Analyzer capable of finding nearest neighbors of tokens in the input.\n\nRefer to the [ArangoDB Nearest Neighbors Docs](https://www.arangodb.com/docs/stable/analyzers.html#nearest_neighbors) for more info.","ref":"ArangoXEcto.Analyzer.html#nearest_neighbors/3","title":"ArangoXEcto.Analyzer.nearest_neighbors/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#nearest_neighbors/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.nearest_neighbors/3","type":"macro"},{"doc":"* `model_location` - (string) - the on-disk path to the trained fastText supervised model.\n    * `top_k` - (number) - the number of class labels that will be produced per input (default: 1).","ref":"ArangoXEcto.Analyzer.html#nearest_neighbors/3-properties","title":"Properties - ArangoXEcto.Analyzer.nearest_neighbors/3","type":"macro"},{"doc":"Defines a ngram type analyzer.\n\nAn Analyzer capable of producing n-grams from a specified input in a range of min..max (inclusive). Can optionally preserve the original input.\n\nRefer to the [ArangoDB NGram Docs](https://www.arangodb.com/docs/stable/analyzers.html#ngram) for more info.","ref":"ArangoXEcto.Analyzer.html#ngram/3","title":"ArangoXEcto.Analyzer.ngram/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#ngram/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.ngram/3","type":"macro"},{"doc":"* `min` - (integer) - minimum n-gram length.\n    * `max` - (integer) - maximum n-gram length.\n    * `preserveOriginal` - (boolean) - whether to include the original value or just use the min & max values.\n    * `startMarker` - (string) - this value will be prepended to n-grams which include the beginning of the input.\n    * `endMarker` - (string) - this value will be appended to n-grams which include the end of the input.\n    * `streamType` - (atom) - type of the input stream.\n      * `:binary` - one byte is considered as one character (default).\n      * `:utf8` - one Unicode codepoint is treated as one character.","ref":"ArangoXEcto.Analyzer.html#ngram/3-properties","title":"Properties - ArangoXEcto.Analyzer.ngram/3","type":"macro"},{"doc":"Defines a norm type analyzer.\n\nAn Analyzer capable of normalizing the text, treated as a single token, i.e. case conversion and accent removal.\n\nRefer to the [ArangoDB Norm Docs](https://www.arangodb.com/docs/stable/analyzers.html#norm) for more info.","ref":"ArangoXEcto.Analyzer.html#norm/3","title":"ArangoXEcto.Analyzer.norm/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#norm/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.norm/3","type":"macro"},{"doc":"* `locale` - (string) - a locale in the format language[_COUNTRY] (square brackets denote optional parts), e.g. \"de\" or \"en_US\".\n    * `accent` - (boolean) - whether to preserve accented characters or convert them to the base characters.\n    * `case` - (atom) - option of how to store case\n      * `:lower` - to convert to all lower-case characters\n      * `:upper` - to convert to all upper-case characters\n      * `:none` - to not change character case (default)","ref":"ArangoXEcto.Analyzer.html#norm/3-properties","title":"Properties - ArangoXEcto.Analyzer.norm/3","type":"macro"},{"doc":"Defines a pipeline type analyzer.\n\nAn Analyzer capable of chaining effects of multiple Analyzers into one. The pipeline is a list of Analyzers, where the output\nof an Analyzer is passed to the next for further processing. The final token value is determined by last Analyzer in the pipeline.\n\nRefer to the [ArangoDB Pipeline Docs](https://www.arangodb.com/docs/stable/analyzers.html#pipeline) for more info.\n\n>","ref":"ArangoXEcto.Analyzer.html#pipeline/3","title":"ArangoXEcto.Analyzer.pipeline/3","type":"macro"},{"doc":">\n> Features are only required on the pipeline and not on the individual analyzers within. Any features on sub analyzers will be ignored if supplied.","ref":"ArangoXEcto.Analyzer.html#pipeline/3-note-info","title":"Note {: .info} - ArangoXEcto.Analyzer.pipeline/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `analyzers` - a block with other analyzers","ref":"ArangoXEcto.Analyzer.html#pipeline/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.pipeline/3","type":"macro"},{"doc":"pipeline :my_pipeline, [:frequency, :norm, :position] do\n      norm \"norm_en\",  %{\n        locale: \"en\",\n        accent: false,\n        case: :lower\n      }\n\n      text \"text_en\", %{\n        locale: \"en\",\n        accent: false,\n        stemming: true,\n        case: :lower\n      }\n    end","ref":"ArangoXEcto.Analyzer.html#pipeline/3-example","title":"Example - ArangoXEcto.Analyzer.pipeline/3","type":"macro"},{"doc":"Defines a segmentation type analyzer.\n\nAn Analyzer capable of breaking up the input text into tokens in a language-agnostic manner as per Unicode Standard Annex #29,\nmaking it suitable for mixed language strings.\n\nRefer to the [ArangoDB Segmentation Docs](https://www.arangodb.com/docs/stable/analyzers.html#segmentation) for more info.","ref":"ArangoXEcto.Analyzer.html#segmentation/3","title":"ArangoXEcto.Analyzer.segmentation/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#segmentation/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.segmentation/3","type":"macro"},{"doc":"* `break` - (atom) - character to break at\n      * `:all` - return all tokens\n      * `:alpha` - return tokens composed of alphanumeric characters only (default). Alphanumeric characters are Unicode codepoints from the\n        Letter and Number categories, see Unicode Technical Note #36.\n      * `:graphic` - return tokens composed of non-whitespace characters only. Note that the list of whitespace characters does not include line breaks:\n        * `U+0009` Character Tabulation\n        * `U+0020` Space\n        * `U+0085` Next Line\n        * `U+00A0` No-break Space\n        * `U+1680` Ogham Space Mark\n        * `U+2000` En Quad\n        * `U+2028` Line Separator\n        * `U+202F` Narrow No-break Space\n        * `U+205F` Medium Mathematical Space\n        * `U+3000` Ideographic Space\n    * `case` - (atom) - option of how to store case\n      * `:lower` - to convert to all lower-case characters\n      * `:upper` - to convert to all upper-case characters\n      * `:none` - to not change character case (default)","ref":"ArangoXEcto.Analyzer.html#segmentation/3-properties","title":"Properties - ArangoXEcto.Analyzer.segmentation/3","type":"macro"},{"doc":"Defines a stem type analyzer.\n\nAn Analyzer capable of stemming the text, treated as a single token, for supported languages.\n\nRefer to the [ArangoDB Stem Docs](https://www.arangodb.com/docs/stable/analyzers.html#stem) for more info.","ref":"ArangoXEcto.Analyzer.html#stem/3","title":"ArangoXEcto.Analyzer.stem/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#stem/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.stem/3","type":"macro"},{"doc":"* `locale` - (string) - a locale in the format language[_COUNTRY] (square brackets denote optional parts), e.g. \"de\" or \"en_US\".","ref":"ArangoXEcto.Analyzer.html#stem/3-properties","title":"Properties - ArangoXEcto.Analyzer.stem/3","type":"macro"},{"doc":"Defines a stopwords type analyzer.\n\nAn Analyzer capable of removing specified tokens from the input.\n\nRefer to the [ArangoDB Stopwords Docs](https://www.arangodb.com/docs/stable/analyzers.html#stopwords) for more info.","ref":"ArangoXEcto.Analyzer.html#stopwords/3","title":"ArangoXEcto.Analyzer.stopwords/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#stopwords/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.stopwords/3","type":"macro"},{"doc":"* `stopwords` - (list of strings) - array of strings that describe the tokens to be discarded.\n    * `hex` - (boolean) - If false (default), then each string in stopwords is used verbatim.","ref":"ArangoXEcto.Analyzer.html#stopwords/3-properties","title":"Properties - ArangoXEcto.Analyzer.stopwords/3","type":"macro"},{"doc":"","ref":"ArangoXEcto.Analyzer.html#t:t/0","title":"ArangoXEcto.Analyzer.t/0","type":"type"},{"doc":"Defines a text type analyzer.\n\nAn Analyzer capable of breaking up strings into individual words while also optionally filtering out stop-words,\nextracting word stems, applying case conversion and accent removal.\n\nRefer to the [ArangoDB Text Docs](https://www.arangodb.com/docs/stable/analyzers.html#text) for more info.","ref":"ArangoXEcto.Analyzer.html#text/3","title":"ArangoXEcto.Analyzer.text/3","type":"macro"},{"doc":"* `name` - atom of the analyzer name\n    * `features` - the features options to be set (see [Analyzer Features](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Analyzer.html#module-features))\n    * `properties` - a map of the properties to be set, see below","ref":"ArangoXEcto.Analyzer.html#text/3-parameters","title":"Parameters - ArangoXEcto.Analyzer.text/3","type":"macro"},{"doc":"* `locale` - (string) - a locale in the format language[_COUNTRY] (square brackets denote optional parts), e.g. \"de\" or \"en_US\".\n    * `accent` - (boolean) - whether to preserve accented characters or convert them to the base characters.\n    * `case` - (string) - option of how to store case\n      * `:lower` - to convert to all lower-case characters\n      * `:upper` - to convert to all upper-case characters\n      * `:none` - to not change character case (default)\n    * `stemming` - (boolean) - whether to apply stemming on returned words or leave as-is\n    * `edgeNgram` - (map) - if present, then edge n-grams are generated for each token (word). \n      * `min` - (integer) - minimum n-gram length.\n      * `max` - (integer) - maximum n-gram length.\n      * `preserveOriginal` - (boolean) - whether to include the original value or just use the min & max values.\n    * `stopwords` - (list of strings) - a list of strings with words to omit from result.\n    * `stopwordsPath` - (string) - path with a language sub-directory (e.g. en for a locale en_US) containing files with words to omit.","ref":"ArangoXEcto.Analyzer.html#text/3-properties","title":"Properties - ArangoXEcto.Analyzer.text/3","type":"macro"},{"doc":"ArangoSearch view module\n\nDefines a schema for a view to help create and manage Arango Search views.","ref":"ArangoXEcto.View.html","title":"ArangoXEcto.View","type":"module"},{"doc":"Like for collections, the same static vs dynamic system is in place.\nIf in dynamic mode and a collection that a view links to doesn't exist when querying,\nit will be created automatically. If in static mode an error will be raised if the the\nappropriate migration has not been made. Check out the `ArangoXEcto.Migration` module for\nmore info.\n\nLike for collections, there is no check if a view exists when use the\n`ArangoXEcto.aql_query/4` function and will error, so make sure the view exists first.","ref":"ArangoXEcto.View.html#module-creation","title":"Creation - ArangoXEcto.View","type":"module"},{"doc":"defmodule Something.UserSearch do\n    use ArangoXEcto.View\n\n    alias ArangoXEcto.View.Link\n\n    view \"user_search\" do\n      primary_sort :created_at, :desc\n      primary_sort :name\n\n      store_value [:email], :lz4\n      store_value [:first_name, :last_name], :none\n\n      link MyApp.Users, %Link{\n        includeAllFields: true,\n        fields: %{\n          name: %Link{\n            analyzers: [:text_en]\n          }\n        }\n      }\n\n      options  [\n        primarySortCompression: :lz4\n      ]\n    end\n  end","ref":"ArangoXEcto.View.html#module-example","title":"Example - ArangoXEcto.View","type":"module"},{"doc":"**When in dynamic mode** the analyzer module needs to be passed to the view so that \nthe analyzers will be automatically created. This option will be ignored in static mode. \nYou can pass the analyzer module by passing the module to the `:analyzer_module` option on \nthe use statement.\n\n    use ArangoXEcto.View, analyzer_module: Something.Analyzers","ref":"ArangoXEcto.View.html#module-defining-the-analyzer-module","title":"Defining the analyzer module - ArangoXEcto.View","type":"module"},{"doc":"Views essentially operate as virtual wrappers of Ecto Schemas. You can use them just like\nother schemas to query on and all the fields available are a culmination of the fields on the\nlink schemas. This is how it works in the ArangoDB so to keep it true to function, it\nfunctions the same here.\n\nQuerying is done exactly the same as a normal schema except the result will not be a struct. This is\nexplained further below.\n\n    iex> Repo.all(MyApp.UsersView)\n    [%{first_name: \"John\", last_name: \"Smith\"}, _]","ref":"ArangoXEcto.View.html#module-querying","title":"Querying - ArangoXEcto.View","type":"module"},{"doc":"Since the Arango Search function heavily relies on the AQL `SEARCH` operation it only makes\nsense for this to also work in Ecto Queries. You can find more info about this under `ArangoXEcto.Query.search/3`\nand `ArangoXEcto.Query.or_search/3`.\n\nThe search operation functions the same as the `Ecto.Query.where/3` clause so that can be used for more reference.\nYou can also use fragments to use special analyzers, an example is given below.\n\n> #### Note {: .info}\n>\n> You must import the ArangoXEcto.Query function to use the search query macro.\n\n    iex> MyApp.UsersView |> search(gender: :male) |> Repo.all()\n    [%{gender: :male}, ...]\n\n    iex> from(MyApp.UsersView) |> search([uv], uv.gender == :female) |> Repo.all()\n    [%{gender: :female}, ...]\n\n    iex> UsersView |> search([uv], fragment(\"ANALYZER(? == ?, \"identity\")\", uv.first_name, \"John\")) |> Repo.all()\n    [%{first_name: \"John\"}]\n\nUnfortunately this won't work as an argument to the Ecto Query `from` macro (e.g. below) and will\nonly work as above.\n\n    # DO NOT USE THIS, IT WILL NOT WORK\n    iex> from(uv in MyApp.UsersView, search: uv.first_name == \"John\")\n    ArgumentError\n\nOf course you can also use AQL queries. Noting that you need to ensure the view is created already\nbecause it will not be checked.\n\n  iex> ArangoXEcto.aql_query(Repo,\n  ...>    \"FOR uv IN @@view SEARCH ANALYZER(uv.first_name == @first_name, \"identity\") RETURN uv\",\n  ...>    \"@view\": UsersView.__view__(:name),\n  ...>    first_name: \"John\"\n  ...> )\n  {:ok, [%{\"first_name\" => \"John\"}]}\n\n\n#### Sorting\n\nSince views work with ecto queries as per usual, you can use functions such as sort in Ecto queries.\nIf you want to sort based on the BM25 score for example, you can use fragments like below.\n\n    iex> UsersView\n    ...> |> search(gender: :male)\n    ...> |> order_by([uv], fragment(\"BM25(?)\", uv))\n    ...> |> select([uv], {uv.first_name, fragment(\"BM25(?)\", uv)})\n    [{\"John\", 1.8}, {\"Bob\", 1.8}]","ref":"ArangoXEcto.View.html#module-ecto-query-search","title":"Ecto Query Search - ArangoXEcto.View","type":"module"},{"doc":"Due to how views function and the possibility of multiple types of returns, the result of a query\ncannot be automatically loaded to a struct with Ecto so you will have to use the `ArangoXEcto.load/2` function.\nIt does still load the values using the field loaders to the correct type, so if you don't need to load\nit into a struct you can skip this.\n\n    iex> Repo.all(MyApp.UsersView) |> ArangoXEcto.load(MyApp.User)\n    [%User{first_name: \"John\", last_name: \"Smith\"}, _]\n\nSince you can have multiple different schemas linked, the `ArangoXEcto.load/2` function supports\npassing multiple module options that will match against the arango `_id` to load the correct module.\n\n    iex> Repo.all(MyApp.UsersView) |> ArangoXEcto.load([MyApp.User, MyApp.Post])\n    [%User{first_name: \"John\", last_name: \"Smith\"}, %Post{name: \"abc\"}]","ref":"ArangoXEcto.View.html#module-loading-results","title":"Loading results - ArangoXEcto.View","type":"module"},{"doc":"","ref":"ArangoXEcto.View.html#t:compression/0","title":"ArangoXEcto.View.compression/0","type":"type"},{"doc":"Defines a link field on the view.\n\nThis can be supplied more than once in a view block to enable multiple link definitions.\nEach link can be of different documents or edges, this is sorted on querying as mentioned at the top of this module.\n\nLinks are defined by the `ArangoXEcto.View.Link` module.","ref":"ArangoXEcto.View.html#link/2","title":"ArangoXEcto.View.link/2","type":"macro"},{"doc":"* `:schema` - the module of the document or edge the link should be created on\n    * `:link` - the link definition, see `ArangoXEcto.View.Link`","ref":"ArangoXEcto.View.html#link/2-parameters","title":"Parameters - ArangoXEcto.View.link/2","type":"macro"},{"doc":"","ref":"ArangoXEcto.View.html#t:name/0","title":"ArangoXEcto.View.name/0","type":"type"},{"doc":"Defines options for view creation.","ref":"ArangoXEcto.View.html#options/1","title":"ArangoXEcto.View.options/1","type":"macro"},{"doc":"The available options are directly from the [ArangoDB View definition](https://www.arangodb.com/docs/stable/arangosearch-views.html).\n\n    * `:consolidationIntervalMsec` - wait at least this many milliseconds between applying consolidationPolicy\n      to consolidate View data store and possibly release space on the filesystem.\n    * `:consolidationPolicy` - the consolidation policy to apply for selecting data store segment merge candidates.\n    * `:commitIntervalMsec` - wait at least this many milliseconds between committing View data store changes\n       and making documents visible to queries.\n    * `:writebufferSizeMax` - maximum memory byte size per writer (segment) before a writer (segment) flush is triggered.\n    * `:writebufferIdle` - maximum number of writers (segments) cached in the pool.\n    * `:writebufferActive` - maximum number of concurrent active writers (segments) that perform a transaction.\n      Other writers (segments) wait till current active writers (segments) finish.\n    * `:cleanupIntervalStep` - waits at least this many commits between removing unused files in its data directory .\n    * `:primarySortCompression` - defines how to compress the primary sort data (introduced in ArangoDB v3.7.0).\n      See `t:ArangoXEcto.View.compression/0`.\n\n      `:lz4` (default) - use LZ4 fast compression\n      `:none` - disable compression to trade space for speed","ref":"ArangoXEcto.View.html#options/1-options","title":"Options - ArangoXEcto.View.options/1","type":"macro"},{"doc":"Defines a primary sort field on the view.\n\nOnly a field name is required but a direction can also be supplied. By default the direction will be ascending (:asc).\n\nThis can be supplied more than once in a view block to enable multiple primary sorts.","ref":"ArangoXEcto.View.html#primary_sort/2","title":"ArangoXEcto.View.primary_sort/2","type":"macro"},{"doc":"* `:field` - atom name of the field to be used as a primary sort\n    * `:direction` - either `:asc` or `:desc` for sort direction","ref":"ArangoXEcto.View.html#primary_sort/2-parameters","title":"Parameters - ArangoXEcto.View.primary_sort/2","type":"macro"},{"doc":"Defines a stored value on the view.\n\nOnly the fields used is required but the compression method can also be supplied. By default the compression will be :lz4.\n\nThis can be supplied more than once in a view block to enable multiple stored value definitions.","ref":"ArangoXEcto.View.html#store_value/2","title":"ArangoXEcto.View.store_value/2","type":"macro"},{"doc":"* `:fields` - a list of atoms for fields for the stored value\n    * `:compression` - either `:none` or `:lz4` for compression, see `t:ArangoXEcto.View.compression/0`","ref":"ArangoXEcto.View.html#store_value/2-parameters","title":"Parameters - ArangoXEcto.View.store_value/2","type":"macro"},{"doc":"","ref":"ArangoXEcto.View.html#t:t/0","title":"ArangoXEcto.View.t/0","type":"type"},{"doc":"Defines a view with a view name and defenitions.\n\nCan only be defined once per view module.","ref":"ArangoXEcto.View.html#view/2","title":"ArangoXEcto.View.view/2","type":"macro"},{"doc":"Defines the structure of a link\n\nThis follows the format of the link properties defined in ArangoDB. For more\ninformation [check out the ArangoDB docs](https://www.arangodb.com/docs/stable/arangosearch-views.html#link-properties).","ref":"ArangoXEcto.View.Link.html","title":"ArangoXEcto.View.Link","type":"module"},{"doc":"%ArangoXEcto.View.Link{\n      includeAllFields: true,\n      fields: %{\n        name: %ArangoXEcto.View.Link{\n          analyzers: [:text_en]\n        }\n      }\n    }","ref":"ArangoXEcto.View.Link.html#module-example","title":"Example - ArangoXEcto.View.Link","type":"module"},{"doc":"","ref":"ArangoXEcto.View.Link.html#t:t/0","title":"ArangoXEcto.View.Link.t/0","type":"type"},{"doc":"Converts to a map for api call\n\nThis will convert any structs into maps in a deeply nested link definition.","ref":"ArangoXEcto.View.Link.html#to_map/1","title":"ArangoXEcto.View.Link.to_map/1","type":"function"},{"doc":"Validates a link\n\nThis checks the types and strucute of a provided link struct.\n\nIf a link is invalid for any reason this will return false. Using this allows for validation before\nsending to the DB.","ref":"ArangoXEcto.View.Link.html#valid?/1","title":"ArangoXEcto.View.Link.valid?/1","type":"function"},{"doc":"Methods for interacting with ArangoDB GeoJSON and geo related functions\n\nThe methods within this module are really just helpers to generate `Geo` structs.","ref":"ArangoXEcto.GeoData.html","title":"ArangoXEcto.GeoData","type":"module"},{"doc":"","ref":"ArangoXEcto.GeoData.html#t:coordinate/0","title":"ArangoXEcto.GeoData.coordinate/0","type":"type"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_coordinate/1","title":"ArangoXEcto.GeoData.is_coordinate/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_latitude/1","title":"ArangoXEcto.GeoData.is_latitude/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_longitude/1","title":"ArangoXEcto.GeoData.is_longitude/1","type":"macro"},{"doc":"Generates a Geo linestring","ref":"ArangoXEcto.GeoData.html#linestring/1","title":"ArangoXEcto.GeoData.linestring/1","type":"function"},{"doc":"Generates a Geo multi linestring","ref":"ArangoXEcto.GeoData.html#multi_linestring/1","title":"ArangoXEcto.GeoData.multi_linestring/1","type":"function"},{"doc":"Generates a Geo multi point","ref":"ArangoXEcto.GeoData.html#multi_point/1","title":"ArangoXEcto.GeoData.multi_point/1","type":"function"},{"doc":"Generates a Geo multi polygon","ref":"ArangoXEcto.GeoData.html#multi_polygon/1","title":"ArangoXEcto.GeoData.multi_polygon/1","type":"function"},{"doc":"Generates a Geo point","ref":"ArangoXEcto.GeoData.html#point/2","title":"ArangoXEcto.GeoData.point/2","type":"function"},{"doc":"Generates a Geo polygon","ref":"ArangoXEcto.GeoData.html#polygon/1","title":"ArangoXEcto.GeoData.polygon/1","type":"function"},{"doc":"Sanitizes coordinates to ensure they are valid\n\nThis function is not automatically applied to Geo constructors and must be applied before hand","ref":"ArangoXEcto.GeoData.html#sanitize/1","title":"ArangoXEcto.GeoData.sanitize/1","type":"function"},{"doc":"Defines a GeoJSON type for use in ecto schemas\n\nThis will handle conversion of a Geo struct into a map for storage in the database.\n\nUse as the type of a field in an Ecto schema. For example:\n\n    schema \"users\" do\n      field :location, ArangoXEcto.Types.GeoJSON\n    end","ref":"ArangoXEcto.Types.GeoJSON.html","title":"ArangoXEcto.Types.GeoJSON","type":"module"},{"doc":"","ref":"ArangoXEcto.Types.GeoJSON.html#embed_as/1","title":"ArangoXEcto.Types.GeoJSON.embed_as/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Types.GeoJSON.html#equal?/2","title":"ArangoXEcto.Types.GeoJSON.equal?/2","type":"function"},{"doc":"Modify the database through static migrations.\n\n> #### NOTE {: .info}\n>\n> ArangoXEcto can dynamically create collections for you and can be enabled by setting `static:\n> false` in your app config. Refer to `ArangoXEcto` for more info. However, this is primarily\nuseful for dev applications and migrations should be used for production level applications.\n\nEach migrations typically have an `up` and `down` function or just a `change` function. If using\nseparate `up` and `down` functions, these will allow for the database to migrated forward or\nrolled back. If using the `change` function then the migrations for going forward are defined and\nautomatically reversed in the case of a rollback.\n\nMigrations that have been executed in a database for ArangoXEcto are stored in a system collection\ncalled `_migrations` (this is a hidden system collection). This allows ArangoXEcto to track what\nhas already been run so that it knows what migrations can be run or rolled back. You can configure\nthe name of this table with the `:migration_source` configuration option and the name of the\nrepository that manages migrations can be configured with `:migration_repo`.","ref":"ArangoXEcto.Migration.html","title":"ArangoXEcto.Migration","type":"behaviour"},{"doc":"You might be wondering what happens when there are two or more applications trying to run\nmigrations on the database at once. If two or more instances of the database try to write to the\ndatabase at the same time this could be a huge problem, causing unpredictable results. A lot of\ndatabases use locking as a mechanism to solve this. I.e. one connection will lock writing to the\ndatabase so others cannot write. ArangoDB doesn't do this, so how can we solve this? \n\nLuckily we don't have to worry about this as ArangoDB handles this for us through dynamic locks.\nEssentially if two connections try to write on the same part of the database then one of them will\njust fail as the first has acquired an exclusive lock.","ref":"ArangoXEcto.Migration.html#module-two-or-more-servers-one-db-oh-no","title":"Two (or more) servers, one DB, oh no!! - ArangoXEcto.Migration","type":"behaviour"},{"doc":"Migration files are stored in the \"priv/MY_REPO/migrations\" (where MY_REPO is a lowercase\nunderscored name of your repository. For example, if you repo module was `MyApp.MyRepo` then the\npath would be \"priv/my_repo/migrations\" or if your repo module was `MyApp.ThisIsTheCoolestRepo`\nthen your migrations would be in \"priv/this_is_the_coolest_repo/migrations\". The directory can\nbe overridden when running the mix tasks or in the configuration.\n\nEach migration filename will have a timestamp number followed by a unique name seperated by an\nunderscore. For example if you wanted to add a users collection could put the migration in a file\nat \"priv/repo/migrations/20240928200000_add_users_collection.exs, with contents:\n\n    defmodule Repo.Migrations.AddUsersCollection do\n      use ArangoXEcto.Migration\n\n      def up do\n        create collection(:users) do\n          add :first_name, :string, comment: \"first_name column\"\n          add :last_name, :string\n          add :gender, :integer\n          add :age, :integer\n          add :location, :map\n\n          timestamps()\n        end\n      end\n\n      def down do\n        drop collection(\"users\")\n      end\n    end\n\nThe `up/0` function is responsible for migrating the database forward, while the `down/0` function\nis responsible for rolling back this migration. The `down/0` function must always reverse the\n`up/0` action. Inside both of these functions we use various API methods defined in this module to\nperform actions on the database. We can manage collections, indexes, views, analyzers and run\ncustom AQL commands.\n\nNow that our migration is setup, all that is left is to execute it. While migrations can be run\nusing code (and there are valid use cases for this, e.g. generating releases for production) we\ngenerally will use mix tasks. To do this run the following from the root directory of your\nproject:\n\n    $ mix arango.migrate\n\nIf we wanted to reverse the migration we could run:\n\n    $ mix arango.rollback -n 1\n\nWe must always specify how many migrations to rollback. Whereas `mix arango.migrate` will always\nrun all pending migrations\n\nThere is one other mix task that is used regularly and it saves us a lot of time. That is the `mix\narango.gen.migration` task. This task will generate a blank migration with boilerplate contents\nand use the proper timestamps of when it was generated.\n\n    $ mix arango.gen.migration add_users_collection\n\nYou can now generate and run migrations successfully. You may be pretty proud of yourself, and\nyou should be, but there is some more parts to it. You can learn all about migrations in\nArangoXEcto in this module documentation.\n\n> #### IMPORTANT {: .warning}\n> **Order matters!!** Make sure you create collections before indexes and views, and analyzers \n> before views if they are used. In general this is a good order to follow:\n>\n>    Analyzers > Collections > Indexes > Views","ref":"ArangoXEcto.Migration.html#module-migration-creation","title":"Migration creation - ArangoXEcto.Migration","type":"behaviour"},{"doc":"ArangoDB can take a schema, but it is not strictly required, hence no fields _need_ to be \nprovided. The only required thing is the collection name. More info below on how to define\nfields. For example, the example provided in the previous section can be provided without defining\nthe fields.\n\n    defmodule Repo.Migrations.AddUsersCollection do\n      use ArangoXEcto.Migration\n\n      def up do\n        create collection(:users)\n      end\n\n      def down do\n        drop collection(\"users\")\n      end\n    end\n\nThis will work perfectly fine, however there is one caveat, documents in the collection will not\nbe validated. This means that the migration will only create the collection and not define a\nJSONSchema for the collection. Schemas a generated using the `ArangoXEcto.Migration.JsonSchema`\nmodule, however all documentation on how to use it can be found in this module.","ref":"ArangoXEcto.Migration.html#module-arangodb-schemas","title":"ArangoDB Schemas - ArangoXEcto.Migration","type":"behaviour"},{"doc":"As seen in the example above, ArangoXEcto has a few built in mix tasks that can be used to help\nwith the development workflow. These tasks are similar to the ones found in `ecto_sql` but are\nadapted for the `ArangoXEcto` context. To allow for using multiple database adapters together they\nfollow a similar but slightly different pattern, as seen below.\n\n  * `mix arango.gen.migration` - generates a migration that the user can fill in with particular\n    commands\n  * `mix arango.migrate` - migrates a repository\n  * `mix arango.rollback` - rolls back a particular migration\n\nFor additional help on how to use these tasks you can run `mix help COMMAND` where `COMMAND` is\nthe command you need assistance with.\n\nAs mentioned previously, you can also run migrations using code using this module. If you think\nyou're pretty cool and know what you're doing you can also use the lower level APIs in\n`ArangoXEcto.Migrator`.","ref":"ArangoXEcto.Migration.html#module-mix-tasks","title":"Mix tasks - ArangoXEcto.Migration","type":"behaviour"},{"doc":"I know what you're thinking, \"Ugh, do I really have to write an `up/0` and `down/0` function for\nevery single migration? Surely you could have just written some code the generate the `down/0`\nversion for me\". Well yes, yes I did. That's what `change/0` is for. You just write your regular\n`up/0` action and it will do some fancy footwork and figure out what the `down/0` version would\nbe. For example, this would operate the same as seen above:\n\n    defmodule Repo.Migrations.AddUsersCollection do\n      use ArangoXEcto.Migration\n\n      def change do\n        create collection(:users) do\n          add :first_name, :string, comment: \"first_name column\"\n          add :last_name, :string\n          add :gender, :integer\n          add :age, :integer\n          add :location, :map\n\n          timestamps()\n        end\n      end\n    end\n\nLike a good salesman I only gave you half the story, there is one caveat when using this. Not all\ncommands are reversible and will raise an `Ecto.MigrationError`.\n\nA great example of a command that cannot be reversed is `execute/1` but can easily be made\nreversible by calling `execute/2` instead. The first argument will be run on `up/0` and the second\nwill be run on `down/0`.\n\nIf you implement `up/0` and `down/0` they will take precedence over `change/0` which won't be\ninvoked.","ref":"ArangoXEcto.Migration.html#module-change-combining-up-and-down","title":"Change (combining up and down) - ArangoXEcto.Migration","type":"behaviour"},{"doc":"The field types provided must match one of the types available in\n`ArangoXEcto.Migration.JsonSchema`. Any other option will raise an error. There is one caveat to\nthis, the `sub_command` option is used through `add_embed` and `add_embed_many` (essentially a\nshortcut for an array of many embeds).\n\nThe options that can be passed to each of the types can be found in the\n`ArangoXEcto.Migration.JsonSchema` docs.","ref":"ArangoXEcto.Migration.html#module-field-types","title":"Field Types - ArangoXEcto.Migration","type":"behaviour"},{"doc":"Most functions won't be executed until the end of the relevant `up`, `change` or `down` callback.\nHowever, if you call any `Ecto.Repo` function they will be executed immediately, unless called\ninside an anonymous function passed to `execute/1`.\n\nYou may want to ensure that all the previous steps have been executed before continuing. To do\nthis you can call the `flush/0` function to execute everything before.\n\nHowever `flush/0` will raise if it would be called from `change` function when doing a rollback.\nTo avoid that we recommend to use `execute/2` with anonymous functions instead.\nFor more information and example usage please take a look at `execute/2` function.","ref":"ArangoXEcto.Migration.html#module-executing-and-flushing","title":"Executing and flushing - ArangoXEcto.Migration","type":"behaviour"},{"doc":"","ref":"ArangoXEcto.Migration.html#module-repo-configuration","title":"Repo configuration - ArangoXEcto.Migration","type":"behaviour"},{"doc":"The following options are not required but can be used to adjust how your migrations are stored\nand run.\n\n  * `:migration_source` - The name of the collection to store the migrations that have already\n    been run. Only the version numbers (timestamps) are stored. The default is `_migrations`.\n    Whatever is specified it will be set as a system collection in ArangoDB. The convention is to\n    prefix the collection name with an underscore (_). You can configure this as follows:\n\n        config :my_app, MyApp.Repo, migration_source: \"_custom_migrations\"\n\n  * `:migration_repo` - This configures which repository the migrations will be stored in. By\n    default this will be the given repository but it can be overridden. A possible use case of\n    this is if you wanted all migrations to be stored in a particular database in ArangoDB.\n\n        config :my_app, MyApp.Repo, migration_repo: MyApp.MigrationRepo\n\n  * `:priv` - the location where to store important assets, such as migrations. By default this\n    will be \"priv/my_reoo\" for a repository module of `MyApp.MyRepo` and migrations would be\n    placed in \"priv/my_repo/migrations\".\n\n  * `:start_apps_before_migration` - A list of applications to be started before migrations are\n    run. Used by `ArangoXEcto.Migrator.with_repo/3` and hence the migration task.\n\n        config :my_app, MyApp.Repo, start_apps_before_migration: [:ssl, :some_custom_logger]","ref":"ArangoXEcto.Migration.html#module-migrator-configuration","title":"Migrator configuration - ArangoXEcto.Migration","type":"behaviour"},{"doc":"The previous section's options were focused on how migrations are run and stored, whereas the\nfollowing options are focused on adjusting what values are stored. If you change these after your\napp goes into production it will cause unexpected behaviour.\n\n  * `:migration_timestamps` - Modifies the type and field names of `:inserted_at` and\n    `:updated_at`. By default timestamps will be stored as `:naive_datetime` but this, and the\n    field names can be configured as seen below:\n\n        config :my_app, MyApp.Repo, migration_timestamps: [\n          type: :utc_datetime,\n          inserted_at: :created_at,\n          updated_at: :changed_at\n        ]\n\n  * `:migration_default_prefix` - By default no prefix is used, but you might want to. You can\n    configure it using this option. This will be overriden by a prefix being passed at the command\n    level.\n\n        config :my_app, MyApp.Repo, migration_default_prefix: \"my_migrations_prefix\"","ref":"ArangoXEcto.Migration.html#module-migrations-configuration","title":"Migrations configuration - ArangoXEcto.Migration","type":"behaviour"},{"doc":"The schema stored in ArangoDB supports storing comments on the fields. You can specify this by\npassing the `:comment` option when using the `add/3`, `add_embed/3`, `add_embed_many/3` and the\nmodify equivalents.\n\n      def up do\n        create collection(:users) do\n          add :name, :string, comment: \"full name column\"\n          add_embed :facts, comment: \"my facts\" do\n            add :statement, :string, comment: \"represents the fact statement\"\n            add :falsehood, :boolean, comment: \"is the fact false?\"\n          end\n\n          timestamps()\n        end\n      end","ref":"ArangoXEcto.Migration.html#module-comments","title":"Comments - ArangoXEcto.Migration","type":"behaviour"},{"doc":"ArangoXEcto fully supports prefixes, which includes in migrations. Unlike in a database provider\nlike PostgreSQL, ArangoDB doesn't have the notion of prefixes, hence prefixes actually create a\nseparate database to create the separation.\n\n      def up do\n        create collection(:users, prefix: \"base_app\") do\n          add :first_name, :string, comment: \"first_name column\"\n          add :last_name, :string\n          add :gender, :integer\n          add :age, :integer\n          add :location, :map\n\n          timestamps()\n        end\n\n        create index(:users, [:age], prefix: \"base_app\")\n      end\n\nNotice here how the same prefix must be specified on both the collection creation and the index\ncreation. If you don't do this then they simply won't be created in the same database and will\nlikely result in an error. You can specify a default prefix, as mentioned above, using the\n`:migration_default_prefix` in your configuration.","ref":"ArangoXEcto.Migration.html#module-prefixes","title":"Prefixes - ArangoXEcto.Migration","type":"behaviour"},{"doc":"Migrations are run in transactions. You may need to perform some actions after beginning a\ntransaction or before committing the migration. You can do this with the `c:after_begin/0` and\n`c:before_commit/0` callbacks to your migration.\n\nSometimes you may want to use these callbacks for every migration and doing so can be quite\nrepetitive. You can solve this by implementing your own migration module that extends the\n`ArangoXEcto.Migration` module:\n\n    defmodule MyApp.Migration do\n      defmacro __using__(_) do\n        quote do\n          use ArangoXEcto.Migration\n\n          def after_begin() do\n            repo().query! \"SOME ARANGO QUERY\"\n          end\n        end\n      end\n    end\n\nThen in your migrations you can replace `use ArangoXEcto.Migration` with `use MyApp.Migration`.","ref":"ArangoXEcto.Migration.html#module-transaction-callbacks","title":"Transaction Callbacks - ArangoXEcto.Migration","type":"behaviour"},{"doc":"defmodule MyProject.Repo.Migrations.CreateUsers do\n      use ArangoXEcto.Migration\n\n      def change do\n        create analyzer(:norm_en, :norm, [:frequency, :position], %{\n           locale: \"en\",\n           accent: false,\n           case: :lower\n         })\n\n        create collection(:users) do\n          add :first_name, :string, comment: \"first_name column\"\n          add :last_name, :string\n          add :gender, :integer\n          add :age, :integer\n          add :location, :map\n\n          timestamps()\n        end\n\n        create index(:users, [:age])\n        create unique_index(:users, [:location])\n\n        create view(:user_search, commitIntervalMsec: 1, consolidationIntervalMsec: 1) do\n          add_sort(:created_at, :desc)\n          add_sort(:first_name)\n\n          add_store([:first_name, :last_name], :none)\n\n          add_link(\"users\", %Link{\n            includeAllFields: true,\n            fields: %{\n              last_name: %Link{\n                analyzers: [:identity, :norm_en]\n              }\n            }\n          })\n        end\n      end\n    end","ref":"ArangoXEcto.Migration.html#module-example","title":"Example - ArangoXEcto.Migration","type":"behaviour"},{"doc":"Adds a field when creating or altering a collection.\n\nThis function accepts certain JSON Schema types and depending on the type it will accept certain\noptions. The available types and options can be found in the\n`ArangoXEcto.Migration.JsonSchema.convert/2` function.","ref":"ArangoXEcto.Migration.html#add/3","title":"ArangoXEcto.Migration.add/3","type":"function"},{"doc":"create collection(\"posts\") do\n      add :title, :string, comment: \"Some comment\"\n    end\n\n    alter collection(\"posts\") do\n      add :summary, :string\n      add :object, :map\n      add :age, :integer, min: 18, max: 99\n    end","ref":"ArangoXEcto.Migration.html#add/3-examples","title":"Examples - ArangoXEcto.Migration.add/3","type":"function"},{"doc":"Adds a field when creating or altering a collection with subfields.\n\nThis makes the field type an object in the JSON Schema with the sub fields and options. See\n`add/3` for available options for an object.","ref":"ArangoXEcto.Migration.html#add_embed/3","title":"ArangoXEcto.Migration.add_embed/3","type":"macro"},{"doc":"add_embed :fields do\n      add :name, :string\n      add :type, :string\n    end\n\n    add_embed :likes, comment: \"my likes\" do\n      add :type, :string\n      add :number, :integer\n    end","ref":"ArangoXEcto.Migration.html#add_embed/3-example","title":"Example - ArangoXEcto.Migration.add_embed/3","type":"macro"},{"doc":"Adds a field that has multiple instances of the object.\n\nThis represents a list of objects. This essentially wraps `add_embed/3`'s JSON Schema output with\nan array so that multiple instances of the object is represented.","ref":"ArangoXEcto.Migration.html#add_embed_many/3","title":"ArangoXEcto.Migration.add_embed_many/3","type":"macro"},{"doc":"Adds a field if it does not exist yet when altering a collection.\n\nThis is identical to `add/3` except only creates it if it hasn't been already.\n\nThis is not reversible as existence can't be known beforehand.","ref":"ArangoXEcto.Migration.html#add_if_not_exists/3","title":"ArangoXEcto.Migration.add_if_not_exists/3","type":"function"},{"doc":"alter collection(\"posts\") do\n      add_if_not_exists :title, :string, comment: \"Some comment\"\n    end","ref":"ArangoXEcto.Migration.html#add_if_not_exists/3-examples","title":"Examples - ArangoXEcto.Migration.add_if_not_exists/3","type":"function"},{"doc":"Adds a link to a view.\n\nUses a `ArangoXEcto.View.Link` struct to define the link. See the module for more information.","ref":"ArangoXEcto.Migration.html#add_link/2","title":"ArangoXEcto.Migration.add_link/2","type":"function"},{"doc":"link \"users\", %Link{\n      includeAllFields: true,\n      fields: %{\n        name: %Link{\n          analyzers: [:text_en]\n        }\n      }\n    }","ref":"ArangoXEcto.Migration.html#add_link/2-example","title":"Example - ArangoXEcto.Migration.add_link/2","type":"function"},{"doc":"Adds a primary sort to a view.\n\nThis adds a sort to the view. TODO: add a description of it.\n\nThis accepts a field name and a sort direction (either `:asc` or `:desc`), defaults to `:asc`.","ref":"ArangoXEcto.Migration.html#add_sort/2","title":"ArangoXEcto.Migration.add_sort/2","type":"function"},{"doc":"Adds a stored value to a view.\n\nThis adds a stored value to the view. TODO: add a description of it.\n\nThis accepts a field name and a compression (either `:lz4` or `:none`), defaults to `:lz4`.","ref":"ArangoXEcto.Migration.html#add_store/2","title":"ArangoXEcto.Migration.add_store/2","type":"function"},{"doc":"Migration code to run immediately after the transaction is opened.\n\nKeep in mind that it is treated like any normal migration code, and should\nconsider both the up *and* down cases of the migration.","ref":"ArangoXEcto.Migration.html#c:after_begin/0","title":"ArangoXEcto.Migration.after_begin/0","type":"callback"},{"doc":"Alters a collection or view.","ref":"ArangoXEcto.Migration.html#alter/2","title":"ArangoXEcto.Migration.alter/2","type":"macro"},{"doc":"alter collection(:users) do\n      add :middle_name, :string\n      modify :people, :integer\n      rename :people, to: :num\n      remove :last_name\n    end","ref":"ArangoXEcto.Migration.html#alter/2-collection-examples","title":"Collection Examples - ArangoXEcto.Migration.alter/2","type":"macro"},{"doc":"alter view(:user_search) do\n      add_link(\"new_test\", %Link{\n        includeAllFields: true,\n        fields: %{\n          last_name: %Link{\n            analyzers: [:text_en]\n          }\n        }\n      })\n\n      remove_link(\"new_test\")\n    end","ref":"ArangoXEcto.Migration.html#alter/2-view-example","title":"View Example - ArangoXEcto.Migration.alter/2","type":"macro"},{"doc":"Represents an analyzer for deletion.\n\nCreates an analyzer struct with only a name so that it can be used to delete an analyzer.","ref":"ArangoXEcto.Migration.html#analyzer/1","title":"ArangoXEcto.Migration.analyzer/1","type":"function"},{"doc":"Creates an analyzer struct that can be passed to an action function.","ref":"ArangoXEcto.Migration.html#analyzer/5","title":"ArangoXEcto.Migration.analyzer/5","type":"function"},{"doc":"* `:name` - the name of the analyzer to be created\n  * `:type` - the type of the analyzer, available options are\n    `t:ArangoXEcto.Migration.Analyzer.type/0`.\n  * `:features` - a list of enabled features, available options are\n    `t:ArangoXEcto.Migration.Analyzer.feature/0`.\n  * `:properties` - additional options for the analyzer, dependant on the type. Refer to the\n    `ArangoDB` docs. TODO: add link.","ref":"ArangoXEcto.Migration.html#analyzer/5-parameters","title":"Parameters - ArangoXEcto.Migration.analyzer/5","type":"function"},{"doc":"Migration code to run immediately before the transaction is closed.\n\nKeep in mind that it is treated like any normal migration code, and should\nconsider both the up *and* down cases of the migration.","ref":"ArangoXEcto.Migration.html#c:before_commit/0","title":"ArangoXEcto.Migration.before_commit/0","type":"callback"},{"doc":"Creates a collection struct that can be provided to one of the action functions.","ref":"ArangoXEcto.Migration.html#collection/2","title":"ArangoXEcto.Migration.collection/2","type":"function"},{"doc":"Accepts an options parameter as the third argument. For available keys please refer to the\n[ArangoDB API doc](https://www.arangodb.com/docs/stable/http/collection-creating.html).\nThe following options differ slightly:\n\n  * `:type` - accepts either `:document` or `:edge`. Default is `:document`.\n  * `:prefix` - the prefix for the collection. This will be the prefix database used for the\n    collection.","ref":"ArangoXEcto.Migration.html#collection/2-options","title":"Options - ArangoXEcto.Migration.collection/2","type":"function"},{"doc":"iex> collection(\"users\")\n    %ArangoXEcto.Migration.Collection{name: \"users\", type: 2)\n\n    iex> collection(\"users\", type: :edge)\n    %ArangoXEcto.Migration.Collection{name: \"users\", type: 3)\n\n    iex> collection(\"users\", keyOptions: %{type: :uuid})\n    %ArangoXEcto.Migration.Collection{name: \"users\", type: 2, keyOptions: %{type: :uuid})","ref":"ArangoXEcto.Migration.html#collection/2-examples","title":"Examples - ArangoXEcto.Migration.collection/2","type":"function"},{"doc":"Creates one of the following: \n\n  * an index\n  * a collection with no schema\n  * an analyzer\n\nWhen reversing (in a `change/0` running backwards), objects are only dropped\nif they exist, and no errors are raised. To enforce dropping an object, use\n`drop/1`.","ref":"ArangoXEcto.Migration.html#create/1","title":"ArangoXEcto.Migration.create/1","type":"function"},{"doc":"create index(\"users\", [:name])\n    create collection(\"posts\")\n    create analyzer(:norm_en, :norm, [:frequency, :position], %{\n        locale: \"en\",\n        accent: false,\n        case: :lower\n      })","ref":"ArangoXEcto.Migration.html#create/1-examples","title":"Examples - ArangoXEcto.Migration.create/1","type":"function"},{"doc":"Creates a collection or view.","ref":"ArangoXEcto.Migration.html#create/2","title":"ArangoXEcto.Migration.create/2","type":"macro"},{"doc":"create collection(:users) do\n      add :first_name, :string\n      add :last_name, :string, default: \"Smith\"\n\n      timestamps()\n    end","ref":"ArangoXEcto.Migration.html#create/2-collection-example","title":"Collection Example - ArangoXEcto.Migration.create/2","type":"macro"},{"doc":"create view(:user_search) do\n      add_sort(:created_at, :desc)\n      add_sort(:first_name)\n\n      add_store([:first_name, :last_name], :none)\n\n      add_link(\"users\", %Link{\n        includeAllFields: true,\n        fields: %{\n          last_name: %Link{\n            analyzers: [:identity, :text_en]\n          }\n        }\n      })\n    end","ref":"ArangoXEcto.Migration.html#create/2-view-example","title":"View Example - ArangoXEcto.Migration.create/2","type":"macro"},{"doc":"Same as `create/1` except it will only create the object if it doesn't exist already.","ref":"ArangoXEcto.Migration.html#create_if_not_exists/1","title":"ArangoXEcto.Migration.create_if_not_exists/1","type":"function"},{"doc":"create_if_not_exists index(\"users\", [:name])\n\n    create_if_not_exists collection(\"posts\")\n\n    create_if_not_exists analyzer(:norm_en, :norm, [:frequency, :position], %{\n        locale: \"en\",\n        accent: false,\n        case: :lower\n      })","ref":"ArangoXEcto.Migration.html#create_if_not_exists/1-examples","title":"Examples - ArangoXEcto.Migration.create_if_not_exists/1","type":"function"},{"doc":"Creates a collection or view if it doesn't exist.\n\nWorks just the same as `create/2` but will raise an error\nwhen the object already exists.","ref":"ArangoXEcto.Migration.html#create_if_not_exists/2","title":"ArangoXEcto.Migration.create_if_not_exists/2","type":"macro"},{"doc":"Gets the migrator direction.","ref":"ArangoXEcto.Migration.html#direction/0","title":"ArangoXEcto.Migration.direction/0","type":"function"},{"doc":"Drops one of the following: \n\n  * an index\n  * a collection\n  * a view\n  * an analyzer","ref":"ArangoXEcto.Migration.html#drop/1","title":"ArangoXEcto.Migration.drop/1","type":"function"},{"doc":"drop index(\"users\", [:name])\n    drop collection(\"posts\")\n    drop view(\"users_search\")\n    drop analyzer(\"users_search\")","ref":"ArangoXEcto.Migration.html#drop/1-examples","title":"Examples - ArangoXEcto.Migration.drop/1","type":"function"},{"doc":"Same as `drop/1` except only drops if it exists.\n\nDoes not raise an error if the specified collection or index does not exist.","ref":"ArangoXEcto.Migration.html#drop_if_exists/1","title":"ArangoXEcto.Migration.drop_if_exists/1","type":"function"},{"doc":"drop_if_exists index(\"users\", [:name])\n    drop_if_exists collection(\"posts\")\n    drop_if_exists view(\"users_search\")\n    drop_if_exists analyzer(\"users_search\")","ref":"ArangoXEcto.Migration.html#drop_if_exists/1-examples","title":"Examples - ArangoXEcto.Migration.drop_if_exists/1","type":"function"},{"doc":"Creates an edge collection struct\n\nSame as passing `:edge` as the type to `collection/3`.","ref":"ArangoXEcto.Migration.html#edge/2","title":"ArangoXEcto.Migration.edge/2","type":"function"},{"doc":"Executes arbitrary AQL.\n\nThe argument is typically a string, containing the AQL command to be executed.\n\nYou can also run arbitrary code as part of your migrations by supplying an anonymous function.\nThis is advantageous as it defers the execution of the anonymous function until after the\nmigration callback is terminated (see [Executing and flushing](#module-executing-and-flushing)).\n\nReversible commands can be defined by calling `execute/2`.","ref":"ArangoXEcto.Migration.html#execute/1","title":"ArangoXEcto.Migration.execute/1","type":"function"},{"doc":"execute \"FOR u IN `users` RETURN u.name\"\n\n    execute(fn -> repo().update_all(\"posts\", set: [published: true]) end)","ref":"ArangoXEcto.Migration.html#execute/1-examples","title":"Examples - ArangoXEcto.Migration.execute/1","type":"function"},{"doc":"Executes reversible AQL commands.\n\nThis is useful for database-specific functionality that does not\nwarrant special support in ArangoXEcto. The `execute/2` form avoids having\nhaving to define separate `up/0` and `down/0` blocks that each contain an `execute/1`\nexpression.\n\nThe allowed parameters are explained in `execute/1`.","ref":"ArangoXEcto.Migration.html#execute/2","title":"ArangoXEcto.Migration.execute/2","type":"function"},{"doc":"defmodule MyApp.MyMigration do\n      use ArangoXEcto.Migration\n\n      def change do\n        execute \"FOR u IN `users` RETURN u\", \"FOR u IN `users` RETURN u\"\n        execute(&execute_up/0, &execute_down/0)\n      end\n\n      defp execute_up, do: repo().query!(\"'Up query …';\", [], [log: :info])\n      defp execute_down, do: repo().query!(\"'Down query …';\", [], [log: :info])\n    end","ref":"ArangoXEcto.Migration.html#execute/2-examples","title":"Examples - ArangoXEcto.Migration.execute/2","type":"function"},{"doc":"Executes queued migration commands","ref":"ArangoXEcto.Migration.html#flush/0","title":"ArangoXEcto.Migration.flush/0","type":"macro"},{"doc":"Creates an index struct that can be passed to an action function.\n\nDefault index type is a hash. To change this pass the `:type` option in options.\n\nThis will generate a name for the index if not provided. This allows for the dropping\nof the index.","ref":"ArangoXEcto.Migration.html#index/3","title":"ArangoXEcto.Migration.index/3","type":"function"},{"doc":"Options only apply to the creation of indexes and has no effect when using the `drop/1` function.\n\n  * `:name` - The name of the index (usefull for phoenix constraints). Defaults to\n  \"idx*_* *_* \".\n  * `:prefix` - the prefix for the collection. This will be the prefix database used for the\n      index collection.\n  * `:type` - The type of index to create\n    * Accepts: `:fulltext`, `:geo`, `:hash`, `:persistent`, `:skiplist` or `:ttl`\n  * `:unique` - If the index should be unique, defaults to false (hash, persistent & skiplist only)\n  * `:sparse` - If index should be spares, defaults to false (hash, persistent & skiplist only)\n  * `:deduplication` - If duplication of array values should be turned off, defaults to true (hash & skiplist only)\n  * `:minLength` - Minimum character length of words to index (fulltext only)\n  * `:geoJson` -  If a geo-spatial index on a location is constructed and geoJson is true, then the order\nwithin the array is longitude followed by latitude (geo only)\n  * `:expireAfter` - Time in seconds after a document's creation it should count as `expired` (ttl only)","ref":"ArangoXEcto.Migration.html#index/3-options","title":"Options - ArangoXEcto.Migration.index/3","type":"function"},{"doc":"Create index on email field\n\n    iex> index(\"users\", [:email])\n    %ArangoXEcto.Migration.Index{collection_name: \"users\", fields: [:email]}\n\nCreate dual index on email and ph_number fields with a specified name\n\n    iex> index(\"users\", [:email, :ph_number], name: \"my_cool_index\")\n    %ArangoXEcto.Migration.Index{collection_name: \"users\", fields: [:email, :ph_number], name:\n    \"my_cool_index\"}\n\nCreate unique email index\n\n    iex> index(\"users\", [:email], unique: true)\n    %ArangoXEcto.Migration.Index{collection_name: \"users\", fields: [:email], unique: true}\n\nCreate a geo index\n\n    iex> index(\"users\", [:coordinates], type: :geo)\n    %ArangoXEcto.Migration.Index{collection_name: \"users\", fields: [:coordinates], type: :geo}","ref":"ArangoXEcto.Migration.html#index/3-examples","title":"Examples - ArangoXEcto.Migration.index/3","type":"function"},{"doc":"Modifies the type of a field when altering a collection.\n\nThis command is not reversible unless the `:from` option is provided. You want to specify all the\nnecessary options and types so that it can be rolled back properly.\n\nSee `add/3` for more information on supported types.","ref":"ArangoXEcto.Migration.html#modify/3","title":"ArangoXEcto.Migration.modify/3","type":"function"},{"doc":"alter collection(\"users\") do\n      modify :name, :string\n    end\n\n    # Self rollback when using the :from option\n    alter collection(\"users\") do\n      modify :name, :string, from: :integer\n    end\n\n    # Modify field with rollback options\n    alter collection(\"users\") do\n      modify :name, :string, null: true, from: {:integer, null: false}\n    end","ref":"ArangoXEcto.Migration.html#modify/3-examples","title":"Examples - ArangoXEcto.Migration.modify/3","type":"function"},{"doc":"Options are the same as `add/3` but with the additional following option.\n\n  * `:from` - specifies the current type and options of the field.","ref":"ArangoXEcto.Migration.html#modify/3-options","title":"Options - ArangoXEcto.Migration.modify/3","type":"function"},{"doc":"Modifies a field when creating or altering a collection with subfields.\n\nSee `modify/3` for options and more info.","ref":"ArangoXEcto.Migration.html#modify_embed/3","title":"ArangoXEcto.Migration.modify_embed/3","type":"macro"},{"doc":"Modifies a field when creating or altering many objects with subfields.\n\nSee `modify_embed/3` for options and more info.","ref":"ArangoXEcto.Migration.html#modify_embed_many/3","title":"ArangoXEcto.Migration.modify_embed_many/3","type":"macro"},{"doc":"Gets the migrator prefix.","ref":"ArangoXEcto.Migration.html#prefix/0","title":"ArangoXEcto.Migration.prefix/0","type":"function"},{"doc":"Removes a field when altering a collection.\n\nIf it doesn't exist it will simply be ignored.\n\nThis command is not reversible as Ecto does not know what type it should add\nthe field back as. See `remove/3` as a reversible alternative.","ref":"ArangoXEcto.Migration.html#remove/1","title":"ArangoXEcto.Migration.remove/1","type":"function"},{"doc":"alter collection(\"users\") do\n      remove :name\n    end","ref":"ArangoXEcto.Migration.html#remove/1-examples","title":"Examples - ArangoXEcto.Migration.remove/1","type":"function"},{"doc":"Removes a field in a reversible way when altering a collection.\n\n`type` and `opts` are exactly the same as in `add/3`, and\nthey are used when the command is reversed.","ref":"ArangoXEcto.Migration.html#remove/3","title":"ArangoXEcto.Migration.remove/3","type":"function"},{"doc":"alter collection(\"users\") do\n      remove :name, :string, min_length: 4\n    end","ref":"ArangoXEcto.Migration.html#remove/3-examples","title":"Examples - ArangoXEcto.Migration.remove/3","type":"function"},{"doc":"Removes a field only if it exists in the schema\n\n`type` and `opts` are exactly the same as in `add/3`, and\nthey are used when the command is reversed.","ref":"ArangoXEcto.Migration.html#remove_if_exists/3","title":"ArangoXEcto.Migration.remove_if_exists/3","type":"function"},{"doc":"alter collection(\"users\") do\n      remove_if_exists :name, :string, min_length: 4\n    end","ref":"ArangoXEcto.Migration.html#remove_if_exists/3-examples","title":"Examples - ArangoXEcto.Migration.remove_if_exists/3","type":"function"},{"doc":"Removes a link when altering a view.\n\nIf it doesn't exist it will simply be ignored.\n\nThis command is not reversible as Ecto does not know what type it should add\nthe field back as. See `remove_link/2` as a reversible alternative.","ref":"ArangoXEcto.Migration.html#remove_link/1","title":"ArangoXEcto.Migration.remove_link/1","type":"function"},{"doc":"alter view(\"user_search\") do\n      remove_link \"users\"\n    end","ref":"ArangoXEcto.Migration.html#remove_link/1-examples","title":"Examples - ArangoXEcto.Migration.remove_link/1","type":"function"},{"doc":"Removes a link in a reversable way when altering a view.\n\n`link` is the same as in `add_link/2`.","ref":"ArangoXEcto.Migration.html#remove_link/2","title":"ArangoXEcto.Migration.remove_link/2","type":"function"},{"doc":"alter view(\"user_search\") do\n      remove_link \"users\", %Link{\n        includeAllFields: true,\n        fields: %{\n          last_name: %Link{\n            analyzers: [:text_en]\n          }\n        }\n      }\n    end","ref":"ArangoXEcto.Migration.html#remove_link/2-examples","title":"Examples - ArangoXEcto.Migration.remove_link/2","type":"function"},{"doc":"Renames a collection, view or a collection field","ref":"ArangoXEcto.Migration.html#rename/2","title":"ArangoXEcto.Migration.rename/2","type":"function"},{"doc":"# rename a collection\n    rename collection(\"users\"), to: collection(\"new_users\")\n\n    # rename a view\n    rename view(\"user_search\"), to: view(\"new_user_search\")\n\n    alter collection(\"users\") do\n      rename :name, to: :first_name\n    end","ref":"ArangoXEcto.Migration.html#rename/2-examples","title":"Examples - ArangoXEcto.Migration.rename/2","type":"function"},{"doc":"Gets the migrator repo.","ref":"ArangoXEcto.Migration.html#repo/0","title":"ArangoXEcto.Migration.repo/0","type":"function"},{"doc":"Adds `:inserted_at` and `:updated_at` timestamp fields.\n\nThose fields are of `:naive_datetime` type and by default cannot be null. A\nlist of `opts` can be given to customize the generated fields.\n\nFollowing options will override the repo configuration specified by\n`:migration_timestamps` option.","ref":"ArangoXEcto.Migration.html#timestamps/1","title":"ArangoXEcto.Migration.timestamps/1","type":"function"},{"doc":"* `:inserted_at` - the name of the column for storing insertion times.\n    Setting it to `false` disables the column.\n  * `:updated_at` - the name of the column for storing last-updated-at times.\n    Setting it to `false` disables the column.\n  * `:type` - the type of the `:inserted_at` and `:updated_at` columns.\n    Defaults to `:naive_datetime`.\n\n  The rest of the options are passed to the fields","ref":"ArangoXEcto.Migration.html#timestamps/1-options","title":"Options - ArangoXEcto.Migration.timestamps/1","type":"function"},{"doc":"Shortcut for creating a unique index.\n\nSame as passing :unique as true to `index/3`. See `index/3` for more information.","ref":"ArangoXEcto.Migration.html#unique_index/3","title":"ArangoXEcto.Migration.unique_index/3","type":"function"},{"doc":"Creates a view struct that can be passed to an action function.\n\nThis is similar to the `collection/2` function except its representation is for a view.","ref":"ArangoXEcto.Migration.html#view/2","title":"ArangoXEcto.Migration.view/2","type":"function"},{"doc":"TODO: Add note about reading the docs for options.\n\n  * `:prefix` - the prefix for the view. This will be the prefix database used for the\n      Arango search view.","ref":"ArangoXEcto.Migration.html#view/2-options","title":"Options - ArangoXEcto.Migration.view/2","type":"function"},{"doc":"iex> view(\"users_search\")\n    %ArangoXEcto.Migration.View{name: \"users_search\")\n\n    iex> view(\"users_search\", primarySortCompression: :none)\n    %ArangoXEcto.Migration.View{name: \"users_search\", primarySortCompression: :none)","ref":"ArangoXEcto.Migration.html#view/2-examples","title":"Examples - ArangoXEcto.Migration.view/2","type":"function"},{"doc":"Used internally by the `ArangoXEcto` migration.\n\nTo define an analyzer in a migration, see `ArangoXEcto.Migration.analyzer/4`.","ref":"ArangoXEcto.Migration.Analyzer.html","title":"ArangoXEcto.Migration.Analyzer","type":"module"},{"doc":"","ref":"ArangoXEcto.Migration.Analyzer.html#t:feature/0","title":"ArangoXEcto.Migration.Analyzer.feature/0","type":"type"},{"doc":"Creates a new Analyzer struct","ref":"ArangoXEcto.Migration.Analyzer.html#new/5","title":"ArangoXEcto.Migration.Analyzer.new/5","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Analyzer.html#t:t/0","title":"ArangoXEcto.Migration.Analyzer.t/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.Analyzer.html#t:type/0","title":"ArangoXEcto.Migration.Analyzer.type/0","type":"type"},{"doc":"Used internally by the `ArangoXEcto` migration.\n\nTo define a collection in a migration, see `ArangoXEcto.Migration.collection/2`.\n\nThe attributes in this struct are directly passed to the\nArangoDB API for creation. No validation is done on the\nattributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Collection.html","title":"ArangoXEcto.Migration.Collection","type":"module"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:collection_option/0","title":"ArangoXEcto.Migration.Collection.collection_option/0","type":"type"},{"doc":"Creates a new Collection struct","ref":"ArangoXEcto.Migration.Collection.html#new/2","title":"ArangoXEcto.Migration.Collection.new/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:t/0","title":"ArangoXEcto.Migration.Collection.t/0","type":"type"},{"doc":"Used internally by the `ArangoXEcto` migration.\n\nRepresents the up and down of a reversible raw command defined by\n`ArangoXEcto.Migration.execute/1`. To make it reversible call `ArangoXEcto.Migration.execute/2`\ninstead.","ref":"ArangoXEcto.Migration.Command.html","title":"ArangoXEcto.Migration.Command","type":"module"},{"doc":"","ref":"ArangoXEcto.Migration.Command.html#t:t/0","title":"ArangoXEcto.Migration.Command.t/0","type":"type"},{"doc":"Used internally by the `ArangoXEcto` migration.\n\nTo define an index in a migration, see `ArangoXEcto.Migration.index/3`.\n\nThe attributes in this struct are directly passed to the\nArangoDB API for creation. No validation is done on the\nattributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Index.html","title":"ArangoXEcto.Migration.Index","type":"module"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:index_option/0","title":"ArangoXEcto.Migration.Index.index_option/0","type":"type"},{"doc":"Creates a new Index struct","ref":"ArangoXEcto.Migration.Index.html#new/3","title":"ArangoXEcto.Migration.Index.new/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:t/0","title":"ArangoXEcto.Migration.Index.t/0","type":"type"},{"doc":"Converts migration command to JSON Schema.\n\nThis will take a list of commands that are generated from a migration and convert it into a\nJSON Schema representation.\n\nArangoDB takes a JSON schema that has 3 arguments:\n\n  * `:rule` - the actual JSON schema to be checked (this is where the commands get converted to)\n  * `:level` - how ArangoDB checks a document against the schema\n  * `:message` - a message that is returned when the schema doesn't match\n\nFor more info on how to change the `:level` and `:message`, see `generate_schema/2`.\n\nThis is based on the JSON Schema definition, more info can be found\n[here](https://json-schema.org/). Note that not all of the JSON Schema attributes are implemented,\nso check the documentation in this module for what is implemented.\n\nFor example, take the following migration.\n\n    create collection(:users) do\n      add :first_name, :string, comment: \"first_name column\"\n      add :last_name, :string\n      add :gender, :integer\n      add :age, :integer\n      add :location, :map\n\n      timestamps()\n    end\n\nThis will result in the following JSON Schema.\n\n    %{\n      rule: %{\n        type: \"object\",\n        properties: %{\n          first_name: %{\n            type: \"string\",\n            comment: \"first_name column\"\n          },\n          last_name: %{type: \"string\"},\n          gender: %{type: \"integer\"},\n          age: %{type: \"integer\"},\n          location: %{type: \"map\"},\n          inserted_at: %{\n            type: \"string\",\n            pattern: \" \"\n          },\n          updated_at: %{\n            type: \"string\",\n            pattern: \" \"\n          }\n        }\n      },\n      level: :strict,\n      message:\n        \"The document does not match the schema. Please ensure the document matches the json schema.\"\n    }","ref":"ArangoXEcto.Migration.JsonSchema.html","title":"ArangoXEcto.Migration.JsonSchema","type":"module"},{"doc":"","ref":"ArangoXEcto.Migration.JsonSchema.html#t:action/0","title":"ArangoXEcto.Migration.JsonSchema.action/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.JsonSchema.html#t:command/0","title":"ArangoXEcto.Migration.JsonSchema.command/0","type":"type"},{"doc":"Converts commands to a JSON schema\n\nThis will take commands in the format of:\n\n    {action, name, type_or_sub_commands, options}\n\nFor example, take the following:\n\n    {:add, :first_name, :string, comment: \"Some comment\"}\n\nWill result in the following part of the JSONSchema.\n\n    %{\"type\" => \"string\", :\"$comment\" => \"Some comment\"}\n\nThe type and options passed are validated to ensure they are the right type.","ref":"ArangoXEcto.Migration.JsonSchema.html#convert/2","title":"ArangoXEcto.Migration.JsonSchema.convert/2","type":"function"},{"doc":"The available actions are available in `t:action/0`","ref":"ArangoXEcto.Migration.JsonSchema.html#convert/2-available-actions","title":"Available actions - ArangoXEcto.Migration.JsonSchema.convert/2","type":"function"},{"doc":"* `:string` - represents a string with the following options available\n    * `:min_length` - Minimum length of the string\n    * `:max_length` - Maximum length of the string\n    * `:pattern` - A string regex pattern\n    * `:format` - A validator for the format, available ones can be found [here](https://json-schema.org/understanding-json-schema/reference/string#built-in-formats).\n    * `:content_encoding` - The content encoding to use, available values can be found [here](https://json-schema.org/understanding-json-schema/reference/non_json_data#contentencoding)\n    * `:content_media_type` - The content media type to use, available values can be found [here](https://json-schema.org/understanding-json-schema/reference/non_json_data#contentencoding)\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null\n\n  * `:integer`, `:number`, `:decimal` or `:float` - represents a numeric value with the following options available\n    * `:minimum` - Minimum value of the number\n    * `:exclusive_minimum` - Minimum value of the number exclusive\n    * `:maximum` - Maximum value of the number\n    * `:exclusive_maximum` - Maximum value of the number exclusive\n    * `:multiple_of` - A value the number must be a multiple of\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null\n\n  * `:enum` - a set of values\n    * `:values` - A list of possible values\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null\n\n  * `:const` - a value\n    * `:value` - A list of possible values\n    * `:comment` - A comment to add to the object, it has no functional application\n\n  * `:date` - represents a date\n    * This essentially just calls `:string` with format=date. Accepts the same parameters as `:string`\n\n  * `:datetime`, `:utc_datetime` or `:utc_datetime_usec` - represents a datetime\n    * This essentially just calls `:string` with format=date-time. Accepts the same parameters as `:string`\n\n  * `:naive_datetime` or `:naive_datetime_usec` - represents a naive datetime\n    * This essentially just calls `:string` with a custom pattern to validate a naive datetime. Accepts the same parameters as `:string`\n\n  * `:uuid` - represents a uuid\n    * This essentially just calls `:string` with a custom pattern to validate a uuid. Accepts the same parameters as `:string`\n\n  * `:boolean` - represents a boolean with the following options available\n    * `:comment` - A comment to add to the object, it has no functional application\n\n  * `:map` - represents a map with the following options available\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null\n\n  * `{:array, sub_type}` - represents an array with the specified `sub_type` which can be any type\n    * `:min_items` - Minimum number of items in the array\n    * `:max_items` - Maximum number of items in the array\n    * `:unique_items` - Boolean if each item must be unique\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null\n\n  * `[sub_command]` - an object with sub properties where `sub_command` is a command\n    * `:pattern_properties` - A pattern for property keys, see [the docs](https://json-schema.org/understanding-json-schema/reference/object#patternProperties) for more info\n    * `:additional_properties` - The type of non-listed properties (or false to disallow), see [the docs](https://json-schema.org/understanding-json-schema/reference/object#additionalproperties) for more info\n    * `:required` - A list of required properties\n    * `:min_properties` - The minimum number of properties\n    * `:max_properties` - The maximum number of properties\n    * `:comment` - A comment to add to the object, it has no functional application\n    * `:null` - If the value can be null","ref":"ArangoXEcto.Migration.JsonSchema.html#convert/2-available-types","title":"Available types - ArangoXEcto.Migration.JsonSchema.convert/2","type":"function"},{"doc":"* `:schema` - the existing schema to apply the migrations to. This is useful so that a schema\n    isn't overwritten and is instead updated.","ref":"ArangoXEcto.Migration.JsonSchema.html#convert/2-options","title":"Options - ArangoXEcto.Migration.JsonSchema.convert/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.JsonSchema.html#t:field/0","title":"ArangoXEcto.Migration.JsonSchema.field/0","type":"type"},{"doc":"Generates an ArangoDB schema.\n\nTakes in the field commands and generates a schema.","ref":"ArangoXEcto.Migration.JsonSchema.html#generate_schema/2","title":"ArangoXEcto.Migration.JsonSchema.generate_schema/2","type":"function"},{"doc":"* `:level` - the level to use for ArangoDB schema checking. Available options are:\n    :none, :new, :moderate, :strict. `:strict` is the default.\n    Please refer to the ArangoDB docs for what these mean.\n    \n  * `:message` - a message to be returned when a document doesn't match the schema.\n\n  Any other options are passed to `convert/2`.","ref":"ArangoXEcto.Migration.JsonSchema.html#generate_schema/2-options","title":"Options - ArangoXEcto.Migration.JsonSchema.generate_schema/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.JsonSchema.html#t:level/0","title":"ArangoXEcto.Migration.JsonSchema.level/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.JsonSchema.html#t:t/0","title":"ArangoXEcto.Migration.JsonSchema.t/0","type":"type"},{"doc":"Used internally by the `ArangoXEcto` migration.\n\nTo define a view in a migration, see `ArangoXEcto.Migration.view/2`.","ref":"ArangoXEcto.Migration.View.html","title":"ArangoXEcto.Migration.View","type":"module"},{"doc":"Creates a new View struct","ref":"ArangoXEcto.Migration.View.html#new/2","title":"ArangoXEcto.Migration.View.new/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.View.html#t:t/0","title":"ArangoXEcto.Migration.View.t/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.View.html#t:view_option/0","title":"ArangoXEcto.Migration.View.view_option/0","type":"type"},{"doc":"Lower level API for managing migrations.\n\nThis module provides functions to manage migrations.\nThe tasks mentioned below are provided to cover the\ngeneral use cases but it may be necessary to use the\nlower level API in some cases. For example, when building\nan Elixir release, Mix is not available, so this module \nprovides the functions needed to migrate the system.\n\nLarge parts of this module is based on the `ecto_sql` `Migrator`\nmodule, so a lot of credit to them.","ref":"ArangoXEcto.Migrator.html","title":"ArangoXEcto.Migrator","type":"module"},{"doc":"The following mix tasks are available. They are similar to `ecto_sql`\ntasks but are appended with `.arango` to allow for both `ecto_sql`\nand `arangox_ecto` to be used side by side.\n\n  * `mix arango.migrate` - migrates an arango repository\n  * `mix arango.rollback` - rolls back a particular migration\n  * `mix arango.gen.migration` - generates a migration file","ref":"ArangoXEcto.Migrator.html#module-mix-tasks","title":"Mix tasks - ArangoXEcto.Migrator","type":"module"},{"doc":"Imagine you have this migration:\n\n    defmodule MyApp.MigrationExample do\n      use ArangoXEcto.Migration\n\n      def up do\n        create collection(\"my_collection\")\n      end\n\n      def down do\n        drop collection(\"my_collection\")\n      end\n    end\n\nYou can execute it manually with:\n\n    ArangoXEcto.Migrator.up(Repo, 20240101120000, MyApp.MigrationExample)","ref":"ArangoXEcto.Migrator.html#module-example-running-an-individual-migration","title":"Example: Running an individual migration - ArangoXEcto.Migrator","type":"module"},{"doc":"Elixir v1.9 introduces `mix release`, which generates a self-contained\ndirectory that consists of your application code, all of its dependencies,\nplus the whole Erlang Virtual Machine (VM) and runtime.\n\nWhen a release is assembled, Mix is no longer available inside a release\nand therefore none of the Mix tasks. Users may still need a mechanism to\nmigrate their databases. This can be achieved with using this `ArangoXEcto.Migrator`\nmodule:\n\n    defmodule MyApp.Release do\n      @app :my_app\n\n      def migrate do\n        for repo <- repos() do\n          {:ok, _, _} = ArangoXEcto.Migrator.with_repo(repo, &ArangoXEcto.Migrator.run(&1, :up, all: true))\n        end\n      end\n\n      def rollback(repo, version) do\n        {:ok, _, _} = ArangoXEcto.Migrator.with_repo(repo, &ArangoXEcto.Migrator.run(&1, :down, to: version))\n      end\n\n      defp repos do\n        Application.load(@app)\n        Application.fetch_env!(@app, :ecto_repos)\n      end\n    end\n\nThe example above uses `with_repo/3` to make sure the repository is\nstarted and then runs all migrations up or a given migration down.\nNote you will have to replace `MyApp` and `:my_app` on the first two\nlines by your actual application name. Once the file above is added\nto your application, you can assemble a new release and invoke the\ncommands above in the release root like this:\n\n    $ bin/my_app eval \"MyApp.Release.migrate\"\n    $ bin/my_app eval \"MyApp.Release.rollback(MyApp.Repo, 20231225190000)\"","ref":"ArangoXEcto.Migrator.html#module-example-running-migrations-in-a-release","title":"Example: Running migrations in a release - ArangoXEcto.Migrator","type":"module"},{"doc":"Add the following to the top of your application child spec:\n\n    {ArangoXEcto.Migrator, \n      repos: Application.fetch_env!(:my_app, :ecto_repos), \n      skip: System.get_env(\"SKIP_MIGRATIONS\") == \"true\")}\n\nThis will allow you to use the environment variable `SKIP_MIGRATIONS` to skip running the\nmigrations on startup.\n\nAdditionally, the other options specified in `up/4` can be passed. For example, if you wanted to\nlog the migrations and run them in a specific prefix you could do the following:\n\n    {ArangoXEcto.Migrator, \n      repos: Application.fetch_env!(:my_app, :ecto_repos), \n      log_migrator: true,\n      prefix: \"super_cool_user\"}\n\nRolling back would be done normally using the `mix arango.rollback` mix task.","ref":"ArangoXEcto.Migrator.html#module-example-running-migrations-on-application-startup","title":"Example: Running migrations on application startup - ArangoXEcto.Migrator","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ArangoXEcto.Migrator.html#child_spec/1","title":"ArangoXEcto.Migrator.child_spec/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Migrator.html#t:command/0","title":"ArangoXEcto.Migrator.command/0","type":"type"},{"doc":"Runs a down migration on a given repo.","ref":"ArangoXEcto.Migrator.html#down/4","title":"ArangoXEcto.Migrator.down/4","type":"function"},{"doc":"* `:log` - the level to use for logging of migration instructions.\n    Defaults to `:info`. Can be any of `Logger.level/0` values or a boolean.\n    If `false`, it also avoids logging messages from the database.\n  * `:log_migrations` - the level to use for logging of ArangoDB commands\n    generated by migrations. Can be any of the `Logger.level/0` values\n    or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:log_migrator` - the level to use for logging of ArangoDB commands emitted\n    by the migrator, such as transactions, locks, etc. Can be any of the `Logger.level/0`\n    values or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:prefix` - the prefix to run the migrations on","ref":"ArangoXEcto.Migrator.html#down/4-options","title":"Options - ArangoXEcto.Migrator.down/4","type":"function"},{"doc":"Executes a command on a repo\n \nThe commands are passed from the migration module. A command essentially is the action of a\nmigration, e.g. create, alter, rename, delete an object.\n\n`opts` is passed to the database query when the command is run. Available options are dependant on\nthe command being executed.","ref":"ArangoXEcto.Migrator.html#execute_command/3","title":"ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"All commands are either a tuple or a string with AQL which is executed. The first element of the\ntuple is the action to be performed, the second is the object of the action. For some commands\n(such as creation of a collection) there is a third element of the tuple.","ref":"ArangoXEcto.Migrator.html#execute_command/3-available-commands","title":"Available Commands - ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"| 2nd element (Object) |              Third element            |\n| -------------------- | ------------------------------------- |\n| Collection           | Field commands passed to `JsonSchema` |\n| Index                |                                       |\n| View                 | View definition commands              |\n| Analyzer             |                                       |","ref":"ArangoXEcto.Migrator.html#execute_command/3-creates-create-and-create_if_not_exists","title":"Creates (:create and :create_if_not_exists) - ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"| 2nd element (Object) |                              Third element                          |\n| -------------------- | ------------------------------------------------------------------- |\n| Existing Collection  | Field commands passed to `JsonSchema` (merged with existing schema) |\n| Existing View        | View definition commands                                            |","ref":"ArangoXEcto.Migrator.html#execute_command/3-alter-alter","title":"Alter (:alter) - ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"| 2nd element (Object) | Third element |\n| -------------------- | ------------- |\n| Existing Collection  |               |\n| Existing Index       |               |\n| Existing View        |               |\n| Existing Analyzer    |               |","ref":"ArangoXEcto.Migrator.html#execute_command/3-drops-drop-and-drop_if_exists","title":"Drops (:drop and :drop_if_exists) - ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"| 2nd element (Object) |       Third element      |\n| -------------------- | ------------------------ |\n| Existing Collection  | Collection with new name |\n| Existing View        | View with new name       |","ref":"ArangoXEcto.Migrator.html#execute_command/3-rename-rename","title":"Rename (:rename) - ArangoXEcto.Migrator.execute_command/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migrator.html#generate_view_definition/2","title":"ArangoXEcto.Migrator.generate_view_definition/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Migrator.html#t:log/0","title":"ArangoXEcto.Migrator.log/0","type":"type"},{"doc":"Gets all migrated versions.\n\nEnsures the migrations collection exists if it doesn't exist yet.","ref":"ArangoXEcto.Migrator.html#migrated_versions/2","title":"ArangoXEcto.Migrator.migrated_versions/2","type":"function"},{"doc":"* `:prefix` - the prefix to run the migrations on\n  * `:dynamic_repo` - the name of the Repo supervisor process.\n    See `c:Ecto.Repo.put_dynamic_repo/1`.\n  * `:skip_collection_creation` - skips any attempt to create the migration collection\n    Useful for situations where user needs to check migrations but has\n    insufficient permissions to create the collection.  Note that migrations\n    commands may fail if this is set to true. Defaults to `false`.  Accepts a\n    boolean.","ref":"ArangoXEcto.Migrator.html#migrated_versions/2-options","title":"Options - ArangoXEcto.Migrator.migrated_versions/2","type":"function"},{"doc":"Returns an array of the statuses of migrations for the given repo\n\nDoesn't actually run the migrations.\n\nThis is a shortcut to\n\n    ArangoXEcto.Migrator.migrations(repo, [ArangoXEcto.Migrator.migrations_path(repo)])","ref":"ArangoXEcto.Migrator.html#migrations/1","title":"ArangoXEcto.Migrator.migrations/1","type":"function"},{"doc":"Returns an array of the statuses of migrations for the given repo\n\nDoesn't actually run the migrations.","ref":"ArangoXEcto.Migrator.html#migrations/3","title":"ArangoXEcto.Migrator.migrations/3","type":"function"},{"doc":"Gets the migrations path for a repository.\n\nAccepts a second option allowing to specify the name of / path to the \ndirectory for the migrations.","ref":"ArangoXEcto.Migrator.html#migrations_path/2","title":"ArangoXEcto.Migrator.migrations_path/2","type":"function"},{"doc":"Runs migrations for the given repository.\n\nEquivalent to:\n\n    ArangoXEcto.Migrator.run(repo, [ArangoXEcto.Migrator.migrations_path(repo)], direction, opts)\n\nSee `run/4` for more information.","ref":"ArangoXEcto.Migrator.html#run/3","title":"ArangoXEcto.Migrator.run/3","type":"function"},{"doc":"Apply migrations to a repository with a given strategy.\n\nThe second argument identifies where the migrations are sourced from.\nA binary representing directory (or a list of binaries representing\ndirectories) may be passed, in which case we will load all files\nfollowing the \"#{VERSION}_#{NAME}.exs\" schema. The `migration_source`\nmay also be a list of tuples that identify the version number and\nmigration modules to be run, for example:\n\n    ArangoXEcto.Migrator.run(Repo, [{0, MyApp.Migration1}, {1, MyApp.Migration2}, ...], :up, opts)\n\nA strategy (which is one of `:all`, `:step`, `:to`, or `:to_exclusive`) must be given as\nan option.","ref":"ArangoXEcto.Migrator.html#run/4","title":"ArangoXEcto.Migrator.run/4","type":"function"},{"doc":"In order to run migrations, at least two database connections are\nnecessary. One is used to lock the \"_migrations\" collection and\nthe other one to effectively run the migrations. This allows multiple\nnodes to run migrations at the same time, but guarantee that only one\nof them will effectively migrate the database.","ref":"ArangoXEcto.Migrator.html#run/4-execution-model","title":"Execution model - ArangoXEcto.Migrator.run/4","type":"function"},{"doc":"* `:all` - runs all available if `true`\n\n  * `:step` - runs the specific number of migrations\n\n  * `:to` - runs all until the supplied version is reached\n    (including the version given in `:to`)\n\n  * `:to_exclusive` - runs all until the supplied version is reached\n    (excluding the version given in `:to_exclusive`)\n\nPlus all other options described in `up/4`.","ref":"ArangoXEcto.Migrator.html#run/4-options","title":"Options - ArangoXEcto.Migrator.run/4","type":"function"},{"doc":"Runs migrations as part of your supervision tree.","ref":"ArangoXEcto.Migrator.html#start_link/1","title":"ArangoXEcto.Migrator.start_link/1","type":"function"},{"doc":"* `:repos` - Required option to tell the migrator which Repo's to\n    migrate. Example: `repos: [MyApp.Repo]`\n\n  * `:skip` - Option to skip migrations. Defaults to `false`.\n\nPlus all other options described in `up/4`.\n\nSee \"Example: Running migrations on application startup\" for more info.","ref":"ArangoXEcto.Migrator.html#start_link/1-options","title":"Options - ArangoXEcto.Migrator.start_link/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Migrator.html#t:subcommand/0","title":"ArangoXEcto.Migrator.subcommand/0","type":"type"},{"doc":"Runs an up migration on a given repo.","ref":"ArangoXEcto.Migrator.html#up/4","title":"ArangoXEcto.Migrator.up/4","type":"function"},{"doc":"* `:log` - the level to use for logging of migration instructions.\n    Defaults to `:info`. Can be any of `Logger.level/0` values or a boolean.\n    If `false`, it also avoids logging messages from the database.\n  * `:log_migrations` - the level to use for logging of ArangoDB commands\n    generated by migrations. Can be any of the `Logger.level/0` values\n    or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:log_migrator` - the level to use for logging of ArangoDB commands emitted\n    by the migrator, such as transactions, locks, etc. Can be any of the `Logger.level/0`\n    values or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:prefix` - the prefix to run the migrations on\n  * `:dynamic_repo` - the name of the Repo supervisor process.\n    See `c:Ecto.Repo.put_dynamic_repo/1`.\n  * `:strict_version_order` - abort when applying a migration with old timestamp\n    (otherwise it emits a warning)","ref":"ArangoXEcto.Migrator.html#up/4-options","title":"Options - ArangoXEcto.Migrator.up/4","type":"function"},{"doc":"Ensures the repo is started to perform migration operations.\n\nAll applications required to run the repo will be started before\nwith the chosen mode. If the repo has not yet been started,\nit is manually started, with a `:pool_size` of 2 (or otherwise passed),\nbefore the given function is executed, and the repo is then terminated.\nIf the repo was already started, then the function is directly executed,\nwithout terminating the repo afterwards.\n\nAlthough this function was designed to start repositories for running\nmigrations, it can be used by any code, Mix task, or release tooling\nthat needs to briefly start a repository to perform a certain operation\nand then terminate.\n\nThe repo may also configure a `:start_apps_before_migration` option\nwhich is a list of applications to be started before the migration\nruns.\n\nIt returns `{:ok, fun_return, apps}`, with all apps that have been\nstarted, or `{:error, term}`.","ref":"ArangoXEcto.Migrator.html#with_repo/3","title":"ArangoXEcto.Migrator.with_repo/3","type":"function"},{"doc":"* `:pool_size` - The pool size to start the repo for migrations.\n    Defaults to 2.\n  * `:mode` - The mode to start all applications.\n    Defaults to `:permanent`.","ref":"ArangoXEcto.Migrator.html#with_repo/3-options","title":"Options - ArangoXEcto.Migrator.with_repo/3","type":"function"},{"doc":"{:ok, _, _} =\n      ArangoXEcto.Migrator.with_repo(repo, fn repo ->\n        ArangoXEcto.Migrator.run(repo, :up, all: true)\n      end)","ref":"ArangoXEcto.Migrator.html#with_repo/3-examples","title":"Examples - ArangoXEcto.Migrator.with_repo/3","type":"function"},{"doc":"A pool for concurrent transactional tests.\n\nBased on the `Ecto.Adapters.SQL.Sandbox` module from `:ecto_sql`.\n\nArangoDB RocksDB storage-engine handles locks and isolations when\nusing transactions.\n\nThe sandbox pool is implemented on top of an ownership mechanism.\nWhen started, the pool is in automatic mode, which means the\nrepository will automatically check connections out as with any\nother pool.\n\nThe `mode/2` function can be used to change the pool mode from\nautomatic to either manual or shared. In the latter two modes,\nthe connection must be explicitly checked out before use.\nWhen explicit checkouts are made, the sandbox will wrap the\nconnection in a transaction by default and control who has\naccess to it. This means developers have a safe mechanism for\nrunning concurrent tests against the database.","ref":"ArangoXEcto.Sandbox.html","title":"ArangoXEcto.Sandbox","type":"module"},{"doc":"The first step is to configure your database to use the\n`ArangoXEcto.Sandbox` pool. You set those options in your\n`config/config.exs` (or preferably `config/test.exs`) if you\nhaven't yet:\n\n    config :my_app, Repo,\n      pool: ArangoXEcto.Sandbox\n\nNow with the test database properly configured, you can write\ntransactional tests:\n\n    # At the end of your test_helper.exs\n    # Set the pool mode to manual for explicit checkouts\n    ArangoXEcto.Sandbox.mode(Repo, :manual)\n\n    defmodule PostTest do\n      # Once the mode is manual, tests can also be async\n      use ExUnit.Case, async: true\n\n      setup do\n        # Explicitly get a connection before each test\n        :ok = ArangoXEcto.Sandbox.checkout(Repo)\n      end\n\n      setup tags do\n        pid = ArangoXEcto.Sandbox.start_owner!(Repo, shared: not tags[:async])\n        on_exit(fn -> ArangoXEcto.Sandbox.stop_owner(pid) end)\n        :ok\n      end\n\n      test \"create post\" do\n        # Use the repository as usual\n        assert %Post{} = Repo.insert!(%Post{})\n      end\n    end","ref":"ArangoXEcto.Sandbox.html#module-example","title":"Example - ArangoXEcto.Sandbox","type":"module"},{"doc":"The example above is straight-forward because we have only\na single process using the database connection. However,\nsometimes a test may need to interact with multiple processes,\nall using the same connection so they all belong to the same\ntransaction.\n\nBefore we discuss solutions, let's see what happens if we try\nto use a connection from a new process without explicitly\nchecking it out first:\n\n    setup do\n      # Explicitly get a connection before each test\n      :ok = ArangoXEcto.Sandbox.checkout(Repo)\n    end\n\n    test \"calls worker that runs a query\" do\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nThe test above will fail with an error similar to:\n\n    ** (DBConnection.OwnershipError) cannot find ownership process for #PID<0.35.0>\n\nThat's because the `setup` block is checking out the connection only\nfor the test process. Once the worker attempts to perform a query,\nthere is no connection assigned to it and it will fail.\n\nThe sandbox module provides two ways of doing so, via allowances or\nby running in shared mode.","ref":"ArangoXEcto.Sandbox.html#module-collaborating-processes","title":"Collaborating processes - ArangoXEcto.Sandbox","type":"module"},{"doc":"The idea behind allowances is that you can explicitly tell a process\nwhich checked out connection it should use, allowing multiple processes\nto collaborate over the same connection. Let's give it a try:\n\n    test \"calls worker that runs a query\" do\n      allow = Process.whereis(MyApp.Worker)\n      ArangoXEcto.Sandbox.allow(Repo, self(), allow)\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nAnd that's it, by calling `allow/3`, we are explicitly assigning\nthe parent's connection (i.e. the test process' connection) to\nthe task.\n\nBecause allowances use an explicit mechanism, their advantage\nis that you can still run your tests in async mode. The downside\nis that you need to explicitly control and allow every single\nprocess. This is not always possible. In such cases, you will\nwant to use shared mode.","ref":"ArangoXEcto.Sandbox.html#module-allowances","title":"Allowances - ArangoXEcto.Sandbox","type":"module"},{"doc":"Shared mode allows a process to share its connection with any other\nprocess automatically, without relying on explicit allowances.\nLet's change the example above to use shared mode:\n\n    setup do\n      # Explicitly get a connection before each test\n      :ok = ArangoXEcto.Sandbox.checkout(Repo)\n      # Setting the shared mode must be done only after checkout\n      ArangoXEcto.Sandbox.mode(Repo, {:shared, self()})\n    end\n\n    test \"calls worker that runs a query\" do\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nBy calling `mode({:shared, self()})`, any process that needs\nto talk to the database will now use the same connection as the\none checked out by the test process during the `setup` block.\n\nMake sure to always check a connection out before setting the mode\nto `{:shared, self()}`.\n\nThe advantage of shared mode is that by calling a single function,\nyou will ensure all upcoming processes and operations will use that\nshared connection, without a need to explicitly allow them. The\ndownside is that tests can no longer run concurrently in shared mode.\n\nAlso, beware that if the test process terminates while the worker is\nusing the connection, the connection will be taken away from the worker,\nwhich will error. Therefore it is important to guarantee the work is done\nbefore the test concludes. In the example above, we are using a `call`,\nwhich is synchronous, avoiding the problem, but you may need to explicitly\nflush the worker or terminate it under such scenarios in your tests.","ref":"ArangoXEcto.Sandbox.html#module-shared-mode","title":"Shared mode - ArangoXEcto.Sandbox","type":"module"},{"doc":"There are two mechanisms for explicit ownerships:\n\n  * Using allowances - requires explicit allowances via `allow/3`.\n    Tests may run concurrently.\n\n  * Using shared mode - does not require explicit allowances.\n    Tests cannot run concurrently.","ref":"ArangoXEcto.Sandbox.html#module-summing-up","title":"Summing up - ArangoXEcto.Sandbox","type":"module"},{"doc":"When running the sandbox mode concurrently, developers may run into\nissues we explore in the upcoming sections.\n\n### \"owner exited\"\n\nIn some situations, you may see error reports similar to the one below:\n\n    23:59:59.999 [error] Arangox.Connection (#PID<>) disconnected:\n        ** (DBConnection.Error) owner #PID<> exited\n    Client #PID<> is still using a connection from owner\n\nSuch errors are usually followed by another error report from another\nprocess that failed while executing a database query.\n\nTo understand the failure, we need to answer the question: who are the\nowner and client processes? The owner process is the one that checks\nout the connection, which, in the majority of cases, is the test process,\nthe one running your tests. In other words, the error happens because\nthe test process has finished, either because the test succeeded or\nbecause it failed, while the client process was trying to get information\nfrom the database. Since the owner process, the one that owns the\nconnection, no longer exists, Ecto will check the connection back in\nand notify the client process using the connection that the connection\nowner is no longer available.\n\nThis can happen in different situations. For example, imagine you query\na GenServer in your test that is using a database connection:\n\n    test \"gets results from GenServer\" do\n      {:ok, pid} = MyAppServer.start_link()\n      ArangoXEcto.Sandbox.allow(Repo, self(), pid)\n      assert MyAppServer.get_my_data_fast(timeout: 1000) == [...]\n    end\n\nIn the test above, we spawn the server and allow it to perform database\nqueries using the connection owned by the test process. Since we gave\na timeout of 1 second, in case the database takes longer than one second\nto reply, the test process will fail, due to the timeout, making the\n\"owner down\" message to be printed because the server process is still\nwaiting on a connection reply.\n\nIn some situations, such failures may be intermittent. Imagine that you\nallow a process that queries the database every half second:\n\n    test \"queries periodically\" do\n      {:ok, pid} = PeriodicServer.start_link()\n      ArangoXEcto.Sandbox.allow(Repo, self(), pid)\n      # assertions\n    end\n\nBecause the server is querying the database from time to time, there is\na chance that, when the test exits, the periodic process may be querying\nthe database, regardless of test success or failure.\n\nTo address this, you can tell ExUnit to manage your processes:\n\n    test \"queries periodically\" do\n      pid = start_supervised!(PeriodicServer)\n      ArangoXEcto.Sandbox.allow(Repo, self(), pid)\n      # assertions\n    end\n\nBy using `start_supervised!/1`, ExUnit guarantess the process finishes\nbefore your test (the connection owner).\n\nIn some situations, however, the dynamic processes are directly started\ninside a `DynamicSupervisor` or a `Task.Supervisor`. You can guarantee\nproper termination in such scenarios by adding an `on_exit` callback\nthat waits until all supervised children terminate:\n\n    on_exit(fn ->\n      for {_, pid, _, _} <- DynamicSupervisor.which_children(MyApp.DynamicSupervisor) do\n        ref = Process.monitor(pid)\n        assert_receive {:DOWN, ^ref, _, _, _}, :infinity\n      end\n    end)\n\n### \"owner timed out because it owned the connection for longer than Nms\"\n\nIn some situations, you may see error reports similar to the one below:\n\n    09:56:43.081 [error] Arangox.Connection (#PID<>) disconnected:\n        ** (DBConnection.ConnectionError) owner #PID<> timed out\n        because it owned the connection for longer than 120000ms\n\nIf you have a long running test (or you're debugging with IEx.pry),\nthe timeout for the connection ownership may be too short.  You can\nincrease the timeout by setting the `:ownership_timeout` options for\nyour repo config in `config/config.exs` (or preferably in `config/test.exs`):\n\n    config :my_app, MyApp.Repo,\n      ownership_timeout: NEW_TIMEOUT_IN_MILLISECONDS\n\nThe `:ownership_timeout` option is part of `DBConnection.Ownership`\nand defaults to 120000ms. Timeouts are given as integers in milliseconds.\n\nAlternately, if this is an issue for only a handful of long-running tests,\nyou can pass an `:ownership_timeout` option when calling\n`ArangoXEcto.Sandbox.checkout/2` instead of setting a longer timeout\nglobally in your config.","ref":"ArangoXEcto.Sandbox.html#module-faq","title":"FAQ - ArangoXEcto.Sandbox","type":"module"},{"doc":"Allows the `allow` process to use the same connection as `parent`.\n\n`allow` may be a PID or a locally registered name.","ref":"ArangoXEcto.Sandbox.html#allow/4","title":"ArangoXEcto.Sandbox.allow/4","type":"function"},{"doc":"Checks in the connection back into the sandbox pool.","ref":"ArangoXEcto.Sandbox.html#checkin/2","title":"ArangoXEcto.Sandbox.checkin/2","type":"function"},{"doc":"Checks a connection out for the given `repo`.\n\nThe process calling `checkout/2` will own the connection\nuntil it calls `checkin/2` or until it crashes in which case\nthe connection will be automatically reclaimed by the pool.","ref":"ArangoXEcto.Sandbox.html#checkout/2","title":"ArangoXEcto.Sandbox.checkout/2","type":"function"},{"doc":"* `:sandbox` - when true the connection is wrapped in\n    a transaction. Defaults to true.\n\n  * `:ownership_timeout` - limits how long the connection can be\n    owned. Defaults to the value in your repo config in\n    `config/config.exs` (or preferably in `config/test.exs`), or\n    120000 ms if not set. The timeout exists for sanity checking\n    purposes, to ensure there is no connection leakage, and can\n    be bumped whenever necessary.","ref":"ArangoXEcto.Sandbox.html#checkout/2-options","title":"Options - ArangoXEcto.Sandbox.checkout/2","type":"function"},{"doc":"Sets the mode for `repo` pool.\n\nThe modes can be:\n\n  * `:auto` - this is the default mode. When trying to use the repository,\n    processes can automatically checkout a connection without calling\n    `checkout/2` or `start_owner/2` before. This is the mode you will run\n    on before your test suite starts\n\n  * `:manual` - in this mode, the connection always has to be explicitly\n    checked before used. Other processes are allowed to use the same\n    connection if they are explicitly allowed via `allow/4`. You usually\n    set the mode to manual at the end of your `test/test_helper.exs` file.\n    This is also the mode you will run your async tests in\n\n  * `{:shared, pid}` - after checking out a connection in manual mode,\n    you can change the mode to `{:shared, pid}`, where pid is the process\n    that owns the connection, most often `{:shared, self()}`. This makes it\n    so all processes can use the same connection as the one owned by the\n    current process. This is the mode you will run your sync tests in\n\nWhenever you change the mode to `:manual` or `:auto`, all existing\nconnections are checked in. Therefore, it is recommend to set those\nmodes before your test suite starts, as otherwise you will check in\nconnections being used in any other test running concurrently.","ref":"ArangoXEcto.Sandbox.html#mode/2","title":"ArangoXEcto.Sandbox.mode/2","type":"function"},{"doc":"Starts a process that owns the connection and returns its pid.\n\nThe owner process is not linked to the caller, it is your responsibility to\nensure it will be stopped. In tests, this is done by terminating the pool\nin an `ExUnit.Callbacks.on_exit/2` callback:\n\n    setup tags do\n      pid = ArangoXEcto.Sandbox.start_owner!(MyApp.Repo, shared: not tags[:async])\n      on_exit(fn -> ArangoXEcto.Sandbox.stop_owner(pid) end)\n      :ok\n    end","ref":"ArangoXEcto.Sandbox.html#start_owner!/2","title":"ArangoXEcto.Sandbox.start_owner!/2","type":"function"},{"doc":"* `:shared` - if `true`, the pool runs in the shared mode. Defaults to `false`\n\nThe remaining options are passed to `checkout/2`.","ref":"ArangoXEcto.Sandbox.html#start_owner!/2-options","title":"Options - ArangoXEcto.Sandbox.start_owner!/2","type":"function"},{"doc":"Stops an owner process started by `start_owner!/2`.","ref":"ArangoXEcto.Sandbox.html#stop_owner/1","title":"ArangoXEcto.Sandbox.stop_owner/1","type":"function"},{"doc":"Runs a function outside of the sandbox.","ref":"ArangoXEcto.Sandbox.html#unboxed_run/2","title":"ArangoXEcto.Sandbox.unboxed_run/2","type":"function"},{"doc":"The association struct for a `edge_many` association.\n\nThis is only to be used by an edge's `:from` and `:to` fields. This is\nessentially the same as a BelongsTo association but modified to allow mupltiple\nrelated fields.\n\nThis is needed because relational databases don't have the same ability to\nhave multiple types of relation to the same name field like graph edges.\n\nNote that this will still only have a cardinality of one. I.e. in an edge\nthe `:from` field will still only reference one other schema but can\nmultiple different types of schemas.\n\nThe available fields are:\n\n  * `cardinality` - The association cardinality, always `:one`\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `related` - The schema that is associated\n  * `related_key` - The key on the `related` schema used for the association\n  * `queryables` - The real query to use for querying association\n  * `defaults` - default fields used when building the association\n  * `relationship` - The relationship to the specified schema, default `:parent`\n  * `on_replace` - The action taken on associations when schema is replaced","ref":"ArangoXEcto.Association.EdgeMany.html","title":"ArangoXEcto.Association.EdgeMany","type":"module"},{"doc":"","ref":"ArangoXEcto.Association.EdgeMany.html#t:t/0","title":"ArangoXEcto.Association.EdgeMany.t/0","type":"type"},{"doc":"The association struct for a `graph` association.\n\nThis is based off of the `many_to_many` relationship but modified to suit\ngraph relations. This is uses graph traversal queries instead of the standard\nSQL format queries.\n\nThis allows for multiple related schemas through the same edge. E.g.\n\n    graph :friends, [Person, Pet], edge: Friends, on_replace: :delete\n\nUse the fields on the schema to identify which schema to use. If any of the fields in the list\nexists then it will use the key as the module.\n\n    graph :friends, %{\n        Person => [:first_name],\n        Pet => [:name]\n      },\n      edge: Friends\n  \n\nThe available fields are:\n\n  * `cardinality` - The association cardinality, always `:many`\n  * `field` - The name of the association field on the schema\n  * `owner` - The schema where the association was defined\n  * `owner_key` - The key on the `owner` schema used for the association\n  * `related` - The schema that is associated\n  * `related_key` - The key on the `related` schema used for the association\n  * `queryables` - The real query to use for querying association\n  * `defaults` - default fields used when building the association\n  * `relationship` - The relationship to the specified schema, default `:parent`\n  * `on_replace` - The action taken on associations when schema is replaced","ref":"ArangoXEcto.Association.Graph.html","title":"ArangoXEcto.Association.Graph","type":"module"},{"doc":"","ref":"ArangoXEcto.Association.Graph.html#assoc_query/2","title":"ArangoXEcto.Association.Graph.assoc_query/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Association.Graph.html#t:t/0","title":"ArangoXEcto.Association.Graph.t/0","type":"type"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"This is a big release with a lot of changes and has been over a year in the\nmaking. I am so glad it is finally finished :)\n\nThere are many breaking changes and the API to lots of functions have changed.\nPlease make sure to check the changes in the docs.\n\nSome of the commits were big and does not follow good commit practices. I know\nsome might say \"why didn't you do this progressively?\" and yes that is correct\nit should have been done that way. However, I have been very busy over the past\nyear which is why it took so long to get this version out of the door. I also\ngot a new computer and moved house so sometimes rules go out of the window.\n\nStatic mode is now the default as it is more production ready. Dynamic mode can\nnow be set by setting `:static` to false in your config.\n\n_This release now requires Ecto 3.12 and Arangox 0.6.0 as a minimum._","ref":"changelog.html#2-0-0","title":"2.0.0 - Changelog","type":"extras"},{"doc":"#### Migrations improvements\n\nComplete rework of migrations functionality to support full migrations including\nusing ArangoDB schemas for database level validation. (#73)\n\n- All database actions are done through migrations\n  - This includes create, modify, rename and remove functions for Collections,\n    Indexes, Views and Analyzers.\n- Add JSON schema generation from migrations for database level validation\n- Migrations are run in a transaction\n- Commands updated to not conflict with other database adapters\n  - `mix ecto.migrate.arango` renamed to `mix arango.migrate`, same for other\n    commands\n  - Use the `execute_ddl/3` function in the adapter to support other\n    database adapters\n- Type of collection is set in options rather then a parameter\n- Migrations can be run as part of a supervision tree (for when doing\n  releases)\n- Option to cast migration version to integer (default false)\n- Supports add, remove, modify, add_if_not_exists and remove_if_exists\n  migration commands\n- Support embedded fields and an array of embedded fields\n- Supports reversing migrations, hence can use just the `change/0` function in\n  migrations rather than needing to use both `up/0` and `down/0`\n\n#### Graphing relations improvements\n\nComplete rework of graph relationships (#39).\n\nThis replaces the many_to_many usage with a purpose build Ecto association.\nThe primary difference is that this allows for multiple different\nschemas to be related, i.e. it is polymorphic. I.e. A User could have\na field called `:my_content` that could have either Posts or Comments\nassociated. This lines up better with how ArangoDB works.\n\nTo get this working it required a few tricks and in some cases it\nrequired a copy and reimplementation of some of Ecto's functions. For\nexample, a new `ArangoXEcto.Changeset` module was created and must be\nimported to use the `cast_graph/3` and `put_graph/4`. These two\nfunctions are essentially the same as the `cast_assoc/3` and\n`put_assoc/4` functions that Ecto provide, except it will allow for\nmultiple types to be applied and supports the mapping by the fields in\nattributes.\n\nIn addition to this the EdgeMany association was created so that it supports\nmultiple schemas being passed. This is so that edges can have multiple schemas\nin the from and to, which lines up more closely with how ArangoDB functions.\n\n- `ArangoXEcto.create_edge/4` supports a list of edge modules\n- Ensure that the from and to schemas exist in an edge definition\n- Add `ArangoXEcto.Query.graph/5` macro to perform graph queries through Ecto\n  queries\n- Added the `ArangoXEcto.Changeset.cast_graph/3` and\n  `ArangoXEcto.Changeset.put_graph/4` functions\n- Add the `ArangoXEcto.Association.EdgeMany` and `ArangoXEcto.Association.Graph`\n  association definitions\n- Add `ArangoXEcto.preload/4` function to support preloading graph associations\n- Removed the `one_outgoing/3`, `one_incoming/3` and `build_foreign_key/1` functions\n  since they were not used and essentially just reimplement Ecto functions.\n\n#### Other Enhancements\n\n- Enhance transaction functionality and add rollback ability\n- Added the ability to stream queries using ArangoDB's cursors\n- Improve logging and telemetry (Based on the `ecto_sql` implementation)\n  - Add logging and telemetry handlers to query, insert, update and delete\n    functions\n  - Supports transactions and AQL queries\n- Add caching of queries in Ecto\n- Improve connection handling with database between Ecto and adapter\n- View links validation is moved to after compile to improve validation checks\n- Static mode is now the default\n  - Added checks for functions that can only be run in static/dynamic mode\n- Add support for typecasting in Ecto queries through `Ecto.Query.API.type/2`\n- Add `ArangoXEcto.Sandbox` for integration testing (similar to the `ecto_sql`\n  implementation)\n- Supports running alongside other database adapters, such as `ecto_sql`\n  - Added dummy implementation of functions that `ecto_sql` uses\n- Add support for fragments (and splicing) in queries that are then converted\n  into AQL\n- Removed the already deprecated `ArangoXEcto.raw_to_struct/2` function\n- Update to support Ecto 3.12 (now minimum required version)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Fix typespec for `t:mod/0` in `ArangoXEcto`\n- ArangoSearch type is forced to be \"arangosearch\"\n- Rename `is_edge?/1`, `is_document?/1` and `is_view?/1` to remove the is\\_\n- Type checking functions (the ones mentioned above) will ensure that the module\n  being checked is loaded before checking\n- Fix type validation on analyzer creation\n- Resolve `:database` option passed to `ArangoXEcto.query/4` not working\n- Allow nil to be passed to type loaders in adapter\n- Make prefix option work universally across all database functions\n- Update mix formatter configuration\n- Clean up `ArangoXEcto.load/2` function\n- Move `:fullCount` property to the adapter so that it is used for all\n  connections\n- Add check so that mix tasks are only run on ArangoXEcto repos\n- Remove `ArangoXEcto.Utils` module\n- Fix issue where collection indexes were not being created in dynamic mode\n- Removed some unused clauses in `ArangoXEcto.Types.GeoJSON`\n- Add missing tests for mix tasks (excluded by default)\n- Migrate formatting to preferred formatting for elixir 1.18 (using mix format\n  --migrate)\n- Various cleanups of the codebase\n- Various test fixes","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Add basic guides\n- Significant improvement to documentation and typespec coverage","ref":"changelog.html#doc-changes","title":"Doc changes - Changelog","type":"extras"},{"doc":"- Update dependencies\n- Improve test coverage to at least 80%\n  - This was a big rework to include more tests and really ensuring\n    functionality works\n- Update doctor configuration\n- Ignore tmp directory in git repo for when tests are run\n- Update CI pipeline\n  - Update elixir and otp versions\n  - Make arangodb version explicit to 3.11\n- Add coveralls for test coverage checking","ref":"changelog.html#repo-changes","title":"Repo changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-3-1","title":"1.3.1 - Changelog","type":"extras"},{"doc":"- Fix for migrations failing due to wrong argument being passed (thanks @ilourt)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"_This release now requires Ecto 3.9 and Arangox 0.5.5 as a minimum._","ref":"changelog.html#1-3-0","title":"1.3.0 - Changelog","type":"extras"},{"doc":"- Add Arango Search capability (#60)\n  - Added `ArangoXEcto.View` definition\n  - Ecto querying of views\n  - Add search extensions for Ecto Querying in `ArangoXEcto.Query` (_BREAKING CHANGE_ requires Ecto 3.9)\n  - Adds analyzer definition for management in code\n  - Add view and analyzer migration capability to `ArangoXEcto.Migration`\n- Allow `ArangoXEcto.load/2` to load from multiple different schmeas\n- Add ability to sort using fragments\n- Add multi-tenancy functionality (#25) using Ecto `:prefix` option\n  - Adds the ability to pass the :prefix option for CRUD operations\n  - Adds ArangoXEcto.get_prefix_database/2 to get the database name\n  - Adds querying prefix option (_BREAKING CHANGE_ requires Arangox >= 0.5.5)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Fix write operation on AQL query not working for delete\n- Fix static mode tests using a static mode repo\n- Add missing query tests #61 (thanks @mpoeter)\n- Adapt AQL query builder for Ecto 3.10 which introduced LimitExpr #61 (thanks @mpoeter)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Add Arango Search related documentation","ref":"changelog.html#doc-changes","title":"Doc changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-2-1","title":"1.2.1 - Changelog","type":"extras"},{"doc":"- Fix for Repo.exists?/1 function not working correctly (thanks @ilourt)\n- Resolved #55 - ability to use custom types (e.g. Enum) (thanks @mpoeter)\n- Fix for when dumping lists\n- Fix for #58 - preloading edge collection's `_to` and `_from` to reference `_id`","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Added clarifying documentation (+ test) for write operations of `ArangoXEcto.aql_query/4` (thanks @mrsisk)","ref":"changelog.html#doc-changes","title":"Doc changes - Changelog","type":"extras"},{"doc":"- Update test versions","ref":"changelog.html#repo-changes","title":"Repo Changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-2-0","title":"1.2.0 - Changelog","type":"extras"},{"doc":"- Added `ArangoXEcto.load/2` which will load deep schemas (so associations will work)\n- Added ability to use Ecto's count and aggregate functionality\n- Added ability to use Ecto's datetime helper functions (e.g. `:datetime_add`)\n- Separate `aql_query/4` with `aql_query!/4` raising an error on fail","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Fix for #49 - unique_constraint not recognized (thanks @jamilabreu)\n- Prevent warnings for `__id__` field on creation\n- Fix for when an update is applied to a non-existent document `:stale` is raised\n- Fix for dates not being queryable using Ecto\n- Prevent error thrown when non `_key` filter is supplied to update, resolves #48","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Added information about one-to-many relationships in the README","ref":"changelog.html#other-doc-changes","title":"Other Doc Changes - Changelog","type":"extras"},{"doc":"- `ArangoXEcto.raw_to_struct/2` is deprecated in favor of `ArangoXEcto.load/2` add will be removed in a future version","ref":"changelog.html#deprecations","title":"Deprecations - Changelog","type":"extras"},{"doc":"- Include a config file for local git hooks","ref":"changelog.html#repo-changes","title":"Repo Changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-1-1","title":"1.1.1 - Changelog","type":"extras"},{"doc":"- Fixed `indexes/1` and `options/1` for use in dynamic edge collection creation (closes #46)\n- Added logging messages for errors in `indexes/1` and `options/1`\n- Add `:name` option for index creation. Fixes issue with phoenix unique constraints\n- Fixes on_conflict issue (arango does not support for specific fields, only will replace all no matter which replace option is passed)\n- Various tests for changes","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-1-0","title":"1.1.0 - Changelog","type":"extras"},{"doc":"- Update `credo` and `git_hooks` dev & test dependencies\n- Add ability to pass collection creation options for migrations\n- Add tests for migrations\n- Use migration module for all collection creation\n- Use underscored module name for repo dir to allow for different repo director name (thanks @hengestone)\n- Big cleanup of README\n\n#### Dynamic Collection Creation Options\n\n- Add `options/1` macro for dynamic collection creation options\n- Add `indexes/1` macro for dynamic collection creation indexes\n- Add tests for creation options and indexes","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Clean up some syntax in line with new credo requirements\n- Add `doctor` config to remove false positive doc requirements with macros\n- Add some missing function docs\n- Various refactoring to clean up code\n- Fix to remove `Map.reject/2` for support of below Elixir 1.13\n- Pass the connection to collection creation to ensure correct repo is used\n- Fix a race condition in testing caused by same named collections for different tests\n- Require `DB_ENDPOINT` env var for tests","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Add Elixir 1.13 to testing matrix\n- Use `erlef/setup-beam` for ci and deploy GH actions (thanks @kianmeng)\n- Add develop branch to ci GH action","ref":"changelog.html#repo-changes","title":"Repo Changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0","title":"1.0.0 - Changelog","type":"extras"},{"doc":"- Added api_query/3 function for direct Arango API queries\n- Added lots of test cases for Ecto base functionality\n- Added missing test cases that should have been previously included\n- Adds ability to do subqueries in AQL queries\n- Improve migration template\n- Rework migration command\n- Removed `create_edge/6` function\n\n#### Graph Relationships\n\n- Complete rework of Edge relationships to conform to Ecto standards\n  E.g.\n\n```elixir\ndefmodule MyProject.User do\n  use ArangoXEcto.Schema\n  schema \"users\" do\n    field :name, :string\n    # Will use the automatically generated edge\n    outgoing :posts, MyProject.Post\n    # Will use the UserPosts edge\n    outgoing :posts, MyProject.Post, edge: MyProject.UserPosts\n    # Creates foreign key\n    one_outgoing :main_post, MyProject.Post\n  end\nend\n\ndefmodule MyProject.Post do\n  use ArangoXEcto.Schema\n  schema \"posts\" do\n    field :title, :string\n    # Will use the automatically generated edge\n    incoming :users, MyProject.User\n    # Will use the UserPosts edge\n    incoming :users, MyProject.User, edge: MyProject.UserPosts\n    # Stores foreign key id in schema\n    one_incoming :user, MyProject.User\n  end\nend\n```\n\n- Adds one-to-one relation (not a graph relation, stores as an ID in field)\n- Add graph tests\n- Automatically insert `_id` field into schemas when interacting with edges, saved in `:__id__` attributed\n- Documentation for graph functionality","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Added Geo functionality with GeoJSON support\n- Added `ArangoXEcto.Types.GeoJSON` type\n- Added `:geo` package dependency\n- Added Geo related helper functions to manage Geo data in `ArangoXEcto.GeoData`\n- Documentation for Geo functionality","ref":"changelog.html#geo-functionality","title":"Geo Functionality - Changelog","type":"extras"},{"doc":"- Added the ability to dynamically create collections or require migrations\n- Added `:static` boolean config option (default if false)\n- Added tests for static/dynamic functionality","ref":"changelog.html#static-or-dynamic-collection-management","title":"Static or Dynamic Collection Management - Changelog","type":"extras"},{"doc":"- Fixed typespec for aql_query/4 (PR #27, thanks @bodbdigr)\n- Fixed missed ArangoX version api change in `Mix.ArangoXEcto`\n- Fixed incorrect length return on writing queries (i.e. updates and deletes)\n- Fixed Arango storing integer as float\n- Complete rewrite of setup command to fix incorrect setup processes and messages\n- Fixed a race condition in edge creation\n- Added missing specs\n- Some other minor fixes that I cbf adding because nobody had experienced it","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- Updated ISSUE templates","ref":"changelog.html#repo-changes","title":"Repo Changes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#0-7-2","title":"0.7.2 - Changelog","type":"extras"},{"doc":"- Updates deps to latest\n- Update to ecto 3.6","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#0-7-1","title":"0.7.1 - Changelog","type":"extras"},{"doc":"- Fix warnings in typecasting\n- Fix ecto version requirement issue with phoenix","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#0-7-0","title":"0.7.0 - Changelog","type":"extras"},{"doc":"- Add support for database transactions\n- Decimal support in database\n- Add schema_type/1 for non error thrown version\n- Additional testing","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"- Fixes duplicate migration entries\n- Stops running migrations again\n- Additional catch in core functions\n- Fix ID key conversion","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"- (#12) Ecto 3.5 version bump\n- Format fixes","ref":"changelog.html#0-6-9","title":"0.6.9 - Changelog","type":"extras"},{"doc":"- (#10) Fixed mix task conflict with ecto\n- (#8) Fixes some incorrect documentation\n- Bump versions of dependencies","ref":"changelog.html#0-6-8","title":"0.6.8 - Changelog","type":"extras"},{"doc":"- Various documentation fixes (thanks @kianmeng)\n- Allows ecto queries by `_id` as well as `_key`\n- Fixed a bug when passing config in migrations ignored anything except the `:endpoints` option\n- Fixed a bug when passing a struct to `edge_module/3`","ref":"changelog.html#0-6-7","title":"0.6.7 - Changelog","type":"extras"},{"doc":"- Added this file\n- Better organised GitHub Actions\n- Allow collection_exists?/3 to use a connection as well as repo\n- Added is_edge?/1, is_document?/1 and schema_type!/1\n  Checking if is a module is an edge\n\n```elixir\niex> ArangoXEcto.is_edge?(MyApp.RealEdge)\ntrue\n```\n\nChecking if is a module is a document collection\n\n```elixir\niex> ArangoXEcto.is_document?(MyApp.Users)\ntrue\n```\n\nChecking a schema type\n\n```elixir\niex> ArangoXEcto.schema_type!(MyApp.RealEdge)\n:edge\n```\n\n- Adde","ref":"changelog.html#0-6-6","title":"0.6.6 - Changelog","type":"extras"},{"doc":"# Getting Started\n\nThis is an introduction on how to use the [ArangoXEcto](https://github.com/TomGrozev/arangox_ecto)\npackage, an ArangoDB adapter for Ecto. This adapts the various facets of the\nEcto implementation to work for graph databases.\n\nThis guide will cover the basics of using ArangoXEcto, including creating,\nreading, updating and deleting records.\n\n> #### Note {: .error}\n>\n> You must already have an ArangoDB database setup","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"Add the following line to your mix dependencies to get started.\n\n```elixir\n{:arangox_ecto, \"~> 2.0\"}\n```\n\nTo install these dependencies, we will run this command:\n\n```shell\nmix deps.get\n```\n\nThis will also install the `Arangox` dependency which is used for the\ncommunication with the Arango database.\n\nTo connect to the database you need to specify the config values like so:\n\n```elixir\nconfig :my_app, MyApp.Repo,\n  database: \"my_db\",\n  endpoints: \"http://1.2.3.4:8529\",\n  username: \"root\",\n  password: \"root\",\n```\n\nOnly `database` and `endpoints` are required but there are other available\noptions can be found in the [Arangox docs](https://hexdocs.pm/arangox/Arangox.html#start_link/1).\n\nThe database should be setup using the following.\n\n```shell\nmix ecto.create\n```","ref":"getting-started.html#adding-arangoxecto-to-your-project","title":"Adding ArangoXEcto to your project - Getting Started","type":"extras"},{"doc":"In addition to default config variables, the `static` boolean config option can\nbe passed to force disable use of migrations. By default the value is `true`\nand hence the by default it is in static mode and migrations are required.\n\nIf in dynamic mode collections that don't exist will be created on insert and\non query no error will be raised. If set to `true` (default), any collections\nthat do not exist on insert or query will result in an error being raised.\n\nWhether dynamic or static is chosen depends on the database design of the\nproject. For a production setup where lots of control is required, it is\nrecommended to have `static` set to `true`, which is the default.\n\n#### Why even have dynamic mode?\n\nDynamic mode can be easier for development and testing (especially when you\ndon't know what the database structure will look like) then after that point\nstatic mode can be turned on for production.","ref":"getting-started.html#static-or-dynamic","title":"Static or Dynamic - Getting Started","type":"extras"},{"doc":"To use the adapter in your repo, make sure your repo uses the `ArangoXEcto.Adapter` module for the adapter.\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo,\n    otp_app: :my_app,\n    adapter: ArangoXEcto.Adapter\nend\n```","ref":"getting-started.html#repo-setup","title":"Repo Setup - Getting Started","type":"extras"},{"doc":"Since ArangoDB uses a slightly different id system, your schema must use the\n`ArangoXEcto.Schema` instead of `Ecto.Schema`.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n    use ArangoXEcto.Schema\n    import Ecto.Changeset\n\n    schema \"users\" do\n      field :first_name, :string\n      field :last_name, :string\n\n      timestamps()\n    end\n\n    @doc false\n    def changeset(app, attrs) do\n      app\n      |> cast(attrs, [:first_name, :last_name])\n      |> validate_required([:first_name, :last_name])\n    end\nend\n```","ref":"getting-started.html#schema-setup","title":"Schema Setup - Getting Started","type":"extras"},{"doc":"The `ArangoXEcto.Schema.options/1` and `ArangoXEcto.Schema.indexes/1` options\ncan be optionally called to set options and indexes to be created in dynamic\nmode. These options will have no effect in static mode.\n\nFor example, if you wanted to use a UUID as the key type and create an index\non the email the following can be used.\n\n```elixir\ndefmodule MyApp.Accounts.User do\n    use ArangoXEcto.Schema\n    import Ecto.Changeset\n\n    options [\n      keyOptions: %{type: :uuid}\n    ]\n\n    indexes [\n      [fields: [:email]]\n    ]\n\n    schema \"users\" do\n      field :email, :string\n\n      timestamps()\n    end\n\n    @doc false\n    def changeset(app, attrs) do\n      app\n      |> cast(attrs, [:first_name, :last_name])\n      |> validate_required([:first_name, :last_name])\n    end\nend\n```\n\nPlease refer to [the Schema documentation](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Schema.html) for more information on the available options.","ref":"getting-started.html#dynamic-mode-options-and-indexes","title":"Dynamic Mode options and indexes - Getting Started","type":"extras"},{"doc":"> #### Note {: .info}\n>\n> **Using migrations is only required in static mode (the default)**\n> If in dynamic mode, the adapter will automatically create collections if they\n> don't already exist.\n\nRefer to [ArangoXEcto.Migration](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Migration.html)\nfor more information on usage.\n\n```elixir\ndefmodule MyApp.Repo.Migrations.CreateUsers do\n    use ArangoXEcto.Migration\n\n    def change do\n      create collection(:users) do\n        add :first_name, :string, comment: \"first_name column\"\n        add :last_name, :string\n\n        timestamps()\n      end\n\n      create index(:users, [:email])\n    end\nend\n```","ref":"getting-started.html#migrations","title":"Migrations - Getting Started","type":"extras"},{"doc":"A lot of the time it is far more efficient to just run a raw AQL query, there's\na function for that.\n\n```elixir\nArangoXEcto.aql_query(\n    Repo,\n    \"FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var\",\n    fname: \"John\",\n    lname: \"Smith\"\n  )\n```\n\nThis query will return a result such as:\n\n```elixir\n{:ok,\n    [\n        %{\n          \"_id\" => \"users/12345\",\n          \"_key\" => \"12345\",\n          \"_rev\" => \"_bHZ8PAK---\",\n          \"first_name\" => \"John\",\n          \"last_name\" => \"Smith\"\n        }\n    ]}\n```\n\nThis is awesome functionality, but a lot of the time we will want to resemble a\nspecific struct. This is actually quite easy with the help of the\n`ArangoXEcto.load/2` function. The same query above could be extended to also\nconvert the output:\n\n```elixir\nArangoXEcto.aql_query(\n    Repo,\n    \"FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var\",\n    fname: \"John\",\n    lname: \"Smith\"\n  )\n  |> case do\n      {:ok, results} ->\n        ArangoXEcto.load(results, User)\n\n      {:error, _reason} -> []\n  end\n```\n\nThis will return something like:\n\n```elixir\n[\n    %User{\n      id: \"12345\",\n      first_name: \"John\",\n      last_name: \"Smith\"\n    }\n]\n```\n\nThis is clearly a much better representation of the result and can be used in\nfurther Ecto methods.","ref":"getting-started.html#raw-aql-queries","title":"Raw AQL queries - Getting Started","type":"extras"},{"doc":"You can find out how to write graph relations in the [Graphing\nGuide](https://hexdocs.pm/arangox_ecto/graphing.html)","ref":"getting-started.html#graph-relations","title":"Graph relations - Getting Started","type":"extras"},{"doc":"To learn about using the schema functions for representing graph relationships and examples, read the docs at\n[ArangoXEcto.Schema](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Schema.html).\n\nTo read more about Edge Schemas and how to extend edge schemas to add additional fields, read the docs on\n[ArangoXEcto.Edge](https://hexdocs.pm/arangox_ecto/ArangoXEcto.Edge.html).\n\nTo learn how to use the helper functions (as well as other useful methods) check out the\n[full documentation](https://hexdocs.pm/arangox_ecto/ArangoXEcto.html).\n\nFor more examples and full documentation, please refer to the [Documentation](https://hexdocs.pm/arangox_ecto).","ref":"getting-started.html#more-information","title":"More information - Getting Started","type":"extras"},{"doc":"# Testing with ArangoXEcto","ref":"testing-with-arangoxecto.html","title":"Testing with ArangoXEcto","type":"extras"},{"doc":"# Graphing\n\nEcto traditionally is made for relational databases and hence relations between schemas is\nrepresented in such a way. A graph relation (edges) can be thought of as many-to-many relations in\nthe relational database world. When it comes to representing graph relations in Ecto, that is\nexactly how it works in ArangoXEcto.\n\nTo use a graph relation you define an `outgoing` and an `incoming` relation on the two schemas,\nfor example the following will create a graph relationship between `User -> UserPost -> Post`.\nThe `UserPost` edge module will be created automatically.\n\n    defmodule MyApp.User do\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n\n        outgoing :posts, MyApp.Post\n      end\n    end\n\n    defmodule MyApp.Post do\n      schema \"posts\" do\n        field :title, :string\n\n        incoming :users, MyApp.User\n      end\n    end\n\nBehind the scenes this works very similar to a `many_to_many` relationships but has a few key\ndifferences. As you may know, in a graph an edge can have multiple different node types that it\nconnects to/from. ArangoXEcto uses a special relation type that is based on the principal of\nmany-to-many but modified to allow such relationships.\n\nFor example, you may have a `User` schema that you want to connect to `Post` and `Comment`. You\nmay not want to have two seperate relationships (e.g. `:posts` and `:comments`) and want to\ncombine it all under one (e.g. `:content`). This is what the ArangoXEcto adapter enables. You can\nread more about this type of relationship below.","ref":"graphing.html","title":"Graphing","type":"extras"},{"doc":"When in dynamic mode the ArangoXEcto adapter dynamically creates and manages edge collections.\nEach edge collection will be created as an Ecto schema when they are first used. This means that\nyou don't need to create edge collections manually. When in static mode you need to define edges\nmanually.\n\nThe edge module will be created under the closest common parent module of the passed modules\nplus the `Edges` alias. The order of the edge name will always be alphabetical to prevent\nduplicate edges. For example, if the modules were `MyApp.Apple.User` and `MyApp.Apple.Banana.Post`\nthen the edge would be created at `MyApp.Apple.Edges.PostUser`. This assumes that the edge\ncollection name was generated and not explicitly defined, if it was `PostUser` would be replaced\nwith the camel case of that collection name (i.e. `address_people` would be `AddressPeople`).","ref":"graphing.html#edge-modules","title":"Edge Modules - Graphing","type":"extras"},{"doc":"Creating a graph relation that has multiple schemas through the same edge is possible in\nArangoXEcto. For example, take the following.\n\n```mermaid\ngraph TD\nUser --> edge(UsersContent)\nedge(UsersContent) --> Post\nedge(UsersContent) --> Comment\n```\n\nUsers can have posts or comments through the same edge. This works because edges can have multiple\nfrom and to schemas. For example if we were to define the `UsersContent` edge it would look like\nthis:\n\n    defmodule MyApp.Edges.UsersContent do\n      use ArangoXEcto.Edge,\n          from: User,\n          to: [Post, Comment]\n\n      schema \"users_content\" do\n        edge_fields()\n      end\n    end\n\nWe can then define our outgoing definition for the User schema like the following.\n\n    defmodule MyApp.User do\n      schema \"users\" do\n        field :first_name, :string\n        field :last_name, :string\n\n        outgoing(\n          :my_content,\n          %{\n            Post => &Post.changeset/2,\n            Comment => &Comment.changeset/2\n          },\n          edge: ArangoXEcto.Edges.UsersContents\n        )\n      end\n    end\n\nThe definition of outgoing here is slightly different from above. Instead of specifying a single\nschema module we specify a map where the keys are schema modules and the values are a list of\nfield names to check if it exists in a result to match it with the module. This is an OR\ncondition, so if you had `[:a, :b]` that would mean if a result had `:a` it would match or if it\nhad `:b` it would match. If a result had both it would match.\n\nNotice that the edge here was explicitly defined. This is not necessary if you aren't doing\nanything custom on the edge and not including this would just generate an edge like the one\nabove. But remember a migration is still required when in static mode.\n\nYou can also do the same with the `ArangoXEcto.Schema.incoming/3` function.\n\n#### Preloading\n\nDue to how graph relations work the regular `c:Ecto.Repo.preload/3` function will only return maps\ninstead of structs. If you want to preload the structs you can use the `ArangoXEcto.preload/4`\nfunction.\n\n    iex> Repo.all(User) |> ArangoXEcto.preload(Repo, [:my_content])\n    [%User{id: 123, my_content: [%Post{...}, %Comment{...}]}]","ref":"graphing.html#multiple-edge-schemas","title":"Multiple edge schemas - Graphing","type":"extras"},{"doc":"# Searching with ArangoSearch\n\nArangoDB uses ArangoSearch to index collections to improve the searchability of them. With this\nyou can do things like have multiple collections in one \"view\". A view is a concept that is\nessentially take one or more collections and indexes it using whatever \"analyzer(s)\" you choose to\napply.\n\nTo query a View you just use it as if you were using any other collection schema. It will return\nthe results the same as if you were querying that schema.\n\nTo create a View it is very similar to how you create a collection schema. The following will\ncreate a view that has primary sorts on :created_at and :name, it will store the :email and the\n:first_name & :last_name fields. Most importantly it will create a link to the `MyApp.Users`\nschema and set the analyzer for the `:name` field to `:text_en`. Some options can be set also.\n\n    defmodule MyApp.UserSearch do\n      use ArangoXEcto.View\n\n      alias ArangoXEcto.View.Link\n\n      view \"user_search\" do\n        primary_sort :created_at, :desc\n        primary_sort :name\n\n        store_value [:email], :lz4\n        store_value [:first_name, :last_name], :none\n\n        link MyApp.Users, %Link{\n          includeAllFields: true,\n          fields: %{\n            name: %Link{\n              analyzers: [:text_en]\n            }\n          }\n        }\n\n        options  [\n          primarySortCompression: :lz4\n        ]\n      end\n    end\n\nQuerying is done exactly the same as a normal schema except the result will not be a struct.\n\n    iex> Repo.all(MyApp.UsersView)\n    [%{first_name: \"John\", last_name: \"Smith\"}, _]\n\nYou can find out more info in the `ArangoXEcto.View` module.\n\nTo query using the search function you can use the `ArangoXEcto.Query.search/3` and\n`ArangoXEcto.Query.or_search/3` functions. For example, if we wanted to search using the text\nanalyzer we could do the following.\n\n    import ArangoXEcto.Query\n\n    from(UsersView)\n    |> search([uv], fragment(\"ANALYZER(? == ?, \\\\\"text_en\\\\\")\", uv.first_name, \"John\"))\n    |> Repo.all()","ref":"arango-search.html","title":"Searching with ArangoSearch","type":"extras"}],"producer":{"name":"ex_doc","version":"0.36.1"},"proglang":"elixir"}